Package: BioQC
Type: Package
Title: Detect tissue heterogeneity in expression profiles with gene
        sets
Version: 1.40.0
Date: 2021-08-18
Authors@R: c(
    person("Jitao David", "Zhang", email="jitao_david.zhang@roche.com", role = c("cre", "aut")), 
    person("Laura", "Badi", role="aut"),
    person("Gregor", "Sturm", email="mail@gregor-sturm.de", role=c("aut")),
    person("Roland", "Ambs", role="aut"),
    person("Iakov", "Davydov", role="aut"))
Description: BioQC performs quality control of high-throughput expression data
    based on tissue gene signatures. It can detect tissue heterogeneity in gene
    expression data. The core algorithm is a Wilcoxon-Mann-Whitney test that is
    optimised for high performance.
biocViews: GeneExpression,QualityControl,StatisticalMethod,
        GeneSetEnrichment
Depends: R (>= 3.5.0), Biobase
Imports: edgeR, Rcpp, methods, stats, utils
Collate: AllClasses.R AllMethods.R as.GmtList.R GmtList-funcs.R
        readSignedGmt.R uniqGenesetsByNamespace.R appendGmtList.R
        utils.R entropy.R gini.R readGmt.R matchGenes.R wmwTest.R
        readCurrentSignatures.R prettySigNames.R leadingEdge.R
Suggests: testthat, knitr, rmarkdown, lattice, latticeExtra,
        rbenchmark, gplots, gridExtra, org.Hs.eg.db, hgu133plus2.db,
        ggplot2, reshape2, plyr, ineq, covr, limma, RColorBrewer
LinkingTo: Rcpp
VignetteBuilder: knitr
License: GPL (>=3) + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.1.1
URL: https://accio.github.io/BioQC
BugReports: https://accio.github.io/BioQC/issues
git_url: https://git.bioconductor.org/packages/BioQC
git_branch: RELEASE_3_23
git_last_commit: b7d663f
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: yes
Packaged: 2026-04-28 20:56:42 UTC; biocbuild
Author: Jitao David Zhang [cre, aut],
  Laura Badi [aut],
  Gregor Sturm [aut],
  Roland Ambs [aut],
  Iakov Davydov [aut]
Maintainer: Jitao David Zhang <jitao_david.zhang@roche.com>

# Generated by roxygen2: do not edit by hand

S3method("[",GmtList)
S3method("[[",GmtList)
export("gsNamespace<-")
export(GmtList)
export(SignedGenesets)
export(SignedIndexList)
export(absLog10p)
export(appendGmtList)
export(as.GmtList)
export(entropy)
export(entropyDiversity)
export(entropySpecificity)
export(filterBySize)
export(filterPmat)
export(getLeadingEdgeIndexFromMatrix)
export(getLeadingEdgeIndexFromVector)
export(gini)
export(gmtlist2signedGenesets)
export(gsDesc)
export(gsGeneCount)
export(gsGenes)
export(gsName)
export(gsNamespace)
export(gsSize)
export(hasNamespace)
export(isValidBaseIndexList)
export(isValidGmtList)
export(isValidIndexList)
export(isValidSignedGenesets)
export(isValidSignedIndexList)
export(prettySigNames)
export(readCurrentSignatures)
export(readGmt)
export(readSignedGmt)
export(sampleSpecialization)
export(setDescAsNamespace)
export(setNamespace)
export(simplifyMatrix)
export(uniqGenesetsByNamespace)
export(wmwLeadingEdge)
export(wmwTestInR)
exportClasses(GmtList)
exportClasses(IndexList)
exportClasses(SignedGenesets)
exportClasses(SignedIndexList)
exportMethods("offset<-")
exportMethods(IndexList)
exportMethods(SignedIndexList)
exportMethods(matchGenes)
exportMethods(offset)
exportMethods(show)
exportMethods(wmwTest)
import(Rcpp)
importClassesFrom(Biobase,eSet)
importClassesFrom(edgeR,DGEList)
importFrom(Biobase,exprs)
importFrom(Biobase,fData)
importFrom(Biobase,featureNames)
importFrom(Rcpp,sourceCpp)
importFrom(methods,is)
importFrom(methods,new)
importFrom(stats,median)
importFrom(stats,pt)
importFrom(stats,wilcox.test)
importFrom(utils,head)
useDynLib(BioQC, .registration=TRUE, .fixes="C_")

[![R-CMD-check-bioc](https://github.com/Accio/BioQC/workflows/R-CMD-check-bioc/badge.svg)](https://github.com/Accio/BioQC/actions)
[![codecov](https://codecov.io/gh/Accio/BioQC/branch/master/graph/badge.svg)](https://codecov.io/gh/Accio/BioQC)
[![install with bioconda](https://img.shields.io/badge/install%20with-bioconda-brightgreen.svg?style=flat)](http://bioconda.github.io/recipes/bioconductor-bioqc/README.html)
[![license](https://img.shields.io/badge/license-GPLv3-green.svg)](https://github.com/accio/BioQC/blob/master/LICENSE.md)
[![docs](https://img.shields.io/badge/docs-pkgdown-blue.svg)](https://accio.github.io/BioQC)

BioQC is a is a R/[Bioconductor](https://bioconductor.org/packages/release/bioc/html/BioQC.html) package to detect tissue heterogeneity in gene expression data. 
Tissue heterogeneity is a consequence of unintended profiling of cells of other origins than the tissue of interest and can
have both technical (*e.g.* imperfect disection) or biological (*e.g.* immune infiltration) reasons. 

We [demonstrated](https://www.biorxiv.org/content/10.1101/2020.12.02.407809v1) that tissue heterogeneity is prevalent
in 5-15% of all gene expression studies. Ignoring tissue heterogeneity reduces statistical power of data analysis and can, in the worst case, invalidate the conclusions of a study.
Therefore, we propose applying BioQC as a routine step in every gene-expression analysis pipeline. 

The BioQC method is described in 

> Zhang, Jitao David, Klas Hatje, Gregor Sturm, Clemens Broger, Martin Ebeling, Martine Burtin, Fabiola Terzi, Silvia Ines Pomposiello, and Laura Badi.
> “Detect Tissue Heterogeneity in Gene Expression Data with BioQC.” BMC Genomics 18 (2017): 277. [doi:10.1186/s12864-017-3661-2](https://doi.org/10.1186/s12864-017-3661-2).


## Basic Usage

BioQC implements a computationally efficient Wilcoxon-Mann-Whitney test for testing
for enrichment of tissue signatures. A database of 150 tissue signatures derived
from large-scale transcriptomics studies is shipped with the BioQC package. 

To apply BioQC to a `genes x samples` gene expression matrix, run:

```R
library(BioQC)

# load the tissue signatures
gmtFile <- system.file("extdata/exp.tissuemark.affy.roche.symbols.gmt",
                        package="BioQC")
gmt <- readGmt(gmtFile)

# perform BioQC enrichment test on a gene expression matrix
bioqc_res = wmwTest(expr_mat, gmt)
bioqc_scores = absLog10p(bioqc_res)
```

The following figure shows the BioQC scores from the [kidney example](https://accio.github.io/BioQC/bioqc.html) 
visualized as heatmap. We note that in samples 23-25 *adipose* and *pancreas* signatures
have been detected, hinting at a containation with those tissues. For this dataset, 
we could validate the contamination with qPCR. 

![example heatmap](man/figures/kidney_heatmap.png)

For a more detailed example explaining how to use other data structures or custom signatures
see
 * [Introduction to BioQC](https://accio.github.io/BioQC/articles/bioqc-introduction.html).
 * [Applying BioQC to a real-world kidney
   dataset](https://accio.github.io/BioQC/articles/BioQC.html).

For advanced usages, check out:
 * [Single sample gene set enrichment analysis with BioQC](https://accio.github.io/BioQC/articles/bioqc-signedGenesets.html)

For benchmarks and details about the algorithm, see:
 * [BioQC-benchmark: Testing Efficiency, Sensitivity and Specificity of BioQC on
   simulated and real-world data](https://accio.github.io/BioQC/articles/bioqc-simulation.html)
 * [Comparing the Wilcoxon-Mann-Whitney to alternative statistical tests](https://accio.github.io/BioQC/articles/bioqc-wmw-test-performance.html)
 * [The BioQC algorithm: speeding up the WMW-test](https://accio.github.io/BioQC/articles/bioqc-efficiency.html)


## Installation

### Bioconductor
BioQC is available [from Bioconductor](https://www.bioconductor.org/packages/release/bioc/html/BioQC.html). 

You can install it using 
```R
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

BiocManager::install("BioQC")
```

### Bioconda
Alternatively, you can use the [conda](https://docs.conda.io/en/latest/miniconda.html) package
manager. 

1. Make sure you [set-up the Bioconda channel correctly](https://bioconda.github.io/user/install.html#set-up-channels). The order of the channels is important!
2. (Optional) Create and activate an environment for BioQC
   
   ```bash
   conda create -n bioqc
   conda activate bioqc
   ```

3. Install the `bioconductor-bioqc` package in your current environment
   
   ```bash
   conda install bioconductor-bioqc
   ```

### From Github

The easiest way to install the development version from GitHub is using the `remotes` package: 

```R
install.packages("remotes")
remotes::install_github("accio/BioQC")
```

## Contact

If you have questions regarding BioQC or want to report a bug, please use the [issue tracker](https://github.com/accio/BioQC/issues). 

Alternatively you can reach out to Jitao David Zhang via [e-mail](jitao_david.zhang@roche.com).

#' Load current BioQC signatures
#' @param uniqGenes Logical, whether duplicated genes should be removed, passed
#' to \code{\link{readGmt}}.
#' @param namespace Character, namespace of the gene-set, or code{NULL}, passed
#' to \code{\link{readGmt}}
#' @return A GmtList
#' @seealso \code{\link{readGmt}}
#' @examples
#' readCurrentSignatures()
#' @export
readCurrentSignatures <- function(uniqGenes=TRUE, namespace=NULL) {
    file <- system.file("extdata/exp.tissuemark.bioqc.roche.symbols.gmt", package="BioQC")
    res <- readGmt(file, uniqGenes=uniqGenes, namespace=namespace)
    return(res)
}

## ----setup, include=FALSE-----------------------------------------------------
library(knitr)
opts_chunk$set(out.extra='style="display:block; margin: auto"', fig.align="center")

## ----seed, include=FALSE------------------------------------------------------
set.seed(1887)

## ----lib, warning=FALSE, message=FALSE, results="hide"------------------------
library(Biobase)
library(BioQC)
gmtFile <- system.file("extdata/exp.tissuemark.affy.roche.symbols.gmt", package="BioQC")
gmt <- readGmt(gmtFile)

## ----gmtReadin----------------------------------------------------------------
Nrow <- 2000L
Nsample <- 5L
gss <- unique(unlist(sapply(gmt, function(x) x$genes)))
myEset <- new("ExpressionSet",
              exprs=matrix(rnorm(Nrow*Nsample), nrow=Nrow),
              featureData=new("AnnotatedDataFrame",
                              data.frame(GeneSymbol=sample(gss, Nrow))))

## ----runBioQC-----------------------------------------------------------------
dummyRes <- wmwTest(myEset, gmt, valType="p.greater", simplify=TRUE)
summary(p.adjust(dummyRes, "BH"))

## ----basicDs------------------------------------------------------------------
myMatrix <- matrix(rnorm(Nrow*Nsample),
                   ncol=Nsample,
                   dimnames=list(NULL, LETTERS[1:Nsample]))
myList <- list(signature1=sample(1:Nrow, 100),
               signature2=sample(1:Nrow, 50),
               signature3=sample(1:Nrow, 200))
wmwTest(myMatrix, myList, valType="p.greater", simplify=TRUE)

## ----benchmark----------------------------------------------------------------
bm.Nrow <- 22000
bw.Nsample <- 5
bm.Ngs <- 5
bm.Ngssize <- sapply(1:bm.Ngs, function(x) sample(1:bm.Nrow/2, replace=TRUE))
ind <- lapply(1:bm.Ngs, function(i) sample(1:bm.Nrow, bm.Ngssize[i]))
exprs <- matrix(round(rnorm(bm.Nrow*bw.Nsample),4), nrow=bm.Nrow)

system.time(Cres <- wmwTest(exprs, ind, valType="p.less", simplify=TRUE))
system.time(Rres <- apply(exprs, 2, function(x)
                          sapply(ind, function(y)
                                 wmwTestInR(x, y, valType="p.less"))))


## ----backgroundMatters--------------------------------------------------------
bgVec <- rnorm(20000)
bgVec[1:10000] <- bgVec[1:10000] + 2
bgVec[1:10] <- bgVec[1:10] + 1
ind <- c(1:10)

(pAllGenes <- wmwTest(bgVec, ind))
(pHighExpGenes <- wmwTest(bgVec[1:10000], ind))

## ----session_info-------------------------------------------------------------
sessionInfo()

##-------------------------------##
## convenience funcs for GmtList
##-------------------------------##
#' Gene-set names
#' 
#' @param x A \code{GmtList} object
#' @return Names as a vector of character strings of the same length as \code{x}
#' @export gsName
gsName <- function(x) sapply(x, function(xx) xx$name)

#' Gene-set descriptions
#' 
#' @param x A \code{GmtList} object
#' @return Descriptions as a vector of character strings of the same length as \code{x}
#' @export gsDesc
gsDesc <-  function(x) sapply(x, function(xx) xx$desc)


#' Gene-set member genes
#' 
#' @param x A \code{GmtList} object
#' @return A list of genes as character strings of the same length as \code{x}
#' @export gsGenes
gsGenes <- function(x) lapply(x, function(xx) xx$genes)


#' Gene-set gene counts
#' 
#' @param x A \code{GmtList} or similar object
#' @param uniqGenes Logical, whether only unique genes are counted
#' 
#' @return Gene counts (aka gene-set sizes) as a vector of integer of the same length as \code{x}
#' @export gsGeneCount
gsGeneCount <- function(x, uniqGenes=TRUE) {
  res <- sapply(x, function(x) {
    genes <- x$genes
    if(uniqGenes)
      genes <- unique(genes)
    return(length(genes))
  })
  return(res)
}

#' gsSize is the synonym of gsGeneCount
#' @rdname gsGeneCount
#' @export gsSize
gsSize <- function(x, uniqGenes=TRUE) gsGeneCount(x, uniqGenes=uniqGenes)


#' Whether namespace is set
#' 
#' @param x A \code{GmtList} object
#' @return Logical, whether all gene-sets have the field \code{namespace} set
#' @export hasNamespace
hasNamespace <- function(x) all(sapply(x, function(xx) !is.null(xx$namespace)))

#' Gene-set namespaces
#' 
#' @param x A \code{GmtList} object
#' @return Namespaces as a vector of character strings of the same length as \code{x}
#' @export gsNamespace
gsNamespace <- function(x) sapply(x, function(xx) xx$namespace)

#' Filter a GmtList by size
#' 
#' @param x A \code{GmtList} object
#' @param min Numeric, gene-sets with fewer genes than \code{min} will be removed
#' @param max Numeric, gene-sets with more genes than \code{max} will be removed
#' 
#' @return A \code{GmtList} object with sizes (count of genes) between \code{min} and \code{max} (inclusive).
#' @export filterBySize
filterBySize <- function(x, min, max) function(x, min, max) {
  sizes <- gsSize(x)
  isKept <- rep.int(TRUE, length(sizes))
  if(!missing(min))
    isKept[sizes<min] <- FALSE
  if(!missing(max))
    isKept[sizes>max] <- FALSE
  res <- x[isKept]
  return(res)
}

#' Set the namespace field in each gene-set within a GmtList
#' 
#' @param x A \code{GmtList} object encoding a list of gene-sets
#' @param namespace It can be either a function that applies to a \code{gene-set list} element of the object (for instance \code{function(x) x$desc} to extract description), or a vector of the same length of \code{x}, or in the special case \code{NULL}, which will erase the field namespace.
#' 
#' Note that using vectors as \code{namespace} leads to poor performance when the input object has many gene-sets.
#' 
#' @examples 
#' myGmtList <- GmtList(list(list(name="GeneSet1", desc="Namespace1", genes=LETTERS[1:3]),
#'   list(name="GeneSet2", desc="Namespace1", genes=rep(LETTERS[4:6],2)),
#'   list(name="GeneSet1", desc="Namespace1", genes=LETTERS[4:6]),
#'   list(name="GeneSet3", desc="Namespace2", genes=LETTERS[1:5])))
#' hasNamespace(myGmtList)
#' myGmtList2 <- setNamespace(myGmtList, namespace=function(x) x$desc)
#' gsNamespace(myGmtList2)
#' ## the function can provide flexible ways to encode the gene-set namespace
#' myGmtList3 <- setNamespace(myGmtList, namespace=function(x) gsub("Namespace", "C", x$desc))
#' gsNamespace(myGmtList3)
#' ## using vectors
#' myGmtList4 <- setNamespace(myGmtList, namespace=c("C1", "C1", "C1", "C2"))
#' gsNamespace(myGmtList4)
#' myGmtList2null <- setNamespace(myGmtList2, namespace=NULL)
#' hasNamespace(myGmtList2null)
#' @export
setNamespace <- function(x, namespace) {
  if(missing(namespace)) {
    stop("'namespace' must be given. It can be a function applied to each geneset-, NULL, or a vector.")
  }
  if(is.function(namespace)) {
    res <- GmtList(lapply(x, function(gs) {
      gs$namespace <- do.call(namespace, list(gs))
      return(gs)
    }))
  } else if (is.null(namespace)) {
    res <- GmtList(lapply(x, function(gs) {
      gs$namespace <- NULL
      return(gs)
    }))
  } else {
    stopifnot((length(namespace) == length(x) ||
                 length(namespace) == 1) &&
                (is.character(namespace) || is.factor(namespace) || 
                   is.numeric(namespace) || is.logical(namespace)))
    if(length(namespace)==1) {
      namespace <- rep_len(namespace, length(x))
    }
    res <- GmtList(lapply(seq(along=x), function(i) {
      gs <- x[[i]]
      gs$namespace <- namespace[i]
      return(gs)
    }))
  }
  return(res)
}

#' gsNamespace<- is the synonym of setGsNamespace
#' @rdname setGsNamespace
#' @param x A \code{GmtList} object
#' @param value \code{namespace} in \code{setGsNamespace}. It can be either a function that applies to a \code{gene-set list} element of the object (for instance \code{function(x) x$desc} to extract description), or a vector of the same length of \code{x}, or in the special case \code{NULL}, which will erase the field namespace.
#' @export gsNamespace<-
`gsNamespace<-` <- function(x, value) {
  return(setNamespace(x, namespace=value))
}

#' Set gene-set description as namespace
#' 
#' @param x A \code{GmtList} object
#' 
#' This function wrapps \code{setNamespace} to set gene-set description as namespace
#' @seealso \code{\link{setNamespace}}
#' @export
setDescAsNamespace <- function(x) {
  return(setNamespace(x, namespace = function(xx) xx$desc))
}

atomEntropy <- function(x) ifelse(x==0, 0, x*log2(x))
atomDiversity <- function(x) {
  rel <- x/sum(x, na.rm=TRUE)
  -sum(atomEntropy(rel))
}

#' Shannon entropy
#' 
#' @param vector A vector of numbers, or characters. Discrete probability of each item is calculated and the Shannon entropy is returned.
#' @return Shannon entropy
#' 
#' Shannon entropy can be used as measures of gene expression
#' specificity, as well as measures of tissue diversity and
#' specialization. See references below.
#' 
#' We use \code{2} as base for the entropy calculation, because in this
#' base the unit of entropy is \emph{bit}.
#'
#' @references   Martinez and Reyes-Valdes (2008) Defining diversity, 
#' specialization, and gene specificity in transcriptomes through information 
#' theory. PNAS 105(28):9709--9714
#'  
#' @author Jitao David Zhang <jitao_david.zhang@roche.com>
#' 
#' @examples 
#' myVec0 <- 1:9
#' entropy(myVec0) ## log2(9)
#' myVec1 <- rep(1, 9)
#' entropy(myVec1)
#' 
#' entropy(LETTERS)
#' entropy(rep(LETTERS, 5))
#' @export
entropy <- function(vector) {
  -sum(atomEntropy(table(vector)/length(vector)))
}

#' Entropy-based gene-expression specificity
#' @param mat A matrix (usually an expression matrix), with genes (features) in rows and samples in columns.
#' @param norm Logical, whether the specificity should be normalized by \code{log2(ncol(mat))}.
#' @return A vector of the length of the row number of the input matrix, namely the specificity score of genes.
#' 
#' @references   Martinez and Reyes-Valdes (2008) Defining diversity, 
#' specialization, and gene specificity in transcriptomes through information 
#' theory. PNAS 105(28):9709--9714
#' 
#' @seealso \code{\link{entropy}}
#' @examples
#' myMat <- rbind(c(3,4,5),c(6,6,6), c(0,2,4))
#' entropySpecificity(myMat)
#' entropySpecificity(myMat, norm=TRUE)
#'
#' myRandomMat <- matrix(runif(1000), ncol=20)
#' entropySpecificity(myRandomMat)
#' entropySpecificity(myRandomMat, norm=TRUE)
#' @export
entropySpecificity <- function(mat, norm=FALSE) {
  mat.rel <- apply(mat, 2L, function(x) x/sum(x, na.rm=TRUE))
  speci <- apply(mat.rel, 1L, function(x) {
    gm <- mean(x, na.rm=TRUE)
    rel <- x/gm
    mean(atomEntropy(rel), na.rm=TRUE)
  })
  if(norm)
    speci <- speci/log2(ncol(mat))
  return(speci)
}

#' Entropy-based sample diversity
#' @param mat A matrix (usually an expression matrix), with genes (features) in rows and samples in columns.
#' @param norm Logical, whether the diversity should be normalized by \code{log2(nrow(mat))}.
#' @return A vector as long as the column number of the input matrix
#' 
#' @references   Martinez and Reyes-Valdes (2008) Defining diversity, 
#' specialization, and gene specificity in transcriptomes through information 
#' theory. PNAS 105(28):9709--9714
#' 
#' @seealso \code{\link{entropy}} and \code{\link{sampleSpecialization}}
#' @examples
#' myMat <- rbind(c(3,4,5),c(6,6,6), c(0,2,4))
#' entropyDiversity(myMat)
#' entropyDiversity(myMat, norm=TRUE)
#'
#' myRandomMat <- matrix(runif(1000), ncol=20)
#' entropyDiversity(myRandomMat)
#' entropyDiversity(myRandomMat, norm=TRUE)
#' @export
entropyDiversity <- function(mat, norm=FALSE) {
  res <- apply(mat, 2L, atomDiversity)
  if(norm)
    res <- res/log2(nrow(mat))
  res
}


#' Entropy-based sample specialization
#' @param mat A matrix (usually an expression matrix), with genes (features) in rows and samples in columns.
#' @param norm Logical, whether the specialization should be normalized by \code{log2(ncol(mat))}.
#' @return A vector as long as the column number of the input matrix
#' 
#' @references   Martinez and Reyes-Valdes (2008) Defining diversity, 
#' specialization, and gene specificity in transcriptomes through information 
#' theory. PNAS 105(28):9709--9714
#' 
#' @seealso \code{\link{entropy}} and \code{\link{entropyDiversity}}
#' @examples
#' myMat <- rbind(c(3,4,5),c(6,6,6), c(0,2,4))
#' sampleSpecialization(myMat)
#' sampleSpecialization(myMat, norm=TRUE)
#'
#' myRandomMat <- matrix(runif(1000), ncol=20)
#' sampleSpecialization(myRandomMat)
#' sampleSpecialization(myRandomMat, norm=TRUE)
#' @export
sampleSpecialization <- function(mat, norm=TRUE) {
  speci <- entropySpecificity(mat, norm=FALSE)
  rel <- apply(mat, 2L, function(x) x/sum(x, na.rm=TRUE))
  res <- apply(rel, 2L, function(x) sum(x*speci, na.rm=TRUE))
  if(norm)
    res <- res/log2(ncol(mat))
  res
}

## ----setup, include=FALSE-----------------------------------------------------
library(knitr)
opts_chunk$set(out.extra='style="display:block; margin: auto"', fig.align="center")

## ----lib, warning=FALSE, message=FALSE, results="hide"------------------------
library(BioQC)

## ----gmt----------------------------------------------------------------------
gmtFile <- system.file("extdata/test.gmt", package="BioQC")
## print the file context
cat(readLines(gmtFile), sep="\n")
## read the file into SignedGenesets
genesets <- readSignedGmt(gmtFile)
print(genesets)

## ----gmtPos-------------------------------------------------------------------
genesets <- readSignedGmt(gmtFile, nomatch="pos")
print(genesets)

## ----data---------------------------------------------------------------------
set.seed(1887)
testN <- 100L
testSampleCount <- 3L
testGenes <- c("AKT1", "AKT2", "ERBB2", "ERBB3", "EGFR","TSC1", "TSC2", "GATA2", "GATA4", "GATA1", "GATA3")
testRows <- c(testGenes, paste("Gene", (length(testGenes)+1):testN, sep=""))
testMatrix <- matrix(rnorm(testN*testSampleCount, sd=0.1), nrow=testN, dimnames=list(testRows, NULL))
testMatrix[1:2,] <- testMatrix[1:2,]+10
testMatrix[6:7,] <- testMatrix[6:7,]-10
testMatrix[3:4,] <- testMatrix[3:4,]-5
testMatrix[5,] <- testMatrix[5,]+5
testEset <- new("ExpressionSet", exprs=testMatrix)

## ----index--------------------------------------------------------------------
testIndex <- matchGenes(genesets, testEset, col=NULL)
print(testIndex)

## ----runWmwGreater------------------------------------------------------------
wmwResult.greater <- wmwTest(testEset, testIndex, valType="p.greater")
print(wmwResult.greater)

## ----runWmwLess---------------------------------------------------------------
wmwResult.less <- wmwTest(testEset, testIndex, valType="p.less")
print(wmwResult.less)

## ----runWmwTwoSided-----------------------------------------------------------
wmwResult.two.sided <- wmwTest(testEset, testIndex, valType="p.two.sided")
print(wmwResult.two.sided)

## ----runWmwQ------------------------------------------------------------------
wmwResult.Q <- wmwTest(testEset, testIndex, valType="Q")
print(wmwResult.Q)

## ----session------------------------------------------------------------------
sessionInfo()

appendOneGmtList <- function(gmtList, newGmtList) {
  res <- gmtList
  res@.Data <- c(res@.Data, newGmtList@.Data)
  names(res) <- c(gsName(gmtList), gsName(newGmtList))
  return(res)
}

#' Append a GmtList object to another one
#' 
#' @param gmtList A \code{GmtList} object
#' @param newGmtList Another \code{GmtList} object to be appended
#' @param ... Further \code{GmtList} object to be appended
#' 
#' @return A new \code{GmtList} list, with all elements in the input appended in the given order
#' 
#' @examples 
#' test_gmt_file<- system.file("extdata/test.gmt", package="BioQC")
#' testGmtList1 <- readGmt(test_gmt_file, namespace="test1")
#' testGmtList2 <- readGmt(test_gmt_file, namespace="test2")
#' testGmtList3 <- readGmt(test_gmt_file, namespace="test3")
#' testGmtAppended <- appendGmtList(testGmtList1, testGmtList2, testGmtList3)
#' @export
appendGmtList <- function(gmtList, newGmtList, ...) {
  res <- appendOneGmtList(gmtList, newGmtList)
  newList <- list(...)
  for (ngs in newList) {
    res <- appendOneGmtList(res, ngs)
  }
  return(res)
}

#' Prettify default signature names
#' @param names Character strings, signature names
#' @param includeNamespace Logical, whether the namespace of the signatures should be included
#' @return Character strings, pretty signature names
#' @examples
#' sig <- readCurrentSignatures()
#' prettyNames <- prettySigNames(names(sig))
#' @export
prettySigNames <- function(names, includeNamespace=TRUE) {
  res <- gsub("_[0-9\\.]*_[0-9]$", "", names)
  res <- gsub("_", " ", res)
  res <- gsub("\\s+", " ", res)
  nss <- c("NR", "Roche", "Linsley", "Fantom5")
  for (ns in nss) {
    isNS <- grepl(ns, res)
    fmt <- paste0(ns, ".*$")
    if(includeNamespace) {
      res[isNS] <- gsub(fmt, paste0("[", ns, "]"), res[isNS])
    } else {
      res[isNS] <- gsub(fmt, "", res[isNS])
    }
  }
  return(res)
}

#' Convert a list of gene symbols into a gmtlist
#'
#' @param list A named list with character vectors of genes. Names will become names of gene sets; character vectors will become genes
#' @param description Character, description of gene-sets. The value will be expanded to the same length of the list.
#' @param uniqGenes Logical, whether redundant genes should be made unique?
#' @param namespace Character or \code{NULL}, namespace of the gene-set
#' 
#' @examples
#' testVec <- list(GeneSet1=c("AKT1", "AKT2"),
#'                GeneSet2=c("MAPK1", "MAPK3"),
#'                GeneSet3=NULL)
#' testVecGmtlist <- as.GmtList(testVec)
#' 
#' @export
as.GmtList <- function(list, description=NULL,
                       uniqGenes=TRUE,
                       namespace=NULL) {
  if(is.null(names(list)))
    stop('The input list must have non-null names')
  names <- names(list)
  if(!is.null(description)) {
    descs <- rep_len(description, length.out=length(list))
  }
  res <- lapply(seq(along=list),
                function(i) {
                  if(!is.null(description)) {
                    desc <- descs[i]
                  } else {
                    desc <- NULL
                  }
                  genes <- list[[i]]
                  if(uniqGenes) {
                    genes <- unique(genes)
                  }
                  list(name=names[i],
                       desc=desc,
                       genes=genes,
                       namespace=namespace)
                })
  names(res) <- names
  return(GmtList(res))
}

##----------------------------------------##
## Helper functions
##----------------------------------------##
getSymbols <- function(exprsMatrixRowNames, featureAnno, col="GeneSymbol") {
   if(is.null(featureAnno)) {
     col <- NULL
   } else {
     if (!is.null(col) && !col %in% colnames(featureAnno)) {
       stop("'col' not found in the column names of feature annotation")
     }
   }
  
  if (is.null(col)) {
    symbols <- exprsMatrixRowNames
  } else {
    symbols <- featureAnno[, col]
  }
  symbols <- as.character(symbols)
  return(symbols)
}

#' @importFrom Biobase fData featureNames
eSetSymbols <- function(object, col="GeneSymbol") {
  getSymbols(featureNames(object), fData(object), col=col)
}

#' @importClassesFrom edgeR DGEList
DGEListSymbols <- function(object, col="GeneSymbol") {
  getSymbols(rownames(object$counts), object$genes, col=col)
}

##----------------------------------------##
## matchGenes for GmtList
##----------------------------------------##
matchGeneExclNA <- function(inputGene, allGene) {
    if(is.null(inputGene)) return(NULL)
    res <- match(inputGene, allGene)
    res[is.na(inputGene)] <- NA ## make sure that NA does not match to NA
    return(res)
}

#' @importFrom methods is
matchGenes.default <- function(gmtList,geneSymbols) {
    if(!is(gmtList, "GmtList"))
        stop(paste("gmtlist be must of class GmtList; now it is", class(gmtList)))
    if(!is.character(geneSymbols))
        stop(paste("geneSymbols be must characters; now it is", class(geneSymbols)))
    genes <- lapply(gmtList, function(x) x$genes)
    names(genes) <- sapply(gmtList, function(x) x$name)
    indList <- lapply(genes, matchGeneExclNA, allGene=geneSymbols)
    res <- IndexList(indList)
    return(res)
}
#' Match genes in a list-like object to a vector of genesymbols
#'
#' @param list A GmtList, list, character or SignedGenesets object
#' @param object Gene symbols to be matched; they can come from a vector of character strings, or
#' a column in the fData of an \code{eSet} object.
#' @param ... additional arguments like \code{col}
#' @param col Column name of \code{fData} in an \code{eSet} object, or \code{genes} in an \code{DGEList} object, to specify where gene symbols are stored.
#' The default value is set to "GeneSymbol"
#' 
#' @return An \code{IndexList} object, which is essentially a list of the same length as input (length of \code{1} in case characters are used as input), with matching indices.
#' 
#' @name matchGenes
#' @examples
#' ## test GmtList, character
#' testGenes <- sprintf("gene%d", 1:10)
#' testGeneSets <- GmtList(list(gs1=c("gene1", "gene2"), gs2=c("gene9", "gene10"), gs3=c("gene100")))
#' matchGenes(testGeneSets, testGenes)
#' 
#' ## test GmtList, matrix
#' testGenes <- sprintf("gene%d", 1:10)
#' testGeneSets <- GmtList(list(gs1=c("gene1", "gene2"), gs2=c("gene9", "gene10"), gs3=c("gene100")))
#' testGeneExprs <- matrix(rnorm(100), nrow=10, dimnames=list(testGenes, sprintf("sample%d", 1:10)))
#' matchGenes(testGeneSets, testGeneExprs)
#' 
#' ## test GmtList, eSet
#' testGenes <- sprintf("gene%d", 1:10)
#' testGeneSets <- GmtList(list(gs1=c("gene1", "gene2"), gs2=c("gene9", "gene10"), gs3=c("gene100")))
#' testGeneExprs <- matrix(rnorm(100), nrow=10, dimnames=list(testGenes, sprintf("sample%d", 1:10)))
#' testFeat <- data.frame(GeneSymbol=rownames(testGeneExprs), row.names=testGenes)
#' testPheno <- data.frame(SampleId=colnames(testGeneExprs), row.names=colnames(testGeneExprs))
#' testEset <- ExpressionSet(assayData=testGeneExprs,
#'     featureData=AnnotatedDataFrame(testFeat),
#'     phenoData=AnnotatedDataFrame(testPheno))
#' matchGenes(testGeneSets, testGeneExprs)
#' ## force using row names
#' matchGenes(testGeneSets, testEset, col=NULL)
#' 
#'  ## test GmtList, DGEList
#'  if(requireNamespace("edgeR")) {
#'     mat <- matrix(rnbinom(100, mu=5, size=2), ncol=10)
#'     rownames(mat) <- sprintf("gene%d", 1:nrow(mat))
#'     y <- edgeR::DGEList(counts=mat, group=rep(1:2, each=5))
#'
#'     ## if genes are not set, row names of the count matrix will be used for lookup
#'     myGeneSet <- GmtList(list(gs1=rownames(mat)[1:2], gs2=rownames(mat)[9:10], gs3="gene100"))
#'     matchGenes(myGeneSet, y)
#'
#'     matchGenes(c("gene1", "gene2"), y)
#'     ## alternatively, use 'col' parameter to specify the column in 'genes'
#'     y2 <- edgeR::DGEList(counts=mat,
#'       group=rep(1:2, each=5),
#'       genes=data.frame(GeneIdentifier=rownames(mat), row.names=rownames(mat)))
#'     matchGenes(myGeneSet, y2, col="GeneIdentifier")
#'  }
#' 
#' ## test character, character
#' matchGenes(c("gene1", "gene2"), testGenes)
#' 
#' ## test character, matrix
#' matchGenes(c("gene1", "gene2"), testGeneExprs)
#' 
#' ## test character, eset
#' matchGenes(c("gene1", "gene2"), testEset)
NULL

#'@rdname matchGenes
#'@aliases matchGenes,GmtList,character-method
setMethod("matchGenes", c("GmtList", "character"), function(list, object) {
              matchGenes.default(list, object)
          })

#'@rdname matchGenes
#'@aliases matchGenes,GmtList,matrix-method
setMethod("matchGenes", c("GmtList", "matrix"), function(list, object) {
              if(is.null(rownames(object)))
                  stop("When used to map genes in GmtList directly to rows in matrix, the matrix's row names must be gene symbols")
              symbols <- rownames(object)
              matchGenes.default(list, as.character(symbols))
          })


#'@rdname matchGenes
#'@aliases matchGenes,GmtList,eSet-method
setMethod("matchGenes", c("GmtList", "eSet"), function(list, object, col="GeneSymbol") {
              symbols <- eSetSymbols(object, col=col)
              matchGenes.default(list, as.character(symbols))
          })

#'@rdname matchGenes
#'@aliases matchGenes,character,character-method
setMethod("matchGenes", c("character", "character"), function(list, object) {
              tempList <- GmtList(list(TempGeneSet=list))
              matchGenes.default(tempList, object)
          })
#'@rdname matchGenes
#'@aliases matchGenes,character,matrix-method
setMethod("matchGenes", c("character", "matrix"), function(list, object) {
              tempList <- GmtList(list(TempGeneSet=list))
              matchGenes(tempList, object)
          })
#'@rdname matchGenes
#'@aliases matchGenes,character,eSet-method
setMethod("matchGenes", c("character", "eSet"), function(list, object) {
              tempList <- GmtList(list(TempGeneSet=list))
              matchGenes(tempList, object)
          })

#'@rdname matchGenes
#'@aliases matchGenes,character,DGEList-method
setMethod("matchGenes", c("character", "DGEList"), function(list, object, col="GeneSymbol") {
    tempList <- GmtList(list(TempGeneSet=list))
    matchGenes(tempList, object)
})

#'@rdname matchGenes
#'@aliases matchGenes,GmtList,DGEList-method
setMethod("matchGenes", c("GmtList", "DGEList"), function(list, object, col="GeneSymbol") {
  symbols <- DGEListSymbols(object, col=col)
  matchGenes.default(list, symbols)
})

##----------------------------------------##
## matchGenes for SignedGenesets
##----------------------------------------##
#' @importFrom methods is
matchGenes.signedDefault <- function(signedGenesets, geneSymbols) {
    if(!is(signedGenesets, "SignedGenesets"))
        stop(paste("signedGenesets be must of class SignedGenesets; now it is", class(signedGenesets)))
    resList <- lapply(signedGenesets, function(geneset) {
        pos <- geneset$pos
        neg <- geneset$neg
        posInd <- matchGeneExclNA(geneset$pos, geneSymbols)
        negInd <- matchGeneExclNA(geneset$neg, geneSymbols)
        return(list(pos=posInd, neg=negInd))
    })
    names(resList) <- sapply(signedGenesets, function(x) x$name)
    res <- SignedIndexList(resList)
    return(res)
}

#'@rdname matchGenes
#'@aliases matchGenes,SignedGenesets,character-method
setMethod("matchGenes", c("SignedGenesets", "character"), function(list, object) {
              matchGenes.signedDefault(list, object)
          })

#'@rdname matchGenes
#'@aliases matchGenes,SignedGenesets,matrix-method
setMethod("matchGenes", c("SignedGenesets", "matrix"), function(list, object) {
              if(is.null(rownames(object)))
                  stop("When used to map genes in GmtList directly to rows in matrix, the matrix's row names must be gene symbols")
              symbols <- rownames(object)
              matchGenes.signedDefault(list, as.character(symbols))
          })

#'@rdname matchGenes
#'@aliases matchGenes,SignedGenesets,eSet-method
setMethod("matchGenes", c("SignedGenesets", "eSet"), function(list, object, col="GeneSymbol") {
  symbols <- eSetSymbols(object, col=col)
  matchGenes.signedDefault(list, symbols)
})

#'@rdname matchGenes
#'@aliases matchGenes,SignedGenesets,DGEList-method
setMethod("matchGenes", c("SignedGenesets", "DGEList"), function(list, object, col="GeneSymbol") {
  symbols <- DGEListSymbols(object, col=col)
  matchGenes.default(list, symbols)
})

#' Absolute base-10 logarithm of p-values
#' 
#' @param x Numeric vector or matrix
#' 
#' The function returns the absolute values of base-10 logarithm of p-values.
#' 
#' @details 
#' The logarithm transformation of p-values is commonly used to visualize
#' results from statistical tests. Although it may cause misunderstanding
#' and therefore its use is disapproved by some experts, it helps to
#' visualize and interpret results of statistical tests intuitively.
#' 
#' The function transforms p-values with base-10 logarithm, and returns its
#' absolute value. The choice of base 10 is driven by the simplicity of
#' interpreting the results.
#' 
#' @return Numeric vector or matrix.
#' @author Jitao David Zhang <jitao_david.zhang@roche.com>
#' @importFrom utils head
#' @examples 
#' testp <- runif(1000, 0, 1)
#' testp.al <- absLog10p(testp)
#' 
#' print(head(testp))
#' print(head(testp.al))
#' 
#' @export
absLog10p <- function(x) abs(log10(x))

#' Filter rows of p-value matrix under the significance threshold
#' 
#' @param x A matrix of p-values. It must be raw p-values and should not be transformed (e.g. logarithmic).
#' @param threshold A numeric value, the minimal p-value used to filter
#' rows. If missing, given the values of \code{NA}, \code{NULL} or
#' number \code{0}, no filtering will be done and the input matrix will
#' be returned.
#' 
#' @return   Matrix of p-values. If no line is left, a empty matrix of the same dimension as input will be returned.
#' @author Jitao David Zhang <jitao_david.zhang@roche.com>
#' 
#' @examples 
#' set.seed(1235)
#' testMatrix <- matrix(runif(100,0,1), nrow=10)
#' 
#' ## filtering
#' (testMatrix.filter <- filterPmat(testMatrix, threshold=0.05))
#' ## more strict filtering
#' (testMatrix.strictfilter <- filterPmat(testMatrix, threshold=0.01))
#' ## no filtering
#' (testMatrix.nofilter <- filterPmat(testMatrix))
#' @export 
filterPmat <- function(x, threshold) {
  if(missing(threshold) || is.null(threshold) || is.na(threshold) || threshold==0)
    return(x)
  fil <- apply(x, 1L, function(x) any(x <= threshold))
  x[fil,,drop=FALSE]
}

#' Simplify matrix in case of single row/columns
#'
#' @param matrix A matrix of any dimension
#'
#' If only one row/column is present, the dimension is dropped and a vector will be returned
#'
#' @examples
#' testMatrix <- matrix(round(rnorm(9),2), nrow=3)
#' simplifyMatrix(testMatrix)
#' simplifyMatrix(testMatrix[1L,,drop=FALSE])
#' simplifyMatrix(testMatrix[,1L,drop=FALSE])
#' @export
simplifyMatrix <- function(matrix) {
    if(nrow(matrix)==1L & ncol(matrix)==1L)  {
        matrix <- matrix[1L,1L]
    } else if (nrow(matrix)==1L) {
        matrix <- matrix[1L,]
    } else if (ncol(matrix)==1L)  {
        matrix <- matrix[,1L]
    } 
    return(matrix)
}

#' Function to validate a GmtList object
#' @param object A GmtList object
#' Use \code{setValidity("GmtList", "isValidGmtList")} to check integrity of GmtList objects.
#' It can be very slow, therefore the feature is not turned on by default
#' @export
isValidGmtList <- function(object) {
  isList <- sapply(object, is.list)
  if(!all(isList))
    return(paste("Not all items in the list is a list. \n",
                 "The following items are not:\n",
                 paste(which(!isList), collapse=","),"\n", sep=""))
  isGmt <- sapply(object, function(x) {
    hasNames <- all(c("name", "desc", "genes") %in% names(x))
    isValidName <- !is.null(x$name) & length(x$name)==1
    isValidDesc <- is.null(x$desc) || length(x$desc)==1
    return(hasNames & isValidName & isValidDesc)
  })
  if(!all(isGmt))
    return(paste("Not all items in the list have three mandatory fields 'name', 'desc', and 'genes'",
                 "Following items are not:\n",
                 paste(which(!isGmt), collapse=","), "\n", sep=""))
  return(TRUE)
}

#' Function to validate a SignedGenesets object
#' @param object A SignedGenesets object
#' Use \code{setValidity("SignedGenesets", "isValidSignedGenesets")} to check integrity of SignedGenesets objects.
#' It can be very slow, therefore the feature is not turned on by default
#' @export
isValidSignedGenesets<- function(object) {
  isList <- sapply(object, is.list)
  if(!all(isList))
    return(paste("Not all items in the list is a list. \n",
                 "The following items are not:\n",
                 paste(which(!isList), collapse=","),"\n", sep=""))
  isSigned <- sapply(object, function(x) all(c("name", "pos", "neg") %in% names(x)))
  if(!all(isSigned))
    return(paste("Not all items in the list have three mandatory fields: 'name', 'pos' and 'neg'\n",
                 "Following items are not:\n",
                 paste(which(!isSigned), collapse=","), "\n", sep=""))
  isPosChar <- sapply(object, function(x) is.null(x$pos) || is.character(x$pos))
  isNegChar <- sapply(object, function(x) is.null(x$neg) || is.character(x$neg))
  if(!all(isPosChar & isNegChar))
    return(paste("Not all items in the list have character vectors in the fields 'pos' and 'neg'\n",
                 "Following items are not:\n",
                 paste(which(!(isPosChar & isNegChar)), collapse=","), "\n", sep=""))
  return(TRUE)
}

#' Function to validate a BaseIndexList  object
#' @param object A BaseIndexList  object
#' Use \code{setValidity("BaseIndexList", "isValidBaseIndexList")} to check integrity of BaseIndexList  objects.
#' It can be very slow, therefore the feature is not turned on by default
#' @export
isValidBaseIndexList <- function(object) {
  if(!(length(object@offset)==1L && is.integer(object@offset)))
    return(sprintf("offset must be a single integer. Its value now:%d; class:%s; length:%d",
                   object@offset,
                   class(object@offset),
                   length(object@offset)))
  
  isList <- is.list(object)
  if(!isList)
    return(paste("object must be a list of indices starting from 1"))
  return(TRUE)
}

#' Function to validate an IndexList object
#' @param object an IndexList object
#' Use \code{setValidity("BaseIndexList", "isValidBaseIndexList")} to check integrity of IndexList objects.
#' It can be very slow, therefore the feature is not turned on by default
#' @export
isValidIndexList <- function(object) {
  isInd <- sapply(object, function(x) is.null(x) || is.integer(x))
  if(!all(isInd))
    return(paste("object must be a list of indices starting from 1\n",
                 sprintf("Followings are not: %s", which(!isInd))))
  return(TRUE)
}

#' Function to validate a SignedIndexList object
#' @param object a SignedIndexList object
#' Use \code{setValidity("SignedIndexList", "isValidSignedIndexList")} to check integrity of SignedIndexList objects.
#' It can be very slow, therefore the feature is not turned on by default
#' @export
isValidSignedIndexList <- function(object) {
  isSigned <- sapply(object, function(x)
    all(c("pos", "neg") %in% names(x)))
  if(!all(isSigned))
    return(paste("In some items there are no pos/neg fields\n",
                 sprintf("Following are not: %s", which(!isSigned))))
  isPosInd <- sapply(object, function(x) is.null(x$pos) || is.integer(x$pos))
  isNegInd <- sapply(object, function(x) is.null(x$neg) || is.integer(x$neg))
  if(!all(isPosInd) & !all(isNegInd))
    return("Indices are not valid in following cses\n")
  return(TRUE)
}

## Class definitions

#' An S4 class to hold geneset in the GMT file in a list, each item in the list 
#' is in in turn a list containing following items: name, desc, and genes.
#' @exportClass GmtList
setClass("GmtList", 
         contains="list")

#' An S4 class to hold signed genesets, each item in the list is in in turn a 
#' list containing following items: name, pos, and neg.
#' @exportClass SignedGenesets
setClass("SignedGenesets", contains="list")


#' An S4 class to hold a list of indices, with the possibility to specify the 
#' offset of the indices. IndexList and SignedIndexList extend this class
#'
#' @slot offset An integer specifying the value of first element. Default 1
#' @slot keepNA Logical, whether NA is kept during construction
#' @slot keepDup Logical, whether duplicated values are kept during construction

setClass("BaseIndexList",
         representation=list("offset"="integer",
             "keepNA"="logical",
             "keepDup"="logical"),
         prototype=prototype(offset=1L, keepNA=FALSE, keepDup=FALSE),
         contains="list")

#' An S4 class to hold a list of integers as indices, with the possibility to specify the offset of the indices
#'
#' @slot offset An integer specifying the value of first element. Default 1
#' @slot keepNA Logical, whether NA is kept during construction
#' @slot keepDup Logical, whether duplicated values are kept during construction
#' @name IndexList-class
#' @exportClass IndexList
setClass("IndexList", contains="BaseIndexList")

#'An S4 class to hold a list of signed integers as indices, with the possibility to specify the offset of the indices
#' @slot offset An integer specifying the value of first element. Default 1
#' @slot keepNA Logical, whether NA is kept during construction
#' @slot keepDup Logical, whether duplicated values are kept during construction
#' @exportClass SignedIndexList
setClass("SignedIndexList", contains="BaseIndexList")

##----------------------------------------##
## set validity checking functions
##----------------------------------------##

## they are disabled by default, since running them over large lists is slow
## setValidity("GmtList", isValidGmtList)
## setValidity("SignedGenesets", isValidSignedGenesets)
## setValidity("BaseIndexList", isValidBaseIndexList)
## setValidity("IndexList", isValidIndexList)
## setValidity("SignedIndexList", isValidSignedIndexList)

##----------------------------------------##
## Constructors
##----------------------------------------##

#' Convert a list to a GmtList object
#' @param list A list of genesets; each geneset is a list of at least three fields: 'name', 'desc', and 'genes'. 'name' and 'desc' contains one character string ('desc' can be NULL while 'name' cannot), and 'genes' can be either NULL or a character vector. In addition, 'namespace' is accepted to represent the namespace.
#'
#' For convenience, the function also accepts a list of character vectors, each containing a geneset. In this case, the function works as a wrapper of \code{as.GmtList}
#' 
#' @seealso If a list of gene symbols need to be converted into a GmtList, use 'as.GmtList' instead
#' 
#' @examples
#' testList <- list(list(name="GS_A", desc=NULL, genes=LETTERS[1:3]),
#'                  list(name="GS_B", desc="gene set B", genes=LETTERS[1:5]),
#'                  list(name="GS_C", desc="gene set C", genes=NULL))
#' testGmt <- GmtList(testList)
#'
#' # as wrapper of as.GmtList
#' testGeneList <- list(GS_A=LETTERS[1:3], GS_B=LETTERS[1:5], GS_C=NULL)
#' testGeneGmt <- GmtList(testGeneList)
#' 
#' @importFrom methods new
#' @export
GmtList <- function(list) {
    isGeneSymbols <- all(sapply(list, function(x) is.null(x) || is.character(x)))
    if(isGeneSymbols) {
        return(as.GmtList(list))
    } else {
        res <- new("GmtList", .Data=list)
        names(res) <- names(list)
        return(res)
    }
}

#' Convert a list to a SignedGenesets object
#' @param list A list of genesets; each geneset is a list of at least three fields: 'name', 'pos', and 'neg'. 'name' contains one non-null character string, and both 'pos' and 'neg' can be either NULL or a character vector.
#'
#' @seealso \code{GmtList}
#' 
#' @importFrom methods new
#' @examples
#' testList <- list(list(name="GS_A", pos=NULL, neg=LETTERS[1:3]),
#'                  list(name="GS_B", pos=LETTERS[1:5], neg=LETTERS[7:9]),
#'                  list(name="GS_C", pos=LETTERS[1:5], neg=NULL),
#'                  list(name="GS_D", pos=NULL, neg=NULL))
#' testSigndGS <- SignedGenesets(testList)
#' 
#' @export
SignedGenesets <- function(list) {
    res <- new("SignedGenesets", .Data=list)
    return(res)
}

context("Test matchGene for GmtList")

testGenes <- c("AKT1", "AKT2", "EGFR", "ERBB2", "TSC1", "TSC2", NA)
testCharQuery <- c("AKT1", "AKT2", "AKT3")
expCharQuery <- list(TempGeneSet=c(1L,2L))
test_that("matchGenes works for characters and characters", {
              expect_equivalent(matchGenes(testCharQuery, testGenes), expCharQuery)
          })

testRawGmtList <- list(GS1=c("AKT1", "AKT2"),
                       GS2=c("EGFR", "ERBB2", "ERBB3", "ERBB4"),
                       GS3=NULL,
                       GS4=c("TSC1", "TSC2", "PR", NA))
testGmtList <- as.GmtList(testRawGmtList)
testGmtListWithMethod <- GmtList(testRawGmtList)
test_that("GmtList works as a wrapper of as.GmtList", {
              expect_equal(testGmtList, testGmtListWithMethod)
          })

testMatch <- matchGenes(testGmtList, testGenes)
expMatchList <- list(1:2, 3:4, NULL, 5:6)

test_that("matchGenes works for GmtList and character", {
    expect_equal(offset(testMatch), 1L)
    expect_equivalent(testMatch@.Data, expMatchList)
})

testMatrix <- matrix(rnorm(49),nrow=length(testGenes), dimnames=list(testGenes, NULL))
testMatrixMatch <- matchGenes(testGmtList, testMatrix)
test_that("matchGenes works for GmtList and matrix", {
    expect_equal(offset(testMatrixMatch), 1L)
    expect_equivalent(testMatrixMatch@.Data, expMatchList)
})

testMatrixCharMatch <- matchGenes(testCharQuery, testMatrix)
test_that("matchGenes works for characters and matrix", {
    expect_equal(offset(testMatrixCharMatch), 1L)
    expect_equivalent(testMatrixCharMatch@.Data, expCharQuery)
})


testMatrix2 <- testMatrix; rownames(testMatrix2)[is.na(rownames(testMatrix2))] <- ""
testEset <- new("ExpressionSet",
                exprs=testMatrix2,
                featureData=new("AnnotatedDataFrame",
                    data.frame(GeneSymbol=testGenes,
                               myID=testGenes,
                               row.names=rownames(testMatrix2))))
testEsetMatch.GeneSymbol <- matchGenes(testGmtList, testEset)
testEsetMatch.myID <- matchGenes(testGmtList, testEset, "myID")
testEsetMatch.fname <- matchGenes(testGmtList, testEset, NULL)
test_that("matchGenes works for GmtList and eSet", {
    expect_equal(offset(testEsetMatch.GeneSymbol), 1L)
    expect_equal(offset(testEsetMatch.myID), 1L)
    expect_equal(offset(testEsetMatch.fname), 1L)

    expect_equivalent(testEsetMatch.GeneSymbol@.Data, expMatchList)
    expect_equivalent(testEsetMatch.myID@.Data, expMatchList)
    expect_equivalent(testEsetMatch.fname@.Data, expMatchList)
})

testEsetCharMatch <- matchGenes(testCharQuery, testEset)
test_that("matchGenes works for character and eSet", {
    expect_equivalent(testEsetCharMatch, expCharQuery)
})
context("Test matchGene for SignedGenesets")
testRawSignedGenesets <- list(GS1=list(name="GS1",
                                  pos=c("AKT1", "AKT2"),
                                  neg=c("TSC1", "TSC2", "PR")),
                              GS2=list(name="GS2",
                                  pos=c("EGFR","ERBB3"),
                                  neg=c("ERBB2", "ERBB4")),
                              GS3=list(name="GS3",
                                  pos=NULL,
                                  neg=c("TSC1", "TSC2")),
                              GS4=list(name="GS4",
                                  pos=c("ERBB2", "ERBB4", "PR", NA),
                                  neg=NULL),
                              GS5=list(name="GS5", pos=NULL, neg=NULL))
testSignedGenesets <- SignedGenesets(testRawSignedGenesets)
testSignedMatch <- matchGenes(testSignedGenesets, testGenes)
expSignedMatchList <- list(GS1=list(pos=1:2, neg=5:6),
                           GS2=list(pos=3L, neg=4L),
                           GS3=list(pos=NULL, neg=5:6),
                           GS4=list(pos=4L, neg=NULL),
                           GS5=list(pos=NULL, neg=NULL))

test_that("matchGenes works for SignedGenesets and character", {
    expect_equal(offset(testSignedMatch), 1L)
    expect_equivalent(testSignedMatch@.Data, expSignedMatchList)
    expect_equal(names(testSignedMatch), names(expSignedMatchList))
})

testMatrixSignedMatch <- matchGenes(testSignedGenesets, testMatrix)
test_that("matchSignedGenes works for SignedGenesets and matrix", {
    expect_equal(offset(testMatrixSignedMatch), 1L)
    expect_equivalent(testMatrixSignedMatch@.Data, expSignedMatchList)
})

testEsetSignedMatch.GeneSymbol <- matchGenes(testSignedGenesets, testEset)
testEsetSignedMatch.myID <- matchGenes(testSignedGenesets, testEset, col="myID")
testEsetSignedMatch.fname <- matchGenes(testSignedGenesets, testEset, col=NULL)
test_that("matchSignedGenes works for SignedGenesets and eSet", {
    expect_equal(offset(testEsetSignedMatch.GeneSymbol), 1L)
    expect_equal(offset(testEsetSignedMatch.myID), 1L)
    expect_equal(offset(testEsetSignedMatch.fname), 1L)

    expect_equal(names(testEsetSignedMatch.GeneSymbol), names(expSignedMatchList))
    expect_equal(names(testEsetSignedMatch.myID), names(expSignedMatchList))
    expect_equal(names(testEsetSignedMatch.fname), names(expSignedMatchList))
    
    expect_equivalent(testEsetSignedMatch.GeneSymbol@.Data, expSignedMatchList)
    expect_equivalent(testEsetSignedMatch.myID@.Data, expSignedMatchList)
    expect_equivalent(testEsetSignedMatch.fname@.Data, expSignedMatchList)
})

## test wmwTest
library(BioQC)

context("Non-exact wilcoxon test in R")
testNums <- 1:10
testSub <- rep_len(c(TRUE, FALSE), length.out=length(testNums))
testP <- wmwTestInR(testNums, testSub, valType="p.two.sided")
testW <- wmwTestInR(testNums, testSub, valType="W")
testIndP <- wmwTestInR(testNums, which(testSub), valType="p.two.sided")
testIndW <- wmwTestInR(testNums, which(testSub), valType="W")
expWMW <- wilcox.test(testNums[seq(1,9,2)],
                      testNums[seq(2,10,2)], alternative="two.sided", exact=FALSE)

test_that("Non-exact wilcoxon test in R is wrapped correctly by wmwTestInR", {
             expect_equal(testP, expWMW$p.value)
             expect_equal(testIndP, expWMW$p.value)
             expect_equivalent(testW, expWMW$statistic)
             expect_equivalent(testIndW, expWMW$statistic)
         })

context("Wilcoxon test in R and C")
set.seed(1887)
NROW <- 2000
NCOL <- 5
GSCOUNT <- 50
TOL <- 1E-2
GSSIZE <- sample(1:as.integer(NROW/2), GSCOUNT, replace=TRUE)
ind <- lapply(GSSIZE, function(x) sample(1:NROW, x))
exprs <- matrix(round(rnorm(NROW*NCOL),4), nrow=NROW)

gc()
system.time(Cless <- wmwTest(exprs, ind, valType="p.less"))
system.time(Cgreater <- wmwTest(exprs, ind, valType="p.greater"))
system.time(Ctwosided <- wmwTest(exprs, ind, valType="p.two.sided"))
wmwTestR <- function(matrix, index, valType) {
  sub <- rep(FALSE, length(matrix))
  sub[index]=TRUE
  wmwTestInR(matrix, sub, valType)
}
system.time(Rless <- apply(exprs, 2, function(x)
                          sapply(ind, function(y) 
                                 wmwTestR(x, y, valType="p.less"))))
system.time(Rgreater <- apply(exprs, 2, function(x)
                              sapply(ind, function(y)
                                     wmwTestR(x, y, valType="p.greater"))))
system.time(Rtwosided <- apply(exprs, 2, function(x)
                              sapply(ind, function(y)
                                     wmwTestR(x, y, valType="p.two.sided"))))

test_that("less is identical", {
              expect_equal(Cless, Rless, tolerance=TOL)
          })
test_that("greater is identical", {
              expect_equal(Cgreater, Rgreater,tolerance=TOL)
          })
test_that("two.sided is identical", {
              isNearOne <- 1-Ctwosided<0.01
              expect_equal(object=Ctwosided[!isNearOne], expected=Rtwosided[!isNearOne], tolerance=TOL, scale=1L)
          })


context("Test different combinations of input and valType in wmwTest")
## R-native data structures

rd <- rnorm(1000)
rl <- sample(c(TRUE, FALSE), 1000, replace=TRUE)
wmwTwoSidedC <- wmwTest(rd, rl, valType="p.two.sided")
wmwTwoSidedR <- wmwTestInR(rd, rl, valType="p.two.sided")

rd1 <- rd + ifelse(rl, 0.5, 0)
wmwGreaterC <- wmwTest(rd1, rl, valType="p.greater")
wmwGreaterR <- wmwTestInR(rd1, rl, valType="p.greater")

rd2 <- rd - ifelse(rl, 0.2, 0)
wmwGreaterC2 <- wmwTest(rd2, rl, valType="p.greater")
wmwTwoSidedC2 <- wmwTest(rd2, rl, valType="p.two.sided")
wmwLessC2 <- wmwTest(rd2, rl, valType="p.less")
wmwGreaterR2 <- wmwTestInR(rd2, rl, valType="p.greater")
wmwTwoSidedR2 <- wmwTestInR(rd2, rl, valType="p.two.sided")
wmwLessR2 <- wmwTestInR(rd2, rl, valType="p.less")
test_that("p-values by C and R implementations are identical", {
    expect_equivalent(wmwTwoSidedC, wmwTwoSidedR)
    expect_equivalent(wmwGreaterC, wmwGreaterR)
    expect_equivalent(wmwGreaterC2, wmwGreaterR2)
    expect_equivalent(wmwTwoSidedC2, wmwTwoSidedR2)
    expect_equivalent(wmwLessC2, wmwLessR2)
})

wmwTwoSidedC.ind <- wmwTest(rd, which(rl), valType="p.two.sided")
test_that("wmwTest accepts logical and numeric vector as IndexList", {
    expect_equivalent(wmwTwoSidedC.ind, wmwTwoSidedC)
})

## matrix forms
rmat <- matrix(c(rd, rd1, rd2), ncol=3, byrow=FALSE)
tsMat <- wmwTest(rmat, rl, valType="p.two.sided")
tsMatR <- apply(rmat, 2, function(x)
                wmwTestInR(x, rl, valType="p.two.sided"))
tsMatLess <- wmwTest(rmat, rl, valType="p.less")
tsMatLessR <- apply(rmat, 2, function(x)
                       wmwTestInR(x, rl, valType="p.less"))
tsMatGreater <- wmwTest(rmat, rl, valType="p.greater")
tsMatGreaterR <- apply(rmat, 2, function(x)
                       wmwTestInR(x, rl, valType="p.greater"))
logicTsMat <- wmwTest(rmat, which(rl), valType="p.two.sided")
logicTsMatGreater <- wmwTest(rmat, which(rl), valType="p.greater")
    
test_that("Matrix as input to wmwTest", {
    expect_equivalent(tsMat, tsMatR)
    expect_equivalent(tsMatGreater, tsMatGreaterR)
    expect_equivalent(tsMatLess, tsMatLessR)
    expect_equivalent(tsMat, logicTsMat)
    expect_equivalent(tsMatGreater, logicTsMatGreater)
})

intMat <- matrix(as.integer(rnorm(3000, 5, 3)), nrow=1000)
intMatTs <- wmwTest(intMat, rl, valType="p.two.sided")
intMatGreater <- wmwTest(intMat, rl, valType="p.greater")
intMatLess <- wmwTest(intMat, rl, valType="p.less")
intMatTsR <- apply(intMat, 2, function(x) wmwTestInR(x, rl, valType="p.two.sided"))
intMatGreaterR <- apply(intMat, 2, function(x) wmwTestInR(x, rl, valType="p.greater"))
intMatLessR <- apply(intMat, 2, function(x) wmwTestInR(x, rl, valType="p.less"))

test_that("Integer matrix as input", {
    expect_equivalent(intMatTs, intMatTsR)
    expect_equivalent(intMatGreater, intMatGreaterR)
    expect_equivalent(intMatLess, intMatLessR)
})
    
## transformed val types
tsMatGreaterScore <- wmwTest(rmat, which(rl), valType="abs.log10p.greater")
expTsMatGreaterScore <- abs(log10(tsMatGreater))

tsMatLessScore <- wmwTest(rmat, which(rl), valType="log10p.less")
expTsMatLessScore <- log10(tsMatLess)

tsMatScore <- wmwTest(rmat, which(rl), valType="abs.log10p.two.sided")
expTsMatScore <- abs(log10(tsMat))

matQ <- wmwTest(rmat, which(rl), valType="Q")
expQ <-expTsMatScore
isRevQ <- tsMatLess < tsMatGreater
expQ[isRevQ] <- -expQ[isRevQ]
test_that("Transformed valTypes", {
   expect_equivalent(tsMatGreaterScore,expTsMatGreaterScore)
   expect_equivalent(tsMatLessScore,expTsMatLessScore)
   expect_equivalent(tsMatScore, expTsMatScore)
   expect_equivalent(matQ, expQ)
})


## using ExpressionSet
data(sample.ExpressionSet)
testSet <- sample.ExpressionSet
fData(testSet)$GeneSymbol <- paste("GENE_",1:nrow(testSet), sep="")
mySig1 <- sample(c(TRUE, FALSE), nrow(testSet), prob=c(0.25, 0.75), replace=TRUE)

eSetInt <- wmwTest(testSet, which(mySig1), valType="p.greater")
eSetIntR <- apply(exprs(testSet), 2, function(x)
                  wmwTestInR(x, which(mySig1), valType="p.greater"))



## using lists
mySig2 <- sample(c(TRUE, FALSE), nrow(testSet), prob=c(0.6, 0.4), replace=TRUE)
sigLogLists <- list(first=mySig1, second=mySig2)
sigIntLists <- list(first=which(mySig1), second=which(mySig2))
sigLog <- wmwTest(testSet, sigLogLists, valType="p.greater")
sigInt <- wmwTest(testSet, sigIntLists, valType="p.greater")
sigLogR <- apply(exprs(testSet), 2, function(x)
                 sapply(sigLogLists, function(vec)
                        wmwTestInR(x, vec, valType="p.greater")))

test_that("eSet", {
    expect_equivalent(eSetInt, eSetIntR)
    expect_equivalent(sigLog, sigInt)
    expect_equivalent(sigLog, sigLogR)
})

## using GMT file
gmt_file <- system.file("extdata/exp.tissuemark.affy.roche.symbols.gmt", package="BioQC")
gmt_list <- readGmt(gmt_file)

gss <- sample(unlist(sapply(gmt_list, function(x) x$genes)), 500)
eset<-new("ExpressionSet",
          exprs=matrix(rnorm(4000), nrow=500L),
          phenoData=new("AnnotatedDataFrame", data.frame(Sample=LETTERS[1:8])),
          featureData=new("AnnotatedDataFrame",data.frame(GeneSymbol=gss)))
esetWmwRes <- wmwTest(eset ,gmt_list, valType="p.greater")
gmtInd <- matchGenes(gmt_list, eset, "GeneSymbol")
esetWmwResR <- wmwTest(exprs(eset), gmtInd, valType="p.greater")
                             
test_that("eSet and GMT", {
    expect_equivalent(esetWmwRes, esetWmwResR)
})

context("Test wmwTest for SignedGenesets")
testGenes <- c("AKT1", "AKT2", "EGFR", "ERBB2", "TSC1", "TSC2", "F4")
testRows <- c(testGenes, paste("Gene", (length(testGenes)+1):100, sep=""))
testRawSignedGenesets <- list(GS1=list(name="GS1",
                                  pos=c("AKT1", "AKT2"),
                                  neg=c("TSC1", "TSC2", "PR")),
                              GS2=list(name="GS2",
                                  pos=c("EGFR","ERBB3"),
                                  neg=c("ERBB2", "ERBB4")),
                              GS3=list(name="GS3",
                                  pos=NULL,
                                  neg=c("TSC1", "TSC2")),
                              GS4=list(name="GS4",
                                  pos=c("EGFR", "ERBB2", "PR", NA),
                                  neg=NULL),
                              GS5=list(name="GS5", pos=NULL, neg=NULL))
testSignedGenesets <- SignedGenesets(testRawSignedGenesets)
testSignedMatch <- matchGenes(testSignedGenesets, testRows)
expSignedMatch <- list(list(pos=1:2, neg=5:6),
                       list(pos=3L, neg=4L),
                       list(pos=NULL, neg=5:6),
                       list(pos=3:4, neg=NULL),
                       list(pos=NULL, neg=NULL))

testMatrix <- matrix(rnorm(1000, sd=0.1),
                     nrow=100,
                     dimnames=list(testRows, NULL))
testMatrix[1,] <- testMatrix[1,]+20
testMatrix[2,] <- testMatrix[2,]+10
testMatrix[3,] <- testMatrix[3,]-40
testMatrix[4,] <- testMatrix[4,]-30
testMatrix[5,] <- testMatrix[5,]-20
testMatrix[6,] <- testMatrix[6,]-10
testMatrixRankHead <- apply(testMatrix, 2, rank)[1:6,]
expMatrixRankHead <- matrix(rep(c(100, 99, 1, 2, 3, 4), each=10), ncol=10, byrow=TRUE)


testSignedGreater <- wmwTest(testMatrix, testSignedMatch, valType="p.greater")
testSignedLess <- wmwTest(testMatrix, testSignedMatch, valType="p.less")
testSignedTwoSided <- wmwTest(testMatrix, testSignedMatch, valType="p.two.sided")
testSignedQ <- wmwTest(testMatrix, testSignedMatch, valType="Q")
testSignedU2 <- wmwTest(testMatrix, testSignedMatch, valType="U2")
## U statistic is calculated by hand
expSignedUBase <- c(0, 99, 4, 196, 0)
expSignedU <- matrix(rep(expSignedUBase, each=10L), ncol=10, byrow=TRUE)

test_that("wmwTest works for signed genesets", {
              expect_equivalent(testMatrixRankHead, expMatrixRankHead)
              expect_equivalent(testSignedMatch, expSignedMatch)
              expect_equivalent(testSignedU2, expSignedU)
          })

testSignedU2col1 <- wmwTest(testMatrix[,1], testSignedMatch, valType="U2")
testSignedEset <- new("ExpressionSet",
                      exprs=testMatrix)
testSignedEsetU2 <- wmwTest(testSignedEset, testSignedMatch, valType="U2")

test_that("wmwTest works for signed genesets and polymorphism", {
              expect_equivalent(testSignedU2col1, expSignedUBase)
              expect_equivalent(testSignedEsetU2, expSignedU)

              expect_equal(names(testSignedU2col1), names(testRawSignedGenesets))
              expect_equal(rownames(testSignedEsetU2), names(testRawSignedGenesets))
          })

readSingleGmt <- function(filename, uniqGenes=TRUE, namespace=NULL) {
  if(!file.exists(filename))
    stop(sprintf("File %s does not exist", filename))
  lines <- readLines(filename)
  splitLines <- strsplit(lines, "\t")
  isValid <- sapply(splitLines, function(x) length(x)>=3)
  validLines <- splitLines[isValid]
  res <- lapply(validLines, function(x)  {
    genes <- x[3:length(x)]
    genes <- genes[!genes %in% ""]
    if(uniqGenes) {
      genes <- unique(genes)
    }
    list(name=x[1], desc=x[2], genes=genes, namespace=namespace)
  })
  names(res) <- sapply(res, function(x) x$name)
  return(res)
}

#' Read in gene-sets from a GMT file
#' 
#' @param ... Named or unnamed characater string vector, giving file names of one or more GMT format files. 
#' @param uniqGenes Logical, whether duplicated genes should be removed
#' @param namespace Character, namespace of the gene-set. It can be used to specify namespace or sources of the gene-sets. If  \code{NULL} is given, so no namespace is used and all gene-sets are assumed to come from the same unspecified namespace. The option can be helpful when gene-sets from multiple namespaces are jointly used.
#' 
#' @return A \code{GmtList} object, which is a S4-class wrapper of a list. Each 
#' element in the object is a list of (at least) three items: 
#' \itemize{
#'   \item gene-set name (field \code{name}), character string, accessible with \code{\link{gsName}}
#'   \item gene-set description (field \code{desc}), character string, accessible with \code{\link{gsDesc}}
#'   \item genes (field \code{genes}), a vector of character strings, , accessible with \code{\link{gsGenes}}
#'   \item namespace (field \code{namespace}), accessible with \code{\link{gsNamespace}}
#' }
#' 
#' @note 
#' Currently, when \code{namespace} is set as \code{NULL}, no namespace is used. This may change in the future, since we may use file base name as the default namespace.
#' 
#' @examples 
#' gmt_file <- system.file("extdata/exp.tissuemark.affy.roche.symbols.gmt", package="BioQC")
#' gmt_list <- readGmt(gmt_file)
#' gmt_nonUniqGenes_list <- readGmt(gmt_file, uniqGenes=FALSE)
#' gmt_namespace_list <- readGmt(gmt_file, uniqGenes=FALSE, namespace="myNamespace")
#' 
#' ## suppose we have two lists of gene-sets to read in
#' test_gmt_file <- system.file("extdata/test.gmt", package="BioQC")
#' gmt_twons_list <- readGmt(gmt_file, test_gmt_file, namespace=c("BioQC", "test"))
#' ## alternatively
#' gmt_twons_list <- readGmt(BioQC=gmt_file, test=test_gmt_file)
#' 
#' @export
readGmt <- function(..., uniqGenes=TRUE, namespace=NULL) {
  files.list <- list(...)
  files <- unlist(files.list, use.names=TRUE)
  fnames <- names(files)
  if(!is.null(fnames)) {
    if(!is.null(namespace))
      warning("'file' have names - the namespace option is ignored")
    namespace <- fnames
  } else if(!is.null(namespace)) {
    namespace <- rep(namespace, length.out=length(files))
  }
  gsList <- lapply(seq(along=files), function(i) {
    readSingleGmt(files[i], uniqGenes=uniqGenes, namespace = namespace[i])
  })
  res <- GmtList(unlist(gsList, use.names=TRUE, recursive=FALSE))
  return(res)
}

## ----setup, include=FALSE-----------------------------------------------------
options(fig_caption=TRUE)
library(knitr)
opts_chunk$set(out.extra='style="display:block; margin: auto"', fig.align="center")

## ----lib, warning=FALSE, message=FALSE, results="hide", include=FALSE---------
library(testthat)
library(BioQC)
library(org.Hs.eg.db) ## to simulate an microarray expression dataset
library(lattice)
library(latticeExtra)
library(gridExtra)
library(gplots)
library(rbenchmark)

pdf.options(family="ArialMT", useDingbats=FALSE)

set.seed(1887)

## list human genes
humanGenes <- keys(revmap(org.Hs.egSYMBOL))

## read tissue-specific gene signatures
gmtFile <- system.file("extdata/exp.tissuemark.affy.roche.symbols.gmt",
                       package="BioQC")
gmt <- readGmt(gmtFile)

## ----algofig, echo=FALSE, fig.height=4.5, fig.cap="BioQC speeds up the Wilcoxon-Mann-Whitney test by avoiding futile sorting operations on the same sample."----
knitr::include_graphics("wmw-speedup.png")

## ----time_benchmark, echo=FALSE-----------------------------------------------
randomMatrix <- function(rows=humanGenes, ncol=5) {
  nrow <- length(rows)
  mat <- matrix(rnorm(nrow*ncol),
                nrow=nrow, byrow=TRUE)
  rownames(mat) <- rows
  return(mat)
}
noSamples <- c(1, 5, 10, 20, 50, 100)
noBenchRep <- 100
tmRandomMats <- lapply(noSamples, function(x) randomMatrix(ncol=x))
tissueInds <- sapply(gmt, function(x) match(x$genes, humanGenes))

wmwTestR <- function(matrix, indices, alternative) {
  res <- apply(matrix, 2, function(x) {
    sapply(indices, function(index) {
      sub <- rep(FALSE, length(x))
      sub[index] <- TRUE
      wt <- wilcox.test(x[sub], x[!sub],
                        alternative=alternative,
                        exact=FALSE)
      return(wt$p.value)
      })
    })
  return(res)
  }

## WARNING: very slow (~1-2 hours)
benchmarkFile <- "simulation-benchmark.RData"
if(!file.exists(benchmarkFile)) {
  bioqcRes <- lapply(tmRandomMats, function(mat) {
    bench <- benchmark(wmwTestRes<- wmwTest(mat,
                                            tissueInds,
                                            valType="p.greater",
                                            simplify=TRUE),
                       replications=noBenchRep)
    elapTime <- c("elapsed"=bench$elapsed,
                  "user"=bench$user.self,
                  "sys"=bench$sys.self)/noBenchRep
    res <- list(elapTime=elapTime,
                wmwTestRes=wmwTestRes)
    return(res)
    })
  
  rRes <- lapply(tmRandomMats, function(mat) {
    elapTime <- system.time(wmwTestRes <- wmwTestR(mat, tissueInds, alternative="greater"))
    res <- list(elapTime=elapTime,
                wmwTestRes=wmwTestRes)
    return(res)
    })
  save(bioqcRes, rRes, file=benchmarkFile)
  } else {
    load(benchmarkFile)
 }
getWmwTestRes <- function(x) x$wmwTestRes
rNumRes <- lapply(rRes, getWmwTestRes)
bioqcNumRes <- lapply(bioqcRes, getWmwTestRes)

## ----time_benchmark_identical-------------------------------------------------
expect_equal(bioqcNumRes, rNumRes)

## ----trellis_prepare, echo=FALSE----------------------------------------------
op <- list(layout.widths = list(left.padding = 0, key.ylab.padding = 0.5,
                                ylab.axis.padding = 0.5, axis.right = 0.5, right.padding = 0),
           layout.heights = list(top.padding = 0, bottom.padding = 0,
                                 axis.top = 0, main.key.padding = 0.5, key.axis.padding = 0.5),
           axis.text=list(cex=1.2),
           par.xlab.text=list(cex=1.4),
           par.sub.text=list(cex=1.4),
           add.text=list(cex=1.4),
           par.ylab.text=list(cex=1.4))

## ----time_benchmark_vis, echo=FALSE, fig.width=8, fig.height=4.5, dev='png', dev.args=list(pointsize=2.5), fig.cap="**Figure 2**: Time benchmark results of BioQC and R implementation of Wilcoxon-Mann-Whitney test. Left panel: elapsed time in seconds (logarithmic Y-axis). Right panel: ratio of elapsed time by two implementations. All results achieved by a single thread on in a RedHat Linux server."----
getTimeRes <- function(x) x$elapTime["elapsed"]
bioqcTimeRes <- sapply(bioqcRes, getTimeRes)
rTimeRes <- sapply(rRes, getTimeRes)
timeRes <- data.frame(NoSample=noSamples,
                      Time=c(bioqcTimeRes, rTimeRes),
                      Method=rep(c("BioQC", "NativeR"), each=length(noSamples)))

timeXY <- xyplot(Time ~ NoSample, group=Method, data=timeRes,  type="b",
                 auto.key=list(columns=2L),
                 xlab="Number of samples", ylab="Time [s]",
                 par.settings=list(superpose.symbol=list(cex=1.25, pch=16, col=c("black", "red")),
                                   superpose.line=list(col=c("black", "red"))),
                 scales=list(tck=c(1,0), alternating=1L,
                             x=list(at=noSamples),
                             y=list(log=2, at=10^c(-2, -1, 0,1,2,3, log10(2000)), labels=c(0.01, 0.1, 1, 10,100,1000, 2000))))
timeFactor <- with(timeRes,
                   tapply(1:nrow(timeRes),
                          list(NoSample),  function(x) {
                            bioqcTime <- subset(timeRes[x,], Method=="BioQC")$Time
                            rTime <- subset(timeRes[x,], Method=="NativeR")$Time
                            rTime/bioqcTime
                            }))
timeFactor.yCeiling <- max(ceiling(timeFactor/500))*500
timeFactorBar <- barchart(timeFactor ~ noSamples, horizontal=FALSE,
                          xlab="Number of samples", ylab="Ratio of elapsed time [R/BioQC]",
                          ylim=c(-20, timeFactor.yCeiling+50), col=colorRampPalette(c("lightblue", "navyblue"))(length(noSamples)),
                          scales=list(tck=c(1, 0), alternating=1L,
                                      y=list(at=seq(0,timeFactor.yCeiling, by=500)),
                                      x=list(at=seq(along=timeFactor), labels=noSamples)))

grid.arrange(timeXY, timeFactorBar, ncol=2)


## ----session_info-------------------------------------------------------------
sessionInfo()

## ----setup, include=FALSE-----------------------------------------------------
options(fig_caption=TRUE)
library(knitr)
opts_chunk$set(out.extra='style="display:block; margin: auto"', fig.align="center")

## ----lib, warning=FALSE, message=FALSE, results="hide"------------------------
library(BioQC)
library(hgu133plus2.db) ## to simulate an microarray expression dataset
library(lattice)
library(latticeExtra)
library(gridExtra)
library(gplots)

pdf.options(family="ArialMT", useDingbats=FALSE)

set.seed(1887)

## list human genes
humanGenes <- unique(na.omit(unlist(as.list(hgu133plus2SYMBOL))))

## read tissue-specific gene signatures
gmtFile <- system.file("extdata/exp.tissuemark.affy.roche.symbols.gmt",
                       package="BioQC")
gmt <- readGmt(gmtFile)
tissueInds <- sapply(gmt, function(x) match(x$genes, humanGenes))

## ----sensitivity_benchmark_fig, echo=FALSE, fig.width=8, fig.height=4.5, dev='png', fig.cap=sprintf("**Figure 1:** Sensitivity benchmark. Expression levels of genes in the ovary signature are dedicately sampled randomly from normal distributions with different mean values. Left panel: enrichment scores reported by *BioQC* for the ovary signature, plotted against the differences in mean expression values; Right panel: rank of ovary enrichment scores in all %s signatures plotted against the difference in mean expression values.", length(gmt))----

tissueInds <- sapply(gmt, function(x) match(x$genes, humanGenes))

randomMatrixButOneSignature <- function(rows=humanGenes, signatureGenes,
                                        amplitudes=seq(0, 3, by=0.5)) {
  nrow <- length(rows)
  ncol <- length(amplitudes)
  mat <- matrix(rnorm(nrow*ncol),
                nrow=nrow, byrow=FALSE)
  rownames(mat) <- rows
  sigInd <- na.omit(match(signatureGenes, humanGenes))
  
  colClass <- factor(amplitudes)
  
  for(colInd in unique(colClass)) {
    isCurrCol <- colInd==colClass
    replaceMatrix <- matrix(rnorm(length(sigInd)*sum(isCurrCol),
                                  mean=amplitudes[isCurrCol][1]),
                            nrow=length(sigInd), byrow=FALSE)
    mat[sigInd, isCurrCol] <-  replaceMatrix
    }
  return(mat)
}

selGeneSet <- "Ovary_NGS_RNASEQATLAS_0.6_3"
selSignature <- gmt[[selGeneSet]]$genes
senseAmplitudes <- rep(c(seq(0, 1, by=0.25),
                         seq(1.5, 3, 0.5)), each=10)
senseMat <- randomMatrixButOneSignature(rows=humanGenes,
                                        signatureGenes=selSignature,
                                        amplitudes=senseAmplitudes)
senseBioQC <- wmwTest(senseMat, tissueInds, valType="p.greater", simplify=TRUE)
senseRank <- apply(senseBioQC, 2, function(x) rank(x)[selGeneSet])
mydot <- function(x,y,abline=1,...) {panel.abline(h=abline, col="darkgray");panel.dotplot(x,y,...)}
senseBW <- bwplot(-log10(senseBioQC[selGeneSet,])~senseAmplitudes, horizontal=FALSE,
                  pch="|", do.out=FALSE,
                  par.settings=list(box.rectangle=list(col="black", fill="#ccddee")),
                  scales=list(tck=c(1,0), alternating=1L,
                              x=list(at=seq(along=unique(senseAmplitudes)), labels=unique(senseAmplitudes))),
                  ylab="Enrichment score", xlab="Mean expression difference")
senseDot <- dotplot(-log10(senseBioQC[selGeneSet,])~senseAmplitudes, horizontal=FALSE,
                    cex=0.9,
                    panel=mydot, abline=0,
                    scales=list(tck=c(1,0), alternating=1L,
                                x=list(at=seq(along=unique(senseAmplitudes)), labels=unique(senseAmplitudes))))

senseRankBW <- bwplot(senseRank~senseAmplitudes, horizontal=FALSE,
                      pch="|", do.out=FALSE, col="black", ylim=c(155, -5),
                      par.settings=list(box.rectangle=list(col="black", fill="#d9dddd")),
                      scales=list(tck=c(1,0), alternating=1L,
                                  y=list(at=c(1,50,100,150)),
                                  x=list(at=seq(along=unique(senseAmplitudes)), labels=unique(senseAmplitudes))),
                      ylab="Enrichment score rank", xlab="Mean expression difference")
senseRankDot <- dotplot(senseRank~senseAmplitudes, horizontal=FALSE,
                        panel=mydot, abline=1,
                        cex=0.9, col="black",ylim=c(155, -5),
                        scales=list(tck=c(1,0), alternating=1L,
                                    x=list(at=seq(along=unique(senseAmplitudes)), labels=unique(senseAmplitudes))))

sensePlot <- senseBW + senseDot
senseRankPlot <- senseRankBW + senseRankDot

grid.arrange(sensePlot, senseRankPlot, ncol=2)

## ----mixing_benchmark, echo=FALSE---------------------------------------------
dogfile <- system.file("extdata/GSE20113.rda", package = "BioQC")
# if(!file.exists(dogfile)) {
#   rawdog <- getGEO("GSE20113")[[1]]
#   filterFeatures <- function(eset) {
#     rawGeneSymbol <- fData(eset)[, "Gene Symbol"]
#     eset <- eset[!rawGeneSymbol %in% "" & !is.na(rawGeneSymbol),]
#     gs <- fData(eset)[, "Gene Symbol"]
#     
#     gsSplit <- split(1:nrow(eset), gs)
#     rmeans <- rowMeans(exprs(eset), na.rm=TRUE)
#     maxMean <- sapply(gsSplit, function(x) x[which.max(rmeans[x])])
#     maxMean <- unlist(maxMean)
#     res <- eset[maxMean,]
#     fData(res)$GeneSymbol <- fData(res)[, "Gene Symbol"]
#     return(res)
#     }
#   dog <- filterFeatures(rawdog)
#   dog$Label <- gsub("[0-9]$", "", as.character(dog$description))
#   save(dog, file=dogfile)
# } else {
# }
load(dogfile)
dogInds <- sapply(gmt, function(x) match(x$genes, fData(dog)$GeneSymbol))
dogBioQC <- wmwTest(dog, gmt, valType="p.greater", simplify=TRUE)
dogEnrich <- absLog10p(dogBioQC)
dogEnrich.best <- apply(dogEnrich,2, which.max)
dogEnrich.second <- apply(dogEnrich,2, function(x) which(rank(-x)==2))
shortLabel <- function(x) gsub("_NR_0\\.7_3|_NGS_RNASEQATLAS_0\\.6_3", "", x)
dogEnrich.bestLabels <- shortLabel(rownames(dogEnrich)[dogEnrich.best])
dogEnrich.secondLabels <- shortLabel(rownames(dogEnrich)[dogEnrich.second])
dogTable <- data.frame(Label=dog$Label,
                       BioQC.best=dogEnrich.bestLabels,
                       BioQC.second=dogEnrich.secondLabels,
                       row.names=sampleNames(dog))

## ----dog_table, echo=FALSE----------------------------------------------------
kable(dogTable, caption="Quality control of the mixing benchmark input data with *BioQC*. Four columns (f.l.t.r.): sample index; tissue         reported by the authors; the tissue signature with the highest enrichment score reported by *BioQC*; the tissue signature with the second-   highest enrichment score.")

## ----hj_mix, echo=FALSE-------------------------------------------------------
heart <- rowMeans(exprs(dog)[, dog$Label=="Heart"])
 jejunum <- rowMeans(exprs(dog)[, dog$Label=="Jejunum"])
 linearMix <- function(vec1, vec2, prop2=0) {
     stopifnot(prop2>=0 && prop2<=1)
     return(vec1*(1-prop2)+vec2*prop2)
 }
 mixProps <- seq(0, 1, 0.05)
 mixPropsLabels <- sprintf("%f%%", mixProps*100)
 hjMix <- sapply(mixProps, function(x) linearMix(jejunum, heart, x))
 hjMixBioQC <- wmwTest(hjMix, dogInds, valType="p.greater", simplify=TRUE)
 hjMixSub <- hjMixBioQC[c("Intestine_small_NR_0.7_3","Muscle_cardiac_NR_0.7_3"),]
 hjMixSubEnrich <- absLog10p(hjMixSub)
 hjMixBioQCrank <- apply(hjMixBioQC, 2, function(x) rank(x))
 hjMixSubRank <- hjMixBioQCrank[c("Intestine_small_NR_0.7_3","Muscle_cardiac_NR_0.7_3"),]
 colnames(hjMixSubEnrich) <- colnames(hjMixSubRank) <- mixPropsLabels

## ----hjMixVis, echo=FALSE, fig.width=8, fig.height=4, dev='png', fig.cap="**Figure 2:** Results of a mixing case study. Left panel: *BioQC* enrichment scores of small intestine and cardiac muscle varying upon different proportions of jejunum; Right panel: ranks of enrichment scores varying upon different proportions of jejunum."----
mixPropsShow <- seq(0, 1, 0.25)
mixPropsShowLabels <- sprintf("%d%%", mixPropsShow*100)
hjCols <- c("orange", "lightblue")
hjTissues <- c("Small intenstine","Cardiac muscle")
hjMixData <- data.frame(Tissue=rep(hjTissues, ncol(hjMixSubEnrich)),
                        Prop=rep(mixProps, each=nrow(hjMixSubEnrich)),
                        EnrichScore=as.vector(hjMixSubEnrich))
hjMixDataRank <- data.frame(Tissue=rep(hjTissues, ncol(hjMixSubEnrich)),
                            Prop=rep(mixProps, each=nrow(hjMixSubEnrich)),
                            EnrichScore=as.vector(hjMixSubRank))
hjMixXY <- xyplot(EnrichScore ~ Prop, group=Tissue, data=hjMixData, type="b",
                  xlab="Proportion of heart", ylab="Enrichment score",
                  par.settings=list(superpose.symbol=list(cex=1.25, pch=16, col=hjCols),
                                    superpose.line=list(col=hjCols)),
                  auto.key=list(columns=1L), abline=list(h=3, col="lightgray", lty=2, lwd=2),
                  scales=list(tck=c(1,0), alternating=1L,
                              x=list(at=mixPropsShow, labels=mixPropsShowLabels)))
hjMixRankXY <- xyplot(EnrichScore ~ Prop, group=Tissue, data=hjMixDataRank, type="b",
                      xlab="Proportion of heart", ylab="Enrichment score rank", ylim=c(155, 0.8),
                      auto.key=list(columns=1L),
                      par.settings=list(superpose.symbol=list(cex=1.25, pch=16, col=hjCols),
                                        superpose.line=list(col=hjCols)),
                      abline=list(h=log2(10), col="lightgray", lty=2, lwd=2),
                      scales=list(tck=c(1,0), alternating=1L,
                                  x=list(at=mixPropsShow, labels=mixPropsShowLabels),
                                  y=list(log=2, at=c(1,2,3,4,6,10,25,50,100,150))))

grid.arrange(hjMixXY, hjMixRankXY, ncol=2)

## ----dog_mix, echo=FALSE------------------------------------------------------
dogFilter <- dog[,-c(1,22,24)]
dogAvg <- tapply(1:ncol(dogFilter), dogFilter$Label, function(x) rowMeans(exprs(dogFilter)[,x]))
dogAvgMat <- do.call(cbind, dogAvg)
dogLabels <- c("Brain_Cortex_prefrontal_NR_0.7_3",
               "Muscle_cardiac_NR_0.7_3",
               "Intestine_small_NR_0.7_3",
               "Kidney_NR_0.7_3",
               "Liver_NR_0.7_3",
               "Lung_NR_0.7_3",
               "Lymphocyte_B_FOLL_NR_0.7_3",
               "Pancreas_Islets_NR_0.7_3",
               "Muscle_skeletal_NR_0.7_3",
               "Monocytes_NR_0.7_3")
dogComb <- subset(expand.grid(1:ncol(dogAvgMat),1:ncol(dogAvgMat)), Var2>Var1)
dogPairwise <- apply(dogComb, 1, function(x) {
                      vec1 <- dogAvgMat[,x[1]]
                      vec2 <- dogAvgMat[,x[2]]
                      label1 <- dogLabels[x[1]]
                      label2 <- dogLabels[x[2]]
                      mix <- sapply(mixProps, function(x) linearMix(vec1, vec2, x))
                      bioqc <- wmwTest(mix, dogInds, valType="p.greater", simplify=TRUE)
                      ranks <- apply(bioqc, 2, rank)
                      enrich <- absLog10p(bioqc)
                      colnames(enrich) <- colnames(ranks) <- mixPropsLabels
                      res <- list(EnrichScore=enrich[c(label1, label2),],
                                  Rank=ranks[c(label1, label2),])
                      return(res)
                   })
contamInd <- sapply(dogPairwise, function(x) {
                      successScore <- x$EnrichScore[2,]>=3
                      successRank <- x$Rank[2,]<=10
                      if(all(successScore) & all(successRank)) {
                        return(NA)
                        }
                      pmin(min(which(successScore)), min(which(successRank)))
                  })
revContamInd <- sapply(dogPairwise, function(x) {
                        successScore <- x$EnrichScore[1,]>=3
                        successRank <- x$Rank[1,]<=10
                        if(all(successScore) & all(successRank)) {
                          return(NA)
                          }
                        pmax(max(which(successScore)), max(which(successRank)))
                      })

fromMinProp <- mixProps[contamInd]
toMaxProp <- 1-mixProps[revContamInd]
comProp <- matrix(NA, nrow=ncol(dogAvgMat), ncol=ncol(dogAvgMat))
colnames(comProp) <- rownames(comProp) <- colnames(dogAvgMat)
for(i in 1:nrow(dogComb)) {
  comProp[dogComb[i,1], dogComb[i,2]] <- fromMinProp[i]
  comProp[dogComb[i,2], dogComb[i,1]] <- toMaxProp[i]
}


## ----dog_mix_vis, echo=FALSE, fig.width=8, fig.height=5, dev='png', fig.cap="**Figure 3:** Results of the pairwise mixing experiment. Each cell represents the minimal percentage of tissue of the column as contamination in the tissue of the row that can be detected by *BioQC*. No values are available for cells on the diagonal because self-mixing was excluded. Heart  and skeletal muscle are very close to each other and therefore their detection limit is not considered."----

dogMixCol <- colorRampPalette(c("#67A9CF", "black", "#EF8A62"))(100)
heatmap.2(x=comProp, Rowv=NULL,Colv=NULL, 
          col = dogMixCol, 
          scale="none",
          margins=c(9,9), # ("margin.Y", "margin.X")
          trace='none', 
          symkey=FALSE, 
          symbreaks=FALSE, 
          breaks=seq(0,1,0.01),
          dendrogram='none',
          density.info='none', 
          denscol="black",
          na.col="gray",
          offsetRow=0, offsetCol=0,
          keysize=1, key.title="Enrichment score", key.xlab="Detection limit", 
          xlab="Contamination tissue", ylab="Originating tissue",
          #( "bottom.margin", "left.margin", "top.margin", "right.margin" )
          key.par=list(mar=c(5,0,1,8)),
          # lmat -- added 2 lattice sections (5 and 6) for padding
          lmat=rbind(c(5, 4, 2), c(6, 1, 3)), lhei=c(1.6, 5), lwid=c(1, 4, 1))


## ----dog_mix_vis_example, echo=FALSE------------------------------------------
cell12 <- comProp[1,2]
cell21 <- comProp[2,1]
cell12Per <- as.integer(cell12*100)
cell21Per <- as.integer(cell21*100)

## ----table_detect_limit, echo=FALSE, tab.cap="Median lower detection limites of tissues as contamination sources."----
meanThr <- colMeans(comProp, na.rm=TRUE)
meanThrShow <- data.frame(Tissue=names(meanThr), MedianDetectionLimit=sprintf("%1.2f%%", meanThr*100))
kable(meanThrShow, caption="Median lower detection limites of tissues as contamination sources.")

## ----brain_low_exp, echo=FALSE, fig.width=6, fig.height=3, dev='png', fig.cap="**Figure 4:** Average expression of tissue-preferential genes in respective tissues. For each tissue (*e.g.* brain), we calculate the median ratio of gene expression level of specific genes over the median expression level of background genes. The value reflects the specificity of tissue-specific genes in respective tissues. Likely due to the sampling of different brain regions, the score of brain ranks the lowest."----
dogAvgGs <- c("Brain_Cortex_prefrontal_NR_0.7_3",
              "Muscle_cardiac_NR_0.7_3",
              "Intestine_small_NR_0.7_3",
              "Kidney_NR_0.7_3",
              "Liver_NR_0.7_3",
              "Lung_NR_0.7_3",
              "Lymphocyte_B_FOLL_NR_0.7_3",
              "Pancreas_Islets_NR_0.7_3",
              "Muscle_skeletal_NR_0.7_3",
              "Monocytes_NR_0.7_3")
dogAvgGsGenes <- sapply(gmt[dogAvgGs], function(x) x$genes)
dogAvgGsGeneInds <- lapply(dogAvgGsGenes, function(x) {
  inds <- match(x, fData(dog)$GeneSymbol)
  inds[!is.na(inds)]
  })

## boxplot of tissue-specific genes
dogAvgGsRel <- sapply(seq(along=dogAvgGsGeneInds), function(i) {
  ind <- dogAvgGsGeneInds[[i]]
  bgInd <- setdiff(1:nrow(dog), ind)
  apply(dogAvgMat, 2, function(x) median(x[ind]/median(x[bgInd])))
  })
colnames(dogAvgGsRel) <- colnames(dogAvgMat)
##heatmap.2(log2(dogAvgGsRel),zlim=c(-1,1),
##          cellnote=round(dogAvgGsRel,1),
##          Rowv=NA, Colv=NA, dendrogram="none",  col=greenred,
##          lwid=c(1,3), lhei=c(1,3),
##          key.title="Enrichment score", key.xlab="Detection limit",
##          xlab="Tissue signature", ylab="Tissue profiles",
##          key.par=list(cex.lab=1.25, mar=c(4,2,1,0), mgp=c(1.5,0.5,0)),
##          cexRow=1.25, margins=c(8,8), trace="none", offsetRow=0, offsetCol=0,
##          density.info="none",na.col="gray", breaks=seq(-1,1,0.01), symbreaks=TRUE)

op <- par(mar=c(8,4,1,1))
barplot(sort(diag(dogAvgGsRel)), las=3, ylab="Median ratio of expression (signature/background)")
abline(h=1.25)
## op <- par(mar=c(8,4,1,1))
## dogAvgES <- absLog10p(diag(wmwTest(dogAvgMat, dogAvgGsGeneInds, alternative="greater")))
## names(dogAvgES) <- colnames(dogAvgMat)
## barplot(sort(dogAvgES, decreasing=TRUE), las=3, ylab="ES in respective average tissue profile")
## par(op)
## plot(meanThr, dogAvgES, type="n", xlab="Average lower detection limit", ylab="BioQC ES score")
## text(meanThr, dogAvgES, colnames(dogAvgMat))


## ----pca, echo=FALSE, fig.width=6, fig.height=6, dev='png', fig.cap="Sample separation revealed by principal component analysis (PCA) of the dog transcriptome dataset."----
par(mar=c(4,4,2,2))
dogEXP <- exprs(dog); colnames(dogEXP) <- dog$Label
dogPCA <- prcomp(t(dogEXP))
expVar <- function(pcaRes, n) {vars <- pcaRes$sdev^2; (vars/sum(vars))[n]}
biplot(dogPCA, col=c("#335555dd", "transparent"), cex=1.15,
       xlab=sprintf("Principal component 1 (%1.2f%%)", expVar(dogPCA,1)*100),
       ylab=sprintf("Principal component 1 (%1.2f%%)", expVar(dogPCA,2)*100))


## ----session_info-------------------------------------------------------------
sessionInfo()

#' Getting leading-edge indices from a vector
#' @param x A numeric vector (\code{getLeadingEdgeIndexFromVector}) or a numeric matrix (\code{getLeadingEdgeIndexFromMatrix}).
#' @param index An integer vector, indicating the indices of genes in a gene-set.
#' @param comparison Character string, are values greater than or less than the reference value considered as leading-edge? This depends on the type of value requested by the user in \code{wmwTest}.
#' @param reference Character string, which reference is used? If \code{background}, genes with expression higher than the median of the background are reported. Otherwise in the case of \code{geneset}, genes with expression higher than the median of the gene-set is reported. Default is \code{background}, which is consistent with the results of the Wilcoxon-Mann-Whitney tests.
#' 
#' @return An integer vector, indicating the indices of leading-edge genes.
#' @seealso \code{\link{wmwTest}}
#' @examples
#' myProfile <- c(rnorm(5, 3), rnorm(15, -3), rnorm(100, 0))
#' getLeadingEdgeIndexFromVector(myProfile, 1:20)
#' getLeadingEdgeIndexFromVector(myProfile, 1:20, comparison="less")
#' getLeadingEdgeIndexFromVector(myProfile, 1:20, comparison="less", reference="geneset")
#' myProfile2 <- c(rnorm(15, 3), rnorm(5, -3), rnorm(100, 0))
#' myProfileMat <- cbind(myProfile, myProfile2)
#' getLeadingEdgeIndexFromMatrix(myProfileMat, 1:20)
#' getLeadingEdgeIndexFromMatrix(myProfileMat, 1:20, comparison="less")
#' getLeadingEdgeIndexFromMatrix(myProfileMat, 1:20, comparison="less", reference="geneset")
#' @export
getLeadingEdgeIndexFromVector <- function(x, index,
                                          comparison = c("greater", "less"),
                                          reference = c("background", "geneset")) {
  indexValues <- x[index]
  reference <- match.arg(reference)
  comparison <- match.arg(comparison)
  ref <- switch(reference,
                geneset = median(indexValues, na.rm=TRUE),
                background = median(x, na.rm=TRUE))
  isLeading <- switch(comparison,
                      greater = indexValues >= ref,
                      less = indexValues <= ref)
  res <- index[isLeading]
  return(res)
}

#' @describeIn getLeadingEdgeIndexFromVector \code{x} is a \code{matrix}.
#' @export
getLeadingEdgeIndexFromMatrix <- function(x, index, 
                                          comparison = c("greater", "less"),
                                          reference = c("background", "geneset")) {
  reference <- match.arg(reference)
  comparison <- match.arg(comparison)
  res <- apply(x, 2, getLeadingEdgeIndexFromVector, index=index,
        reference=reference, comparison=comparison)
  if(is.matrix(res)) {
    resList <- lapply(seq_len(ncol(res)), function(i) res[,i,drop=TRUE])
    names(resList) <- colnames(res)
    res <- resList
  }
  return(res)
}


#' Identify BioQC leading-edge genes of one gene-set
#' 
#' @param matrix A numeric matrix
#' @param indexVector An integer vector, giving indices of a gene-set of interest
#' @param valType Value type, consistent with the types in \code{wmwTest}
#' @param thr Threshold of the value, greater or less than which the gene-set is considered significantly enriched in one sample
#' @param reference Character string, which reference is used? If \code{background}, genes with expression higher than the median of the background are reported. Otherwise in the case of \code{geneset}, genes with expression higher than the median of the gene-set is reported. Default is \code{background}, which is consistent with the results of the Wilcoxon-Mann-Whitney tests.
#' @return A list of integer vectors.
#' 
#' BioQC leading-edge genes are defined as those features whose expression is higher than the median expression of the background in a sample.  The function identifies leading-edge genes of a given dataset (specified by the index vector) in a number of samples (specified by the matrix, with genes/features in rows and samples in columns) in three steps. The function calls \code{wmwTest} to run BioQC and identify samples in which the gene-set is significantly enriched. The enrichment criteria is specified by \code{valType} and \code{thr}. Then the function identifies genes in the gene-set that have greater or less expresion than the median value of the \code{reference} in those samples showing significant enrichment. Finally, it reports either leading-edge genes in individual samples, or the intersection/union of leading-edge genes in multiple samples.
#' 
#' @seealso \code{\link{wmwTest}}
#' @importFrom stats median
#' @examples
#' myProfile <- c(rnorm(5, 3), rnorm(15, -3), rnorm(100, 0))
#' myProfile2 <- c(rnorm(15, 3), rnorm(5, -3), rnorm(100, 0))
#' myProfile3 <- c(rnorm(10, 5), rnorm(10, 0), rnorm(100, 0))
#' myProfileMat <- cbind(myProfile, myProfile2, myProfile3)
#' wmwLeadingEdge(myProfileMat, 1:20, valType="p.greater")
#' wmwLeadingEdge(myProfileMat, 1:20, valType="log10p.less")
#' wmwLeadingEdge(myProfileMat, 1:20, valType="U", reference="geneset")
#' wmwLeadingEdge(myProfileMat, 1:20, valType="abs.log10p.greater")
#' @export
wmwLeadingEdge <- function(matrix, 
                               indexVector, 
                               valType = c("p.greater", "p.less", "p.two.sided", "U", "abs.log10p.greater",
                                           "log10p.less", "abs.log10p.two.sided", "Q", "r", "f", "U1", "U2"),
                               thr = 0.05, 
                               reference = c("background", "geneset")) {
  if (missing(valType)) {
    valType <- "p.greater"
  } else {
    valType <- match.arg(valType)
  }
  reference <- match.arg(reference)

  lowerIsSig <- grepl("^p\\.", valType) || valType %in% c("log10p.less")
  higherIsSig <- grepl("^abs\\.", valType) || valType %in% c("U", "Q", "r", "f", "U1", "U2")
  if (!lowerIsSig & !higherIsSig) {
    stop(paste("Not implemented: is lower or higher values of", valType, "significant?"))
  }
  
  higherIsLeading <- grepl("greater", valType) || valType %in% c("U", "Q", "r", "f", "U1", "U2")
  lowerIsLeading <- grepl("less", valType)
  if (!higherIsLeading & !lowerIsLeading) {
    stop(paste("Not implemented: leading-edge genes are not implemented for ", valType, "yet"))
  } else {
    stopifnot(!(higherIsLeading & lowerIsLeading))
    compType <- ifelse(higherIsLeading, "greater", "less")
  }
  wmwTestRes <- wmwTest(matrix, indexVector, 
                                valType=valType, simplify=TRUE)
  if(lowerIsSig) {
    isSig <- wmwTestRes <= thr
  } else {
    isSig <- wmwTestRes >= thr
  }
  if(!any(isSig)) {
    warning("No single sample reported significant results. Please make sure the threshold is set correcly.")
    return(NULL)
  }
  sigMatrix <- matrix[, isSig, drop=FALSE]

  resList <- getLeadingEdgeIndexFromMatrix(sigMatrix, index=indexVector,
                                           reference=reference, 
                                           comparison=compType)
  return(resList)
}

#' Convert gmtlist into a list of signed genesets
#'
#' @param gmtlist A gmtlist object, probably read-in by \code{readGmt}
#' @param posPattern Regular expression pattern of positive gene sets. It is trimmed from the original name to get the stem name of the gene set. See examples below.
#' @param negPattern Regular expression pattern of negative gene sets. It is trimmed from the original name to get the stem name of the gene set. See examples below.
#' @param nomatch Options to deal with gene sets that match neither positive nor negative patterns. ignore: they will be ignored (but not discarded, see details below); pos: they will be counted as positive signs; neg: they will be counted as negative signs
#' @return An S4-object of \code{SignedGenesets}, which is a list of signed_geneset, each being a two-item list; the first item is 'pos', containing a character vector of positive genes; and the second item is 'neg', containing a character vector of negative genes.
#'
#' Gene set names are detected whether they are positive or negative. If neither positive nor negative, nomatch will determine how will they be interpreted. In case of \code{pos} (or \code{neg}), such genesets will be treated as positive (or negative) gene sets.In case nomatch is set to \code{ignore}, the gene set will appear in the returned values with both positive and negative sets set to \code{NULL}.
#' 
#' @examples
#' testInputList <- list(list(name="GeneSetA_UP",genes=LETTERS[1:3]),
#'                list(name="GeneSetA_DN", genes=LETTERS[4:6]),
#'                list(name="GeneSetB", genes=LETTERS[2:4]),
#'                list(name="GeneSetC_DN", genes=LETTERS[1:3]),
#'                list(name="GeneSetD_UP", genes=LETTERS[1:3]))
#' testOutputList.ignore <- gmtlist2signedGenesets(testInputList, nomatch="ignore")
#' testOutputList.pos <- gmtlist2signedGenesets(testInputList, nomatch="pos")
#' testOutputList.neg <- gmtlist2signedGenesets(testInputList, nomatch="neg")
#' @export
gmtlist2signedGenesets <- function(gmtlist, posPattern="_UP$", negPattern="_DN$",
                                   nomatch=c("ignore", "pos", "neg")) {
  nomatch <- match.arg(nomatch)
  genes <- gsGenes(gmtlist)
  names <- gsName(gmtlist)
  hasNS <- hasNamespace(gmtlist)
  namespace <- gsNamespace(gmtlist)
  isPos <- grepl(posPattern, names)
  isNeg <- grepl(negPattern, names)
  stemNames <- gsub(negPattern, "", gsub(posPattern, "", names))
  stemFactor <- factor(stemNames, levels=unique(stemNames))
  res <- tapply(seq(along=gmtlist), stemFactor, function(i) {
    pos <- NULL
    neg <- NULL
    if(hasNS) {
      ns <- unique(namespace[i])
      if(length(ns)>1) {
        stop("Signed gene-sets found in multiple namespaces - please report the bug to the developer")
      }
    } else {
      ns <- NULL 
    }
    if(!any(isPos[i]) && !any(isNeg[i])) {
      otherGenes <- unique(unlist(genes[i]))
      if (nomatch=="pos") {
        pos <- otherGenes
      } else if (nomatch=="neg") {
        neg <- otherGenes
      } else if (nomatch!="ignore") {
        stop("should not be here")
      }
    } else {
      if(any(isPos[i])) {
        pos <- setdiff(unique(unlist(genes[i][isPos[i]])), "")
      }
      if(any(isNeg[i])) {
        neg <- setdiff(unique(unlist(genes[i][isNeg[i]])), "")
      }
    }
    return(list(name=as.character(stemFactor)[i][1], 
                pos=pos, 
                neg=neg,
                namespace=ns))
  })
  names(res) <- levels(stemFactor)
  return(SignedGenesets(res))
}

#' Read signed GMT files
#'
#' @param filename A gmt file
#' @param posPattern Pattern of positive gene sets
#' @param negPattern Pattern of negative gene sets
#' @param nomatch options to deal with gene sets that match to neither posPattern nor negPattern patterns
#' @param uniqGenes Logical, whether genes should be made unique
#' @param namespace Character string or \code{NULL}, namespace of gene-sets
#' 
#' @seealso \code{\link{gmtlist2signedGenesets}} for parameters \code{posPattern}, \code{negPattern}, and \code{nomatch}
#' @examples
#' testGmtFile <- system.file("extdata/test.gmt", package="BioQC")
#' testSignedGenesets.ignore <- readSignedGmt(testGmtFile, nomatch="ignore")
#' testSignedGenesets.pos <- readSignedGmt(testGmtFile, nomatch="pos")
#' testSignedGenesets.neg <- readSignedGmt(testGmtFile, nomatch="neg")
#' @export
readSignedGmt <- function(filename, posPattern="_UP$", negPattern="_DN$",
                          nomatch=c("ignore", "pos", "neg"),
                          uniqGenes=TRUE, 
                          namespace=NULL) {
  gmt <- readGmt(filename, uniqGenes=uniqGenes, namespace=namespace)
  res <- gmtlist2signedGenesets(gmt, posPattern=posPattern, negPattern=negPattern, nomatch=nomatch)
  return(res)
  
}

library(BioQC)

## readGmt
context("Read gmt file into a GmtList object")
testFile <- system.file("extdata/test.gmt", package="BioQC")
testGmt <- readGmt(testFile)
expGmt <- list(list(name="GS_A", desc="TestGeneSet_A", genes=c("AKT1", "AKT2", "AKT3"), namespace=NULL),
               list(name="GS_B", desc="TestGeneSet_B", genes=c("MAPK1", "MAPK3", "MAPK8"), namespace=NULL),
               list(name="GS_C_UP", desc="TestGeneSet_C", genes=c("ERBB2", "ERBB3"), namespace=NULL),
               list(name="GS_C_DN", desc="TestGeneSet_C", genes=c("EGFR", "ERBB4"), namespace=NULL),
               list(name="GS_D_UP", desc="TestGeneSet_D", genes=c("GATA2", "GATA4"), namespace=NULL),
               list(name="GS_D_DN", desc="TestGeneSet_D", genes=c("GATA1", "GATA3"), namespace=NULL),
               list(name="GS_E_DN", desc="TestGeneSet_E", genes=c("TSC1", "TSC2"), namespace=NULL))
test_that("readGmt reads one GMT file without namespace",{
             expect_equivalent(expGmt, testGmt)
         })

context("Read gmt file into a GmtList object with namespace")
testFile <- system.file("extdata/test.gmt", package="BioQC")
testGmt <- readGmt(testFile, namespace="BioQC")
expGmt <- list(list(name="GS_A", desc="TestGeneSet_A", genes=c("AKT1", "AKT2", "AKT3"), namespace="BioQC"),
               list(name="GS_B", desc="TestGeneSet_B", genes=c("MAPK1", "MAPK3", "MAPK8"), namespace="BioQC"),
               list(name="GS_C_UP", desc="TestGeneSet_C", genes=c("ERBB2", "ERBB3"), namespace="BioQC"),
               list(name="GS_C_DN", desc="TestGeneSet_C", genes=c("EGFR", "ERBB4"), namespace="BioQC"),
               list(name="GS_D_UP", desc="TestGeneSet_D", genes=c("GATA2", "GATA4"), namespace="BioQC"),
               list(name="GS_D_DN", desc="TestGeneSet_D", genes=c("GATA1", "GATA3"), namespace="BioQC"),
               list(name="GS_E_DN", desc="TestGeneSet_E", genes=c("TSC1", "TSC2"), namespace="BioQC"))
test_that("readGmt reads one GMT file with namespace",{
  expect_equivalent(expGmt, testGmt)
})

context("Read two gmt files into a GmtList object with namespaces")
testFile <- system.file("extdata/test.gmt", package="BioQC")
testGmtNamed <- readGmt(ns1=testFile, ns2=testFile)
testGmtUnnamed <- readGmt(testFile, testFile, namespace=c("ns1", "ns2"))
expGmtTwoNs <- list(list(name="GS_A", desc="TestGeneSet_A", genes=c("AKT1", "AKT2", "AKT3"), namespace="ns1"),
               list(name="GS_B", desc="TestGeneSet_B", genes=c("MAPK1", "MAPK3", "MAPK8"), namespace="ns1"),
               list(name="GS_C_UP", desc="TestGeneSet_C", genes=c("ERBB2", "ERBB3"), namespace="ns1"),
               list(name="GS_C_DN", desc="TestGeneSet_C", genes=c("EGFR", "ERBB4"), namespace="ns1"),
               list(name="GS_D_UP", desc="TestGeneSet_D", genes=c("GATA2", "GATA4"), namespace="ns1"),
               list(name="GS_D_DN", desc="TestGeneSet_D", genes=c("GATA1", "GATA3"), namespace="ns1"),
               list(name="GS_E_DN", desc="TestGeneSet_E", genes=c("TSC1", "TSC2"), namespace="ns1"),
               list(name="GS_A", desc="TestGeneSet_A", genes=c("AKT1", "AKT2", "AKT3"), namespace="ns2"),
               list(name="GS_B", desc="TestGeneSet_B", genes=c("MAPK1", "MAPK3", "MAPK8"), namespace="ns2"),
               list(name="GS_C_UP", desc="TestGeneSet_C", genes=c("ERBB2", "ERBB3"), namespace="ns2"),
               list(name="GS_C_DN", desc="TestGeneSet_C", genes=c("EGFR", "ERBB4"), namespace="ns2"),
               list(name="GS_D_UP", desc="TestGeneSet_D", genes=c("GATA2", "GATA4"), namespace="ns2"),
               list(name="GS_D_DN", desc="TestGeneSet_D", genes=c("GATA1", "GATA3"), namespace="ns2"),
               list(name="GS_E_DN", desc="TestGeneSet_E", genes=c("TSC1", "TSC2"), namespace="ns2"))
test_that("readGmt reads two GMT files with namespace",{
  expect_equivalent(expGmtTwoNs, testGmtNamed)
  expect_equivalent(expGmtTwoNs, testGmtUnnamed)
})


## appendGmtList
context("appendGmtList works as expected")
testFile <- system.file("extdata/test.gmt", package="BioQC")
testGmt1 <- readGmt(testFile, namespace="ns1")
testGmt2 <- readGmt(testFile, namespace = "ns2")
testGmt3 <- readGmt(testFile, namespace = "ns3")
testGmtAppended <- appendGmtList(testGmt1, testGmt2, testGmt3)
testGmtAppendedExp <- readGmt(ns1=testFile, ns2=testFile, ns3=testFile)
test_that("appendGmtList works as expected for three GmtList objects",{
  expect_equivalent(testGmtAppended, testGmtAppendedExp)
})

context("Read gmt file into a SignedGenesets object")
testSignedGenesets <- readSignedGmt(testFile, nomatch="pos")
expSignedGenesets <- list(list(name="GS_A", pos=c("AKT1", "AKT2", "AKT3"), neg=NULL, namespace=NULL),
                          list(name="GS_B", pos=c("MAPK1", "MAPK3", "MAPK8"), neg=NULL, namespace=NULL),
                          list(name="GS_C", pos=c("ERBB2", "ERBB3"), neg=c("EGFR", "ERBB4"), namespace=NULL),
                          list(name="GS_D", pos=c("GATA2", "GATA4"), neg=c("GATA1", "GATA3"), namespace=NULL),
                          list(name="GS_E", pos=NULL, neg=c("TSC1", "TSC2"), namespace=NULL))
test_that("readSignedGmt", {
              expect_equal(expSignedGenesets, testSignedGenesets@.Data)
          })

## as.GmtList
context("Convert a list of gene symbols into a gmtlist object")
testVec <- list(GeneSet1=c("AKT1", "AKT2"),
                GeneSet2=c("MAPK1", "MAPK3"),
                GeneSet3=NULL)
testVecGmtlist <- as.GmtList(testVec)
expVecGmtlist <- list(GeneSet1=list(name="GeneSet1", desc=NULL, genes=c("AKT1", "AKT2"), namespace=NULL),
                      GeneSet2=list(name="GeneSet2", desc=NULL, genes=c("MAPK1", "MAPK3"), namespace=NULL),
                      GeneSet3=list(name="GeneSet3", desc=NULL, genes=NULL, namespace=NULL))

testVecGmtlist.desc <- as.GmtList(testVec, desc=c("GS1", "GS2", "GS3"))
expVecGmtlist.desc <- list(GeneSet1=list(name="GeneSet1", desc="GS1", genes=c("AKT1", "AKT2"), namespace=NULL),
                      GeneSet2=list(name="GeneSet2", desc="GS2", genes=c("MAPK1", "MAPK3"), namespace=NULL),
                      GeneSet3=list(name="GeneSet3", desc="GS3", genes=NULL, namespace=NULL))
test_that("as.GmtList",{
              expect_equivalent(testVecGmtlist, expVecGmtlist)
              expect_equivalent(testVecGmtlist.desc, expVecGmtlist.desc)
         })

## gmtlist2signedGenesets
context("Convert gmtlist (a memory-copy of a GMT file) to a list of signed gene sets")

testInputList <- list(list(name="GeneSetA_UP",genes=LETTERS[1:3], namespace=NULL),
                      list(name="GeneSetA_DN", genes=LETTERS[4:6], namespace=NULL),
                      list(name="GeneSetB", genes=LETTERS[2:4], namespace=NULL),
                      list(name="GeneSetC_DN", genes=LETTERS[1:3], namespace=NULL),
                      list(name="GeneSetD_UP", genes=LETTERS[1:3], namespace=NULL))
outList.ignore <- gmtlist2signedGenesets(testInputList, nomatch="ignore")
outList.pos <- gmtlist2signedGenesets(testInputList, nomatch="pos")
outList.neg <- gmtlist2signedGenesets(testInputList, nomatch="neg")

exp.ignore <- list("GeneSetA"=list(name="GeneSetA", pos=LETTERS[1:3], neg=LETTERS[4:6], namespace=NULL),
                   "GeneSetB"=list(name="GeneSetB", pos=NULL, neg=NULL, namespace=NULL),
                   "GeneSetC"=list(name="GeneSetC", pos=NULL, neg=LETTERS[1:3], namespace=NULL),
                   "GeneSetD"=list(name="GeneSetD", pos=LETTERS[1:3], neg=NULL, namespace=NULL))

exp.pos <- list("GeneSetA"=list(name="GeneSetA", pos=LETTERS[1:3], neg=LETTERS[4:6], namespace=NULL),
                "GeneSetB"=list(name="GeneSetB", pos=LETTERS[2:4], neg=NULL, namespace=NULL),
                "GeneSetC"=list(name="GeneSetC", pos=NULL, neg=LETTERS[1:3], namespace=NULL),
                "GeneSetD"=list(name="GeneSetD", pos=LETTERS[1:3], neg=NULL, namespace=NULL))

exp.neg <- list("GeneSetA"=list(name="GeneSetA", pos=LETTERS[1:3], neg=LETTERS[4:6], namespace=NULL),
                "GeneSetB"=list(name="GeneSetB",pos=NULL, neg=LETTERS[2:4], namespace=NULL),
                "GeneSetC"=list(name="GeneSetC", pos=NULL, neg=LETTERS[1:3], namespace=NULL),
                "GeneSetD"=list(name="GeneSetD", pos=LETTERS[1:3], neg=NULL, namespace=NULL))

test_that("gmtlist2signedGenesets, ignore non-matching genesets", {
              expect_equivalent(outList.ignore, exp.ignore)
          })

test_that("gmtlist2signedGenesets, ignore non-matching as positive", {
             expect_equivalent(outList.pos, exp.pos)
         })
test_that("gmtlist2signedGenesets, ignore non-matching as negative", {
             expect_equivalent(outList.neg, exp.neg)
         })

## readSignedGmt
context("Read in gmt file into a signed_genesets object")
testSignedGenesets.ignore <- readSignedGmt(testFile, nomatch="ignore")
testSignedGenesets.pos <- readSignedGmt(testFile, nomatch="pos")
testSignedGenesets.neg <- readSignedGmt(testFile, nomatch="neg")

expSignedGenesets.ignore <- list(GS_A=list(name="GS_A", pos=NULL, neg=NULL, namespace=NULL),
                                 GS_B=list(name="GS_B", pos=NULL, neg=NULL, namespace=NULL),
                                 GS_C=list(name="GS_C", pos=c("ERBB2", "ERBB3"), neg=c("EGFR", "ERBB4"), namespace=NULL),
                                 GS_D=list(name="GS_D", pos=c("GATA2", "GATA4"), neg=c("GATA1", "GATA3"), namespace=NULL),
                                 GS_E=list(name="GS_E", pos=NULL, neg=c("TSC1", "TSC2"), namespace=NULL))
expSignedGenesets.pos <- list(GS_A=list(name="GS_A", pos=c("AKT1", "AKT2", "AKT3"), neg=NULL, namespace=NULL),
                              GS_B=list(name="GS_B", pos=c("MAPK1", "MAPK3", "MAPK8"), neg=NULL, namespace=NULL),
                              GS_C=list(name="GS_C", pos=c("ERBB2", "ERBB3"), neg=c("EGFR", "ERBB4"), namespace=NULL),
                              GS_D=list(name="GS_D", pos=c("GATA2", "GATA4"), neg=c("GATA1", "GATA3"), namespace=NULL),
                              GS_E=list(name="GS_E", pos=NULL, neg=c("TSC1", "TSC2"), namespace=NULL))
expSignedGenesets.neg <- list(GS_A=list(name="GS_A", pos=NULL, neg=c("AKT1", "AKT2", "AKT3"), namespace=NULL),
                              GS_B=list(name="GS_B", pos=NULL, neg=c("MAPK1", "MAPK3", "MAPK8"), namespace=NULL),
                              GS_C=list(name="GS_C", pos=c("ERBB2", "ERBB3"), neg=c("EGFR", "ERBB4"), namespace=NULL),
                              GS_D=list(name="GS_D", pos=c("GATA2", "GATA4"), neg=c("GATA1", "GATA3"), namespace=NULL),
                              GS_E=list(name="GS_E", pos=NULL, neg=c("TSC1", "TSC2"), namespace=NULL))
test_that("readSignedGmt, nomatch ingore",{
              expect_equivalent(testSignedGenesets.ignore, expSignedGenesets.ignore)
          })
test_that("readSignedGmt, nomatch pos",{
              expect_equivalent(testSignedGenesets.pos, expSignedGenesets.pos)
          })
test_that("readSignedGmt, nomatch neg",{
              expect_equivalent(testSignedGenesets.neg, expSignedGenesets.neg)
          })

#' Make names of gene-sets unique by namespace, and member genes of gene-sets unique
#' 
#' @param gmtList A \code{GmtList} object, probably from \code{\link[BioQC]{readGmt}}. The object must have namespaces defined by \code{setNamespace}.
#' 
#' The function make sure that
#' \itemize{
#'   \item names of gene-sets within each namespace are unique, by merging gene-sets with duplicated names
#'   \item genes within each gene-set are unique, by removing duplicated genes
#' }
#' 
#' Gene-sets with duplicated names and different \code{desc} are merged, \code{desc} are made unique, and in case of multiple values, concatenated (with \code{|} as the collapse character).
#' 
#' @return A \code{GmtList} object, with unique gene-sets and unique gene lists. If not already present, a new item \code{namespace} is appended to each \code{list} element in the \code{GmtList} object, recording the namespace used to make gene-sets unique. The order of the returned \code{GmtList} object is given by the unique gene-set name of the input object.
#' 
#' @importFrom methods is
#' 
#' @examples 
#' myGmtList <- GmtList(list(list(name="GeneSet1", desc="Namespace1", genes=LETTERS[1:3]),
#'   list(name="GeneSet2", desc="Namespace1", genes=rep(LETTERS[4:6],2)),
#'   list(name="GeneSet1", desc="Namespace1", genes=LETTERS[4:6]),
#'   list(name="GeneSet3", desc="Namespace2", genes=LETTERS[1:5])))
#'  
#' print(myGmtList)
#' myGmtList <- setNamespace(myGmtList, namespace=function(x) x$desc)
#' myUniqGmtList <- uniqGenesetsByNamespace(myGmtList)
#' print(myUniqGmtList)
#' 
#' @export
uniqGenesetsByNamespace <- function(gmtList) {
  stopifnot(is(gmtList, "GmtList"))
  if(!hasNamespace(gmtList)) {
    stop(paste0("Gene-set namespace is not defined. ",
                "Please run 'setNamespace' or 'setDescAsNamespace' first to define namespace."))
  }
  
  inputNamespace <- gsNamespace(gmtList)
  cg <- factor(inputNamespace)
  
  inputNames <- gsName(gmtList)
  inputDescs <- gsDesc(gmtList)
  geneList <- gsGenes(gmtList)
  gsSize <- sapply(geneList, length)
  cgFac <- rep(cg, gsSize)
  tbl <- data.frame(namespace=cgFac,
                    name=rep(gsName(gmtList), gsSize),
                    genes=as.character(unlist(geneList)),
                    stringsAsFactors = FALSE)
  nameFactor <- factor(tbl$name)
  tbl$nameFac <- as.integer(nameFactor)
  tbl$cgFac <- as.integer(cgFac)
  tbl <- unique(tbl) ## unique combination of namespace, name, genes
  splitGenes <- with(tbl,
                     split(genes,
                           list(cgFac, nameFac), 
                           drop=TRUE))
  resList <- lapply(seq(along=splitGenes),
                    function(i) {
                      genes <- splitGenes[[i]]
                      intName <- names(splitGenes)[i]
                      intNameSplit <- strsplit(intName, "\\.")[[1]]
                      cgLevel <- as.integer(intNameSplit[1])
                      nameLevel <- as.integer(intNameSplit[2])
                      resNamespace <- levels(cg)[cgLevel]
                      resName <- levels(nameFactor)[nameLevel]
                      matchingDesc <- inputDescs[inputNames==resName &
                                                   inputNamespace==resNamespace]
                      resDesc <- paste(unique(matchingDesc),
                                       collapse="|")
                      res <- list(name=resName, 
                                  desc=resDesc, 
                                  genes=genes,
                                  namespace=resNamespace)
                      return(res)
                    })
  names(resList) <- sapply(resList, function(x) x$name)
  oResList <- resList[unique(inputNames)]
  res <- GmtList(oResList)
  return(res)
}

#'Calculate Gini Index of a numeric vector
#'
#'Calculate the Gini index of a numeric vector.
#'
#'@param x A numeric vector.
#'
#'@details The Gini index (Gini coefficient) is a measure of statistical dispersion. A Gini
#'coefficient of zero expresses perfect equality where all values are
#'the same. A Gini coefficient of one expresses maximal inequality among values.
#'
#'@return A numeric value between 0 and 1.
#'
#'@references Gini. C. (1912) \emph{Variability and Mutability}, C. Cuppini, Bologna
#'156 pages.
#'
#'@author Jitao David Zhang <jitao_david.zhang@roche.com>
#'
#'@examples 
#'testValues <- runif(100)
#'gini(testValues)
#'@export
gini <- function(x)  {
    storage.mode(x) <- "double"
    hasNeg <- any(x<0)
    if(!is.na(hasNeg) & hasNeg)
      stop("Gini index is only applicable to non-negative values!")
    isVec <- !is.matrix(x)
    if(isVec) {
        x <- x[!is.na(x)]
        x <- sort(x, decreasing=FALSE)
        res <- .Call(C_gini_numeric, x, length(x))
    } else {
        res <- .Call(C_gini_matrix, x, nrow(x), ncol(x))
    }
    return(res)
}

library(ineq)

vec1 <- c(2.0, 3.0, 4.0, 5.0)
vec2 <- rep(2.0, 4)
vec3 <- c(4.0, 5.0, 2.0, 3.0, NA, 4.4)
vec4 <- c(1L, 2L, 4L, 3L, 5L, NA)
vec5 <- c(0, 1, 2, -1, 3, 4, NA)

context("Test gini for vectors")

test_that("BioQC::gini works as well as ineq::Gini",
          expect_equal(ineq::Gini(vec1),
                       BioQC::gini(vec1)))

test_that("BioQC::gini works as well as ineq::Gini for all-equal cases",
          expect_equal(ineq::Gini(vec2),
                       BioQC::gini(vec2)))

test_that("BioQC::gini works well for unsorted numeric vectors with NA values",
          expect_equal(ineq::Gini(vec3[!is.na(vec3)]),
                       BioQC::gini(vec3)))

test_that("BioQC::gini works well for unsorted integer vectors with NA values",
          expect_equal(ineq::Gini(vec4[!is.na(vec4)]),
                       BioQC::gini(vec4)))

test_that("BioQC::gini reports error if there are negative values",
          expect_error(BioQC::gini(vec5)))

context("Test gini for matrix")

testMat <- rbind(vec1, vec2)
testMatGini <- gini(testMat)
testMatExpGini <- c(ineq::Gini(vec1),
                    ineq::Gini(vec2))
test_that("BioQC::gini works with Matrix",
          expect_equal(testMatGini, testMatExpGini))

context("Test gini for large matrix")
set.seed(1887)
testMat2 <- abs(matrix(rnorm(1000), nrow=10))
testMat2Gini <- gini(testMat2)
testMat2ExpGini <- apply(testMat2, 1, ineq::Gini)
test_that("BioQC::gini works with large matrix",
          expect_equal(testMat2Gini, testMat2ExpGini))

#' @useDynLib BioQC, .registration=TRUE, .fixes="C_"
#' @importFrom Rcpp sourceCpp

## FOR REFERENCE rankSumTestWithCorrelation function from the limma package (version 3.18.13)

## authors: Gordon Smyth and Di Wu, following Zar, JD (1999) Biostatistical Analysis 4th Edition
## used under GPL(>=2) license. The function has been slihtly modified to allow reporting results

#' @importFrom stats pt
rankSumTestWithCorrelation <- function (index, statistics, correlation = 0, df = Inf) {
    n <- length(statistics)
    r <- rank(statistics)
    r1 <- r[index]
    n1 <- length(r1)
    n2 <- n - n1
    U <- n1 * n2 + n1 * (n1 + 1)/2 - sum(r1)
    mu <- n1 * n2/2
    if (correlation == 0 || n1 == 1) {
        sigma2 <- n1 * n2 * (n + 1)/12
    }
    else {
        sigma2 <- asin(1) * n1 * n2 + asin(0.5) * n1 * n2 * (n2 - 
            1) + asin(correlation/2) * n1 * (n1 - 1) * n2 * (n2 - 
            1) + asin((correlation + 1)/2) * n1 * (n1 - 1) * 
            n2
        sigma2 <- sigma2/2/pi
    }
    TIES <- (length(r) != length(unique(r)))
    if (TIES) {
        NTIES <- table(r)
        prod <- sum(NTIES * (NTIES + 1) * (NTIES - 1))
        denom <- (n * (n + 1) * (n - 1))
        adjustment <- prod/denom
        sigma2 <- sigma2 * (1 - adjustment)
    }
    zlowertail <- (U + 0.5 - mu)/sqrt(sigma2)
    zuppertail <- (U - 0.5 - mu)/sqrt(sigma2)
    less <-pt(zuppertail, df = df, lower.tail = FALSE)
    greater <- pt(zlowertail, df = df)
    res <- c(U=U,
             mu=mu,
             n1=n1,
             n2=n2,
             sigma2=sigma2,
             r1sum=sum(r1),
             zlt=zlowertail,
             zut=zuppertail,
             less = less,
             greater = greater)
    return(res)
}


#' Wilcoxon-Mann-Whitney test in R
#'
#' @param x A numerical vector
#' @param sub A logical vector or integer vector to subset \code{x}. Numbers in \code{sub} are compared with numbers out of \code{sub}
#' @param valType Type of retured-value. Supported values: p.greater, p.less, p.two.sided, and W statistic (note it is different from the U statistic)
#' 
#' @importFrom stats wilcox.test
#' 
#' @examples
#' testNums <- 1:10
#' testSub <- rep_len(c(TRUE, FALSE), length.out=length(testNums))
#' wmwTestInR(testNums, testSub)
#' wmwTestInR(testNums, testSub, valType="p.two.sided")
#' wmwTestInR(testNums, testSub, valType="p.less")
#' wmwTestInR(testNums, testSub, valType="W")
#' @export
wmwTestInR <- function(x, sub, valType=c("p.greater", "p.less", "p.two.sided",  "W")) {
    if(is.numeric(sub)) {
        tmp <- rep(FALSE, length(x))
        tmp[sub] <- TRUE
        sub <- tmp
    }
    valType <- match.arg(valType)
    if(!is.logical(sub))
        stop("sub must be either numeric indices or logical")
    isStat <- valType=="W"
    if(!any(sub)) return(ifelse(isStat, 0, 1))
    if(!isStat) {
        alternative <- substr(valType, 3, nchar(valType))
        stopifnot(alternative %in% c("greater", "less", "two.sided"))
    } else {
        alternative <- "two.sided"
    }
    wt <- wilcox.test(x[sub],x[!sub],
                      alternative=alternative, exact=FALSE)
    return(ifelse(isStat, wt$statistic, wt$p.value))
}

## type2int and formatMatrixInd are helper functions for wmwTest and wmwSignedTest
TYPE_CODES <- c("p.greater"=0L, "p.less"=1L,
                "p.two.sided"=2L,
		"U"=3L,
                "abs.log10p.greater"=4L,
                "log10p.less"=5L,
                "abs.log10p.two.sided"=6L,
                "Q"=7L,
                "r"=8L,
		"f"=9L,
                "U1"=10L,
		"U2"=11L)

#'prints the options of valTypes of wmwTest
valTypes <- function() names(TYPE_CODES)

type2int <- function(type) {
    if(!type %in% names(TYPE_CODES))
        stop("Should not happen! This is the wrong code")
    return(TYPE_CODES[type])
}

##----------------------------------------##
## wmwTest
##----------------------------------------##
#' @importFrom methods is
wmwTest.default <- function(matrix,
                            indexList,
                            valType=c("p.greater", "p.less", "p.two.sided", "U",
                                "abs.log10p.greater","log10p.less","abs.log10p.two.sided",
                                "Q", "r", "f", "U1", "U2"),
                            simplify=TRUE) {
    if(!is.matrix(matrix) || !is(indexList, "IndexList"))
        stop("'matrix' and 'indexList' must be matrix and an IndexList object, respectively")
    if(missing(simplify))  simplify <- TRUE
    if(missing(valType)) {
        valType <- "p.greater"
    } else {
        valType <- match.arg(valType)
    }
    typeInt <- type2int(valType)

    if(storage.mode(matrix)=="character")
        stop("Input must be a numeric matrix or anything that can be converted into a numeric matrix")

    if(storage.mode(matrix)!="double")
        storage.mode(matrix) <- "double"

    if(offset(indexList)!=0L)
        offset(indexList) <- 0L

    res <- .Call(C_wmw_test, matrix, indexList, typeInt)
    rownames(res) <- names(indexList)
    colnames(res) <- colnames(matrix)

    if(simplify) {
        res <- simplifyMatrix(res)
    }
    return(res)
}
#'@describeIn wmwTest \code{x} is a \code{matrix} and \code{indexList} is a \code{IndexList}
setMethod("wmwTest", c("matrix", "IndexList"),
          function(x, indexList,
                   valType="p.greater", simplify = TRUE) {
              wmwTest.default(x, indexList, valType=valType, simplify=simplify)
          })
#'@describeIn wmwTest \code{x} is a \code{numeric} and \code{indexList} is a \code{IndexList}
setMethod("wmwTest", c("numeric", "IndexList"),
          function(x, indexList,
                    valType="p.greater", simplify = TRUE) {
              x <- matrix(x, ncol=1)
              wmwTest.default(x, indexList, valType=valType, simplify=simplify)
          })
#'@describeIn wmwTest \code{x} is a \code{matrix} and \code{indexList} is a \code{GmtList}
setMethod("wmwTest", c("matrix", "GmtList"),
          function(x, indexList,
                   valType="p.greater", simplify = TRUE) {
              indexList <- matchGenes(indexList, x)
              wmwTest.default(x, indexList, valType=valType, simplify=simplify)
          })
#'@describeIn wmwTest \code{x} is a \code{eSet} and \code{indexList} is a \code{GmtList}
setMethod("wmwTest", c("eSet", "GmtList"),
          function(x, indexList, col="GeneSymbol",
                   valType="p.greater", simplify = TRUE) {
              indexList <- matchGenes(indexList, x, col = col)
              wmwTest.default(exprs(x), indexList, valType=valType, simplify=simplify)
          })
#'@describeIn wmwTest \code{x} is a \code{eSet} and \code{indexList} is a \code{numeric}
setMethod("wmwTest", c("eSet", "numeric"),
          function(x, indexList, col="GeneSymbol",
                   valType="p.greater", simplify = TRUE) {
              wmwTest(exprs(x), indexList, valType=valType, simplify=simplify)
          })
#'@describeIn wmwTest \code{x} is a \code{eSet} and \code{indexList} is a \code{logical}
setMethod("wmwTest", c("eSet", "logical"),
          function(x, indexList, col="GeneSymbol",
                   valType="p.greater", simplify = TRUE) {
              wmwTest(exprs(x), indexList, valType=valType, simplify=simplify)
          })
#'@describeIn wmwTest \code{x} is a \code{eSet} and \code{indexList} is a \code{list}
setMethod("wmwTest", c("eSet", "list"),
          function(x, indexList, col="GeneSymbol",
                   valType="p.greater", simplify = TRUE) {
              indexList <- IndexList(indexList)
              wmwTest(exprs(x), indexList, valType=valType, simplify=simplify)
          })
#'@describeIn wmwTest \code{x} is \code{ANY} and \code{indexList} is a \code{numeric}
setMethod("wmwTest", c("ANY", "numeric"),
          function(x, indexList, valType="p.greater", simplify = TRUE) {
              indexList <- IndexList(indexList)
              wmwTest(x, indexList, valType=valType, simplify=simplify)
          })
#'@describeIn wmwTest \code{x} is \code{ANY} and \code{indexList} is a \code{logical}
setMethod("wmwTest", c("ANY", "logical"),
          function(x, indexList, valType="p.greater", simplify = TRUE) {
              indexList <- IndexList(indexList)
              wmwTest(x, indexList, valType=valType, simplify=simplify)
          })
#'@describeIn wmwTest \code{x} is \code{ANY} and \code{indexList} is a \code{list}
setMethod("wmwTest", c("ANY", "list"),
          function(x, indexList, valType="p.greater", simplify = TRUE) {
              indexList <- IndexList(indexList)
              wmwTest(x, indexList, valType=valType, simplify=simplify)
          })


##----------------------------------------##
## wmwTestSignedGenesets
##----------------------------------------##
#'@title
#' 
#'Wilcoxon-Mann-Whitney rank sum test for high-throughput expression
#'profiling data
#'
#'@description
#'wmwTest is a highly efficient Wilcoxon-Mann-Whitney rank sum
#'test for high-dimensional data, such as gene expression profiling. For datasets with
#'more than 100 features (genes), the function can be more than 1,000
#'times faster than its R implementations (\code{wilcox.test} in 
#'\code{stats}, or \code{rankSumTestWithCorrelation} in \code{limma}).
#'
#'@param x A numeric matrix. All other data types (e.g. numeric vectors
#'or \code{ExpressionSet} objects) are coerced into matrix.
#'
#'@param indexList A list of integer indices (starting from 1) indicating
#'signature genes. Can be of length zero. Other data types (e.g. a list
#'of numeric or logical vectors, or a numeric or logical vector) are
#'coerced into such a list. See \code{details} below for a special case
#'using GMT files.
#'
#'@param valType The value type to be returned, allowed values
#'include \code{p.greater}, \code{p.less}, \code{abs.log10p.greater} and
#'\code{abs.log10p.less} (one-sided tests),\code{p.two.sided}, and \code{U}
#'statistic (or more specifically, either \code{U1} or \code{U2}), and a few
#'other variants. See details below.
#'
#'@param col a string sometimes used with a \code{eSet}
#'
#'@param simplify Logical. If not, the returning value is in matrix
#'format; if set to \code{TRUE}, the results are simplified into
#'vectors when possible (default).
#'
#'@details The basic application of the function is to test the enrichment of
#'gene sets in expression profiling data or differentially expressed
#'data (the matrix with feature/gene in rows and samples in columns).
#'
#'A special case is when \code{x} is an \code{eSet} object
#'(e.g. \code{ExpressionSet}), and \code{indexList} is a list returned
#'from \code{readGmt} function. In this case, the only requirement is
#'that one column named \code{GeneSymbol} in the \code{featureData}
#'contain gene symbols used in the GMT file. The same applies to signed Gmt files. See the example below.
#'
#'Besides the conventional value types such as \sQuote{p.greater},
#'\sQuote{p.less}, \sQuote{p.two.sided} , and \sQuote{U} (the U-statistic),
#'\code{wmwTest} (from version 0.99-1) provides further value types:
#'\code{abs.log10p.greater} and \code{log10p.less} perform log10
#'transformation on respective \emph{p}-values and give the
#'transformed value a proper sign (positive for greater than, and
#'negative for less than); \code{abs.log10p.two.sided} transforms
#'two-sided \emph{p}-values to non-negative values; and \code{Q} score
#'reports absolute log10-transformation of \emph{p}-value of the
#'two-side variant, and gives a proper sign to it, depending on whether it is
#'rather greater than (positive) or  less than (negative).
#'
#'From version 1.19.1, the rank-biserial correlation coefficient (\sQuote{r})
#'and the common language effect size (\sQuote{f}) are supported value types.
#'
#'Before version 1.19.3, the \sQuote{U} statistic returned is in fact
#'\sQuote{U2}. From version 1.19.3, \sQuote{U1} is returned when \sQuote{U} is
#'used, and users can specify additional parameter values \sQuote{U1} and
#'\sQuote{U2}. The sum of \sQuote{U1} and \sQuote{U2} is the product of the
#'sizes of two vectors to be compared.
#'
#'@return A numeric matrix or vector containing the statistic.
#'
#'@references Barry, W.T., Nobel, A.B., and Wright, F.A. (2008). A statistical framework for testing functional namespaces in microarray data. _Annals of Applied Statistics_ 2, 286-315.
#'
#'Wu, D, and Smyth, GK (2012). Camera: a competitive gene set test
#'accounting for inter-gene correlation. _Nucleic Acids Research_ 40(17):e133
#'
#'Zar, JH (1999). _Biostatistical Analysis 4th Edition_. Prentice-Hall International, Upper Saddle River, New Jersey.
#'
#' @import Rcpp
#' @importFrom Biobase fData exprs
#' @importClassesFrom Biobase eSet
#'
#'@author Jitao David Zhang <jitao_david.zhang@roche.com>, with critical inputs
#'from Jan Aettig and Iakov Davydov about U statistics.
#'
#'@note The function has been optimized for expression profiling data. It
#'avoids repetitive ranking of data as done by native R implementations
#'and uses efficient C code to increase the performance and control
#'memory use. Simulation studies using expression profiles of 22000
#'genes in 2000 samples and 200 gene sets suggested that the C
#'implementation can be >1000 times faster than the R
#'implementation. And it is possible to further accelerate by
#'parallel calling the function with \code{mclapply} in the \code{multicore} package.
#'
#'@seealso code{wilcox.test} in the \code{stats} package, and \code{rankSumTestWithCorrelation} in
#'the \code{limma} package.
#'
#'@examples
#'## R-native data structures
#'set.seed(1887)
#'rd <- rnorm(1000)
#'rl <- sample(c(TRUE, FALSE), 1000, replace=TRUE)
#'wmwTest(rd, rl, valType="p.two.sided")
#'wmwTest(rd, which(rl), valType="p.two.sided")
#'rd1 <- rd + ifelse(rl, 0.5, 0)
#'wmwTest(rd1, rl, valType="p.greater")
#'wmwTest(rd1, rl, valType="U")
#'rd2 <- rd - ifelse(rl, 0.2, 0)
#'wmwTest(rd2, rl, valType="p.greater")
#'wmwTest(rd2, rl, valType="p.two.sided")
#'wmwTest(rd2, rl, valType="p.less")
#'wmwTest(rd2, rl, valType="r")
#'wmwTest(rd2, rl, valType="f")
#'
#'## matrix forms
#'rmat <- matrix(c(rd, rd1, rd2), ncol=3, byrow=FALSE)
#'wmwTest(rmat, rl, valType="p.two.sided")
#'wmwTest(rmat, rl, valType="p.greater")
#'
#'wmwTest(rmat, which(rl), valType="p.two.sided")
#'wmwTest(rmat, which(rl), valType="p.greater")
#'
#'## other valTypes
#'wmwTest(rmat, which(rl), valType="U")
#'wmwTest(rmat, which(rl), valType="abs.log10p.greater")
#'wmwTest(rmat, which(rl), valType="log10p.less")
#'wmwTest(rmat, which(rl), valType="abs.log10p.two.sided")
#'wmwTest(rmat, which(rl), valType="Q")
#'wmwTest(rmat, which(rl), valType="r")
#'wmwTest(rmat, which(rl), valType="f")
#'
#'## using ExpressionSet
#'data(sample.ExpressionSet)
#'testSet <- sample.ExpressionSet
#'fData(testSet)$GeneSymbol <- paste("GENE_",1:nrow(testSet), sep="")
#'mySig1 <- sample(c(TRUE, FALSE), nrow(testSet), prob=c(0.25, 0.75), replace=TRUE)
#'wmwTest(testSet, which(mySig1), valType="p.greater")
#'
#'## using integer
#'exprs(testSet)[,1L] <- exprs(testSet)[,1L] + ifelse(mySig1, 50, 0)
#'wmwTest(testSet, which(mySig1), valType="p.greater")
#'
#'## using lists
#'mySig2 <- sample(c(TRUE, FALSE), nrow(testSet), prob=c(0.6, 0.4), replace=TRUE)
#'wmwTest(testSet, list(first=mySig1, second=mySig2))

#'## using GMT file
#'gmt_file <- system.file("extdata/exp.tissuemark.affy.roche.symbols.gmt", package="BioQC")
#'gmt_list <- readGmt(gmt_file)
#'
#'gss <- sample(unlist(sapply(gmt_list, function(x) x$genes)), 1000)
#'eset<-new("ExpressionSet",
#'          exprs=matrix(rnorm(10000), nrow=1000L),
#'          phenoData=new("AnnotatedDataFrame", data.frame(Sample=LETTERS[1:10])),
#'          featureData=new("AnnotatedDataFrame",data.frame(GeneSymbol=gss)))
#'esetWmwRes <- wmwTest(eset ,gmt_list, valType="p.greater")
#'summary(esetWmwRes)
#'
#'## using signed GMT file
#'signed_gmt_file <- system.file("extdata/test.gmt", package="BioQC")
#'signed_gmt <- readSignedGmt(signed_gmt_file)
#'esetSignedWmwRes <- wmwTest(eset, signed_gmt, valType="p.greater")
#'
#'esetMat <- exprs(eset); rownames(esetMat) <- fData(eset)$GeneSymbol
#'esetSignedWmwRes2 <- wmwTest(esetMat, signed_gmt, valType="p.greater")
#'@name wmwTest
NULL

#' @importFrom methods is
wmwTestSignedGenesets.default <- function(matrix,
                                          signedIndexList,
                                          valType=c("p.greater", "p.less", "p.two.sided", "U",
                                              "abs.log10p.greater","log10p.less","abs.log10p.two.sided",
                                              "Q", "r", "f", "U1", "U2"),
                                          simplify=TRUE) {
    if(!is.matrix(matrix) || !is(signedIndexList, "SignedIndexList"))
        stop("'matrix' and 'signedIndexList' must be matrix and an SignedIndexList object, respectively")
    if(missing(simplify))  simplify <- TRUE
    if(missing(valType)) {
        valType <- "p.greater"
    } else {
        valType <- match.arg(valType)
    }
    typeInt <- type2int(valType)

    if(storage.mode(matrix)=="character")
        stop("Input must be a numeric matrix or anything that can be converted into a numeric matrix")

    if(storage.mode(matrix)!="double")
        storage.mode(matrix) <- "double"

    if(offset(signedIndexList)!=0L)
        offset(signedIndexList) <- 0L

    res <- .Call(C_signed_wmw_test, matrix,
                 signedIndexList,
                 typeInt)
    rownames(res) <- names(signedIndexList)
    colnames(res) <- colnames(matrix)

    if(simplify) {
        res <- simplifyMatrix(res)
    }
    return(res)
}
#'@describeIn wmwTest \code{x} is a \code{matrix} and \code{indexList} is a 
#'\code{SignedIndexList}
setMethod("wmwTest", c("matrix", "SignedIndexList"), function(x, indexList, valType, simplify = TRUE) {
    wmwTestSignedGenesets.default(x, indexList, valType=valType, simplify = simplify)
})

#'@describeIn wmwTest \code{x} is a \code{eSet} and \code{indexList} is a 
#'\code{SignedIndexList}
setMethod("wmwTest", c("matrix", "SignedGenesets"),
          function(x, indexList,
                   valType, simplify = TRUE) {
            indexList <- matchGenes(indexList, x)
            wmwTestSignedGenesets.default(x, indexList, valType=valType, simplify=simplify)
          })

#'@describeIn wmwTest \code{x} is a \code{numeric} and \code{indexList} is a 
#'\code{SignedIndexList}
setMethod("wmwTest", c("numeric", "SignedIndexList"), function(x, indexList, valType, simplify = TRUE) {
              x <- matrix(x, ncol=1L)
              wmwTestSignedGenesets.default(x, indexList, valType=valType, simplify = simplify)
})
#'@describeIn wmwTest \code{x} is a \code{eSet} and \code{indexList} is a 
#'\code{SignedIndexList}
setMethod("wmwTest", c("eSet", "SignedIndexList"), function(x, indexList, valType, simplify = TRUE) {
              wmwTestSignedGenesets.default(exprs(x), indexList, valType=valType, simplify = simplify)
          })

#'@describeIn wmwTest \code{x} is a \code{eSet} and \code{indexList} is a 
#'\code{SignedIndexList}
setMethod("wmwTest", c("eSet", "SignedGenesets"),
          function(x, indexList, col="GeneSymbol",
                   valType, simplify = TRUE) {
            indexList <- matchGenes(indexList, x, col = col)
            wmwTestSignedGenesets.default(exprs(x), indexList, valType=valType, simplify=simplify)
          })

## ----setup, include=FALSE-----------------------------------------------------
options(fig_caption=TRUE)
library(knitr)
opts_chunk$set(out.extra='style="display:block; margin: auto"', fig.align="center")

## ----lib, warning=FALSE, message=FALSE, results="hide"------------------------
library(BioQC)
gmtFile <- system.file("extdata/exp.tissuemark.affy.roche.symbols.gmt",
                        package="BioQC")
gmt <- readGmt(gmtFile)

## ----load_data, warning=FALSE, message=FALSE, results="hide"------------------
file <- system.file("extdata/kidney_example.rda", package="BioQC")
load(file)
print(eset)

## ----run_bioqc----------------------------------------------------------------
system.time(bioqcRes <- wmwTest(eset, gmt,
                                 valType="p.greater"))

## ----visualize_bioqc_prepare--------------------------------------------------
bioqcResFil <- filterPmat(bioqcRes, 1E-6)
bioqcAbsLogRes <- absLog10p(bioqcResFil)

## ----heatmap, fig.width=8, fig.height=4, dev='png', fig.cap="BioQC scores (defined as abs(log10(p))) of the samples visualized in heatmap. Red and blue indicate high and low scores respectively."----
library(RColorBrewer)
heatmap(bioqcAbsLogRes, Colv=NA, Rowv=TRUE,
        cexRow=0.85, scale="row",
        col=rev(brewer.pal(7, "RdBu")),
        labCol=1:ncol(bioqcAbsLogRes))

## ----vis_bioqc, fig.width=8, fig.height=4, dev='png', fig.cap="BioQC scores (defined as abs(log10(p))) of the samples. K and P represent kidney and pancreas signature scores respectively."----
filRes <- bioqcAbsLogRes[c("Kidney_NGS_RNASEQATLAS_0.6_3",
                           "Pancreas_Islets_NR_0.7_3"),]
matplot(t(filRes), pch=c("K", "P"), type="b", lty=1L,
        ylab="BioQC score", xlab="Sample index")

## ----rt_pcr_result, fig.width=8, fig.height=4, dev='png', fig.cap="Correlation between qRT-PCR results and BioQC pancreas score"----
amylase <- eset$Amylase
elastase <- eset$Elastase
pancreasScore <- bioqcAbsLogRes["Pancreas_Islets_NR_0.7_3",]
par(mfrow=c(1,2), mar=c(3,3,1,1), mgp=c(2,1,0))
plot(amylase~pancreasScore, log="y", pch=21, bg="red",
     xlab="BioQC pancreas score", ylab="Amylase")
text(pancreasScore[23:25],amylase[23:25], 23:25, pos=1)
plot(elastase~pancreasScore, log="y", pch=21, bg="red",
     xlab="BioQC pancreas score", ylab="Elastase")
text(pancreasScore[23:25],elastase[23:25], 23:25, pos=1)


## ----load_limma, warning=FALSE, message=FALSE, results="hide"-----------------
library(limma)

## ----deg, fig.width=5, fig.height=5, dev='png', fig.cap="Log2 fold change (logFC) values reported by *limma* with one contaminated sample included (y-axis) or excluded (x-axis). Genes strongly affected by the contamination are indicated by red dots."----

isNeph <- with(pData(eset), Strain=="FVB/NJ" &
                 TREATMENTNAME %in% c("Nephrectomy-Losartan", "Sham-Losartan"))
isContam <- with(pData(eset), INDIVIDUALNAME %in% c("BN7", "FNL8", "FN6"))
esetNephContam <- eset[,isNeph]
esetNephExclContam <- eset[, isNeph & !isContam]
getDEG <- function(eset) {
  group <- factor(eset$TREATMENTNAME, levels=c("Sham-Losartan","Nephrectomy-Losartan"))
  design <- model.matrix(~group)
  colnames(design) <- c("ShamLo", "NephLo")
  contrast <- makeContrasts(contrasts="NephLo", levels=design)
  exprs(eset) <- normalizeBetweenArrays(log2(exprs(eset)))
  fit <- lmFit(eset, design=design)
  fit <- contrasts.fit(fit, contrast)
  fit <- eBayes(fit)
  tt <- topTable(fit, n=nrow(eset))
  return(tt)
}

esetNephContam.topTable <- getDEG(esetNephContam)
esetNephExclContam.topTable <- getDEG(esetNephExclContam)
esetFeats <- featureNames(eset)
esetNephTbl <- data.frame(feature=esetFeats,
                          OrigGeneSymbol=esetNephContam.topTable[esetFeats,]$OrigGeneSymbol,
                          GeneSymbol=esetNephContam.topTable[esetFeats,]$GeneSymbol,
                          Contam.logFC=esetNephContam.topTable[esetFeats,]$logFC,
                          ExclContam.logFC=esetNephExclContam.topTable[esetFeats,]$logFC)
par(mfrow=c(1,1), mar=c(3,3,1,1)+0.5, mgp=c(2,1,0))
with(esetNephTbl, smoothScatter(Contam.logFC~ExclContam.logFC,
                                xlab="Excluding one contaminating sample [logFC]",
                                ylab="Including one contaminating sample [logFC]"))
abline(0,1)
isDiff <- with(esetNephTbl, abs(Contam.logFC-ExclContam.logFC)>=2)
with(esetNephTbl, points(Contam.logFC[isDiff]~ExclContam.logFC[isDiff], pch=16, col="red"))
diffTable <- esetNephTbl[isDiff,]
diffGenes <- unique(diffTable[,"GeneSymbol"])
pancreasSignature <- gmt[["Pancreas_Islets_NR_0.7_3"]]$genes
diffGenesPancreas <- diffGenes %in% pancreasSignature
diffTable$isPancreasSignature <- diffTable$GeneSymbol %in% pancreasSignature
colnames(diffTable) <- c("Probeset", "GeneSymbol", "Human ortholog",
                         "Log2FC", "Log2FC (excl. contam.)",
                         "IsPancreasSignature")
diffTable <- diffTable[order(diffTable$Log2FC, decreasing=TRUE),]

## ----kable_deg, include=FALSE-------------------------------------------------
kable(diffTable, caption="Genes that are identified as strongly changed ony if the contaminated sample is included.")

## ----session_info-------------------------------------------------------------
sessionInfo()

##----------------------------------------##
## standard generics
##----------------------------------------##
#' @rdname offset
#' @usage offset(object)
#' @exportMethod offset
setGeneric("offset", function(object) standardGeneric("offset"))

#'@rdname offset-set
#'@usage `offset<-`(object, value)
#'@exportMethod offset<-
setGeneric("offset<-", function(object, value) standardGeneric("offset<-"))

#' @rdname IndexList
#' @exportMethod IndexList
setGeneric("IndexList", function(object, ..., keepNA = FALSE, keepDup = FALSE, offset =1L) standardGeneric("IndexList"))

#'@rdname SignedIndexList
#'@exportMethod SignedIndexList
setGeneric("SignedIndexList", function(object, ...) standardGeneric("SignedIndexList"))

#'@rdname matchGenes
#'@exportMethod matchGenes
setGeneric("matchGenes", function(list, object, ...) standardGeneric("matchGenes"))

#'@rdname wmwTest
#'@exportMethod wmwTest
setGeneric("wmwTest", function(x, indexList, col = "GeneSymbol", valType = c("p.greater", "p.less",
    "p.two.sided", "U", "abs.log10p.greater", "log10p.less",
    "abs.log10p.two.sided", "Q", "r", "f", "U1", "U2"), simplify = TRUE) standardGeneric("wmwTest"))


##--------------------##
## IndexList
##--------------------##
parseIndex <- function(x, keepNA=FALSE, keepDup=FALSE) {
    if(is.null(x))
        return(x)
    if(is.logical(x))
        x <- which(x)
    x <- as.integer(x)
    if(!keepNA)
        x <- x[!is.na(x)]
    if(!keepDup)
        x <- unique(x)
    return(x)
}

#' @importFrom methods new
IndexListFromList <- function(inlist, keepNA=FALSE, keepDup=FALSE, offset=1L) {
    outlist <- lapply(inlist, parseIndex, keepNA=keepNA, keepDup=keepDup)
    ## note that .Data must be the first data slot in the new command
    res <- new("IndexList", .Data=outlist, keepNA=keepNA, keepDup=keepDup, offset=as.integer(offset))
    return(res)
}

#' Convert a list to an IndexList object
#' 
#' @param object Either a list of unique integer indices, NULL and logical 
#' vectors (of same lengths), or a numerical vector or a logical vector. NA is discarded.
#' @param ...  If \code{object} isn't a list, additional vectors can go here.
#' @param keepNA Logical, whether NA indices should be kept or not. Default: FALSE (removed)
#' @param keepDup Logical, whether duplicated indices should be kept or not. Default: FALSE (removed)
#' @param offset Integer, the starting index. Default: 1 (as in the convention of R)
#' @return The function returns a list of vectors
#' @examples
#' testList <- list(GS_A=c(1,2,3,4,3),
#'                  GS_B=c(2,3,4,5),
#'                  GS_C=NULL,
#'                  GS_D=c(1,3,5,NA),
#'                  GS_E=c(2,4))
#' testIndexList <- IndexList(testList)
#' @name IndexList
#' @exportMethod IndexList
NULL

#' @rdname IndexList
setMethod("IndexList", "numeric", function(object, ..., keepNA=FALSE, keepDup=FALSE, offset=1L) {
              olist <- list(...)
              list <- c(list(object), olist)
              IndexListFromList(list, keepNA=keepNA, keepDup=keepDup, offset=offset)
          })


#' @examples
#' IndexList(c(FALSE, TRUE, TRUE), c(FALSE, FALSE, TRUE), c(TRUE, FALSE, FALSE), offset=0)
#' @rdname IndexList
setMethod("IndexList", "logical", function(object, ..., keepNA=FALSE, keepDup=FALSE, offset=1L) {
              olist <- list(...)
              list <- c(list(object), olist)
              if(!all(sapply(list, is.logical)))
                  stop("all input must be logical vectors")
              if(!length(unique(length(sapply(list, length))))==1)
                  stop("all input must be of the same length")
              list <- lapply(list, which)
              IndexListFromList(list, keepNA=keepNA, keepDup=keepDup, offset=offset)
          })

#' @examples
#' IndexList(list(A=1:3, B=4:5, C=7:9))
#' IndexList(list(A=1:3, B=4:5, C=7:9), offset=0)
#' @rdname IndexList
setMethod("IndexList", "list", function(object, keepNA=FALSE, keepDup=FALSE, offset=1L) {
              IndexListFromList(object, keepNA=keepNA, keepDup=keepDup, offset=offset)
          })

##----------------------------------------##
## SignedIndexList
##----------------------------------------##
#' @importFrom methods new
SignedIndexListFromList <- function(inlist, keepNA=FALSE, keepDup=FALSE, offset=1L) {
  if(length(inlist)==2 && all(c("pos", "neg") %in% names(inlist))) {
    inlist <- list(inlist)
  }
  outlist <- lapply(inlist, function(x) list(pos=parseIndex(x$pos, keepNA=keepNA, keepDup=keepDup),
                                             neg=parseIndex(x$neg, keepNA=keepNA, keepDup=keepDup)))
  ## note that .Data must be the first data slot in the new command
  res <- new("SignedIndexList", .Data=outlist, keepNA=keepNA, keepDup=keepDup, offset=as.integer(offset))
  return(res)
}

#'Convert a list into a SignedIndexList
#'@name SignedIndexList
NULL
#'@param object A list of lists, each with two elements named `pos` or `neg`, can be logical vectors or integer indices
#'@param ... additional arguments, currently ignored
#'@param keepNA Logical, whether NA indices should be kept or not. Default: 
#'FALSE (removed)
#'@param keepDup Logical, whether duplicated indices should be kept or not. 
#'Default: FALSE (removed) 
#'@param offset offset; 1 if missing
#'@return A SignedIndexList, a list of lists, containing two vectors named `positive` and `negative`, 
#' which contain the indices of genes that are either positively or negatively associated with a certain
#' phenotype
#' 
#'@examples
#'myList <- list(a = list(pos = list(1, 2, 2, 4), neg = c(TRUE, FALSE, TRUE)), 
#'b = list(NA), c = list(pos = c(c(2, 3), c(1, 3))))
#'SignedIndexList(myList)
#'
#'## a special case of input is a single list with two elements, \code{pos} and \code{neg}
#'SignedIndexList(myList[[1]])
#'@rdname SignedIndexList
#'@export SignedIndexList
setMethod("SignedIndexList", "list", function(object, keepNA=FALSE, keepDup=FALSE, offset=1L) {
     SignedIndexListFromList(object, keepNA=keepNA, keepDup=keepDup, offset=offset)
})

##--------------------##
## offset
##--------------------##
#' Get offset from an IndexList object
#'
#' @param object An IndexList object
#'
#' @examples
#' myIndexList <- IndexList(list(1:5, 2:7, 3:8), offset=1L)
#' offset(myIndexList)
#' @name offset
#' @aliases offset,BaseIndexList-method
#' @export 
setMethod("offset", "BaseIndexList", function(object) return(object@offset))

modOffset <- function(x, diff) {
    if(is.null(x)) return(NULL)
    return(x-diff)
}

#' Set the offset of an \code{IndexList} or a \code{SignedIndexList} object
#' 
#' @param object An \code{IndexList} or a \code{SignedIndexList} object
#' @param value The value, that the offset of \code{object} is set too. If it 
#' isn't an integer, it's coerced into an integer.
#' @examples 
#' myIndexList <- IndexList(list(1:5, 2:7, 3:8), offset=1L)
#' offset(myIndexList)
#' offset(myIndexList) <- 3
#' offset(myIndexList)
#' @name offset-set
NULL

#'@rdname offset-set
setMethod("offset<-", c("IndexList", "numeric"), function(object, value) {
              value <- as.integer(value)
              diff <- object@offset - value
              object@offset <- value
              resList <- lapply(object@.Data, modOffset, diff=diff)
              object@.Data <- resList
              return(object)
          })

#'@rdname offset-set
setMethod("offset<-", c("SignedIndexList", "numeric"), function(object, value) {
    value <- as.integer(value)
    diff <- object@offset - value
    object@offset <- value
    resList <- lapply(object@.Data, function(x) list(pos=modOffset(x$pos, diff),
                                                     neg=modOffset(x$neg, diff)))
    object@.Data <- resList
    return(object)
})

##--------------------##
## subsetting
##--------------------##
#' Subsetting GmtList object into another GmtList object
#' @param x A GmtList object
#' @param i Index to subset
#' @param drop In case only one element remains, should a list representing the single geneset returned? Default: FALSE
#' 
#' @importFrom methods new
#' 
#' @examples 
#' myGmtList <- GmtList(list(gs1=letters[1:3], gs2=letters[3:4], gs3=letters[4:5]))
#' myGmtList[1:2]
#' myGmtList[1] ## default behaviour: not dropping
#' myGmtList[1,drop=TRUE] ## force dropping
#' @export
`[.GmtList` <- function(x, i, drop=FALSE) {
  if(is.character(i))
    i <- match(i, names(x))
  res <- new("GmtList", .Data=x@.Data[i])
  if(length(res)==1 && drop)
    res <- res@.Data[[1]]
  names(res) <- names(x)[i]
  return(res)
}

#' Subsetting GmtList object to fetch one gene-set
#' @param x A GmtList object
#' @param i The index to subset
#' @examples 
#' myGmtList <- GmtList(list(gs1=letters[1:3], gs2=letters[3:4], gs3=letters[4:5]))
#' myGmtList[[1]]
#' @export
`[[.GmtList` <- function(x, i) {
  if(is.character(i))
    i <- match(i, names(x))
  res <- x@.Data[[i]]
  return(res)
}

##--------------------##
## show for GmtList
##--------------------##
showGeneSet <- function(geneset, nGene=3, indent=2) {
    genes <- geneset$genes
    geneLen <- length(genes)
    sprintf("%s%s (%sn=%d): %s%s",
            paste(rep(" ", indent), collapse=""),
            geneset$name,
            ifelse(is.null(geneset$desc), "", paste(geneset$desc, ",", sep="")),
            geneLen,
            paste(geneset$genes[1:pmin(nGene, geneLen)],collapse=","),
            ifelse(geneLen>nGene, ",...", ""))
}

#' Show method for GmtList
#' 
#' @param object An object of the class \code{GmtList}
#' 
#' @export
setMethod("show", "GmtList", function(object) {
  str <- sprintf("A gene-set list in GMT format with %d genesets\n", length(object))

  indent <- 2
  if(hasNamespace(object)) {
    namespaces <- gsNamespace(object)
    catTbl <- table(namespaces)
    catHead <- sprintf("%s:\n",
                       ifelse(length(catTbl)>1,
                              "Namespaces", "Namespace"))
    catDetailsStr <- sprintf("%s[%d] %s (n=%d)\n",
            paste(rep(" ", indent), collapse=""),
            seq(along=catTbl),
            names(catTbl),
            unname(catTbl))
    catStr <- paste(catHead, 
                    paste(catDetailsStr, collapse=""),
                    sep="")
  } else {
    catStr <- ""
  }
  showHead <- "Gene-sets:\n"
  if(length(object)>6) {
    heads <- object[1:3]
    tails <- object[(length(object)-2):length(object)]
    shows <- c(sapply(heads, showGeneSet, indent=indent),
               paste(paste(rep(" ", indent), collapse=""), "...", sep=""),
               sapply(tails, showGeneSet, indent=indent))
  } else {
    shows <- sapply(object, showGeneSet, indent=indent)
  }
  concStr <- paste(c(str, catStr, showHead,
                     paste(shows, collapse="\n"), "\n"), collapse="")
  cat(concStr)
})


##--------------------##
## show for SignedGenesets
##--------------------##
showSignedGeneset <- function(geneset, nGene=3, indent=2) {
    name <- geneset$name
    pos <- geneset$pos
    neg <- geneset$neg
    posLen <- length(pos)
    negLen <- length(neg)
    idents <- paste(rep(" ", indent), collapse="")
    doubleIdents <- paste(rep(" ", indent*2), collapse="")
    if(posLen+negLen==0) {
        res <- sprintf("%s%s (no genes)", idents, name)
    } else {
        res <- sprintf("%s%s[n=%d]\n%spositive[n=%d]:%s%s\n%snegative[n=%d]:%s%s",
                       idents,
                       name,
                       posLen+negLen,
                       doubleIdents,
                       posLen,
                       paste(pos[1:pmin(posLen, nGene)],collapse=","),
                       ifelse(posLen>nGene, ",...", ""),
                       doubleIdents,
                       negLen,
                       paste(neg[1:pmin(negLen, nGene)],collapse=","),
                       ifelse(negLen>nGene, ",...", ""))
    }
    return(res)
}

#' Show method for SignedGenesets
#' 
#' @param object An object of the class \code{SignedGenesets}
#' 
#' @export
setMethod("show", "SignedGenesets", function(object) {
              str <- sprintf("A list of %d signed gene-sets", length(object))
              indent <- 2
              if(length(object)>6) {
                  heads <- object[1:3]
                  tails <- object[(length(object)-2):length(object)]
                  shows <- c(sapply(heads, showSignedGeneset, indent=indent),
                             paste(paste(rep(" ", indent), collapse=""), "...", sep=""),
                             sapply(tails, showSignedGeneset, indent=indent))
              } else {
                  shows <- sapply(object, showSignedGeneset, indent=indent)
              }
              concStr <- paste(c(str, shows, ""), collapse="\n")
              cat(concStr)
          })

##--------------------##
## show for IndexList
##--------------------##
showIndices <- function(indices, name, nInd=3, indent=2) {
    if(is.null(name))
        name <- "NONAME"
    len <- length(indices)
    if(is.null(indices)) {
        isNA <- FALSE
    } else {
        isNA <- is.na(indices)
    }
    sprintf("%s%s (n=%d%s): %s%s",
            paste(rep(" ", indent), collapse=""),
            name,
            len,
            ifelse(any(isNA), sprintf(", with %d NA", sum(isNA)), ""),
            paste(indices[1:pmin(len, nInd)],collapse=","),
            ifelse(len>nInd, ",...", ""))
}

#' Show method for IndexList
#' 
#' @param object An object of the class \code{IndexList}
#' 
#' @export
setMethod("show", "IndexList", function(object) {
              str <- sprintf("A list of %d indices with offset=%d", length(object), object@offset)
              opts <- sprintf("Options: NA removed: %s; duplicates removed: %s", !object@keepNA, !object@keepDup)
              indent <- 2
              if(length(object)>6) {
                  heads <- object[1:3]
                  tails <- object[(length(object)-2):length(object)]
                  shows <- c(sapply(seq(along=heads), function(i) showIndices(heads[[i]], names(heads)[i], indent=indent)),
                             paste(paste(rep(" ", indent), collapse=""), "...", sep=""),
                             sapply(seq(along=tails), function(i) showIndices(tails[[i]], names(tails)[i], indent=indent)))
              } else {
                  shows <- sapply(seq(along=object), function(i) showIndices(object[[i]], names(object)[i], indent=indent))
              }
              concStr <- paste(c(str, opts, shows, ""), collapse="\n")
              cat(concStr)
          })

##--------------------##
## show for SignedIndexList
##--------------------##
showSignedIndices <- function(indices, name, nInd=3, indent=2) {
    if(is.null(name)) name <- "NONAME"
    indices$name <- name
    showSignedGeneset(indices, nGene=nInd, indent=indent)
}

#' Show method for SignedIndexList
#' 
#' @param object An object of the class \code{SignedIndexList}
#' 
#' @export
setMethod("show", "SignedIndexList", function(object) {
              str <- sprintf("A list of %d signed indices with offset=%d", length(object), object@offset)
              opts <- sprintf("Options: NA removed: %s; duplicates removed: %s", !object@keepNA, !object@keepDup)
              indent <- 2
              if(length(object)>6) {
                  heads <- object[1:3]
                  tails <- object[(length(object)-2):length(object)]
                  shows <- c(sapply(seq(along=heads), 
                                    function(i) showSignedIndices(heads[[i]], names(heads)[i], indent=indent)),
                             paste(paste(rep(" ", indent), collapse=""), "...", sep=""),
                             sapply(seq(along=tails), function(i) 
                               showSignedIndices(tails[[i]], names(tails)[i], indent=indent)))
              } else {
                  shows <- sapply(seq(along=object), function(i) 
                    showSignedIndices(object[[i]], names(object)[i], indent=indent))
              }
              concStr <- paste(c(str, opts, shows, ""), collapse="\n")
              cat(concStr)
          })

## ----setup, include=FALSE-----------------------------------------------------
options(fig_caption=TRUE)
library(knitr)
opts_chunk$set(out.extra='style="display:block; margin: auto"', fig.align="center")

## ----lib, warning=FALSE, message=FALSE, results="hide", include=FALSE---------
library(testthat)
library(BioQC)
library(hgu133plus2.db) ## to simulate an microarray expression dataset
library(lattice)
library(latticeExtra)
library(gridExtra)
library(gplots)
library(reshape2)
library(plyr)
library(ggplot2)
library(rbenchmark)

pdf.options(family="ArialMT", useDingbats=FALSE)

set.seed(1887)

## list human genes
humanGenes <- unique(na.omit(unlist(as.list(hgu133plus2SYMBOL))))

## read tissue-specific gene signatures
gmtFile <- system.file("extdata/exp.tissuemark.affy.roche.symbols.gmt",
                       package="BioQC")
gmt <- readGmt(gmtFile)

## ----helper, include=FALSE----------------------------------------------------
## Summarizes data.
## Gives count, mean, standard deviation, standard error of the mean, and confidence interval (default 95%).
##   data: a data frame.
##   measurevar: the name of a column that contains the variable to be summariezed
##   groupvars: a vector containing names of columns that contain grouping variables
##   na.rm: a boolean that indicates whether to ignore NA's
##   conf.interval: the percent range of the confidence interval (default is 95%)
## 
## Source: http://www.cookbook-r.com/Manipulating_data/Summarizing_data/
summarySE <- function(data=NULL, measurevar, groupvars=NULL, na.rm=FALSE,
        conf.interval=.95, .drop=TRUE) {

# New version of length which can handle NA's: if na.rm==T, don't count them
        length2 <- function (x, na.rm=FALSE) {
            if (na.rm) sum(!is.na(x))
            else       length(x)
        }

# This does the summary. For each group's data frame, return a vector with
# N, mean, and sd
    datac <- ddply(data, groupvars, .drop=.drop,
            .fun = function(xx, col) {
            c(N    = length2(xx[[col]], na.rm=na.rm),
                mean = mean   (xx[[col]], na.rm=na.rm),
                sd   = sd     (xx[[col]], na.rm=na.rm)
             )
            },
            measurevar
            )

# Rename the "mean" column    
        datac <- rename(datac, c("mean" = measurevar))

        datac$se <- datac$sd / sqrt(datac$N)  # Calculate standard error of the mean

# Confidence interval multiplier for standard error
# Calculate t-statistic for confidence interval: 
# e.g., if conf.interval is .95, use .975 (above/below), and use df=N-1
        ciMult <- qt(conf.interval/2 + .5, datac$N-1)
        datac$ci <- datac$se * ciMult

        return(datac)
}

## ----sensitivity_benchmark_fig, echo=FALSE, warning=FALSE, fig.width=8, fig.height=3.5, dev='png', fig.cap="**Figure 1:** Sensitivity benchmark. Expression levels of genes in the ovary signature are dedicately sampled randomly from normal distributions with different mean values. The lines show the enrichment score for the Wilcoxon-Mann-Whitney test, the t-test and the Kolmogorov-Smirnov test respectively. In the right panel, outliers were added by adding a random value to 1% of the simulated genes. "----

randomMatrixButOneSignature <- function(rows=humanGenes, signatureGenes,
                                        amplitudes=seq(0, 3, by=0.5)) {
  nrow <- length(rows)
  ncol <- length(amplitudes)
  mat <- matrix(rnorm(nrow*ncol),
                nrow=nrow, byrow=FALSE)
  rownames(mat) <- rows
  sigInd <- na.omit(match(signatureGenes, humanGenes))
  
  colClass <- factor(amplitudes)
  
  for(colInd in unique(colClass)) {
    isCurrCol <- colInd==colClass
    replaceMatrix <- matrix(rnorm(length(sigInd)*sum(isCurrCol),
                                  mean=amplitudes[isCurrCol][1]),
                            nrow=length(sigInd), byrow=FALSE)
    mat[sigInd, isCurrCol] <-  replaceMatrix
    }
  return(mat)
}

addNoise = function(matrix, fractionAffected=.1, stdv=1) {
  noise = matrix(rnorm(nrow(matrix)*ncol(matrix), sd=stdv), nrow=nrow(matrix), byrow=FALSE)
  addNoise = matrix(runif(nrow(matrix)*ncol(matrix)), nrow=nrow(matrix), byrow=FALSE) < fractionAffected
  matrix = matrix + addNoise*noise
  return(matrix)
}

do.test <- function(senseMat, tissueInd, test=t.test, alternative="greater") {
  bgInds = setdiff(1:nrow(senseMat), tissueInd)
  res = apply(senseMat, 2, function(col) {
    gs = col[tissueInd]
    bg = col[bgInds]
    return(test(gs, bg, alternative=alternative)$p.value)
  })
  return(res)
}


selGeneSet <- "Ovary_NGS_RNASEQATLAS_0.6_3"
selSignature <- gmt[[selGeneSet]]$genes
tissueInds <- sapply(gmt, function(x) match(x$genes, humanGenes))
senseAmplitudes <- rep(c(seq(0, 1, by=0.25), seq(1.5, 3, by=0.5)), each=50)
senseMat <- randomMatrixButOneSignature(rows=humanGenes,
                                        signatureGenes=selSignature,
                                        amplitudes=senseAmplitudes)
senseMatOutlier = addNoise(senseMat, stdv=15, fractionAffected=.01)

compare.tests = function(senseMat) {
  senseBioQC <- wmwTest(senseMat, tissueInds, valType="p.greater", simplify=TRUE)[selGeneSet,]
  senseTTest <- do.test(senseMat, tissueInds[[selGeneSet]], test=t.test)
  senseKS <- do.test(senseMat, tissueInds[[selGeneSet]], test=function(x, y, alternative) {
    ks.test(y, x, alternative=alternative)})
  
  comp = data.frame(BioQC=-log10(senseBioQC), t.test=-log10(senseTTest), ks.test=-log10(senseKS), senseAmplitudes=senseAmplitudes)
  comp.molten = melt(comp, id="senseAmplitudes")
  comp.summary = summarySE(comp.molten, measurevar="value", groupvars=c("senseAmplitudes", "variable"))
  return(comp.summary)
}

comp.summary = compare.tests(senseMat)
comp.summary.outlier = compare.tests(senseMatOutlier)

plot.comp = ggplot(comp.summary, aes(x=senseAmplitudes, y=value, color=variable)) + 
  geom_line() +
  geom_errorbar(aes(ymin=value-se, ymax=value+se), width=.1) + 
  geom_point() + xlab("Mean expression differnce") + ylab("Enrichment score") + ggtitle("without noise") + theme_bw() + scale_color_brewer(palette="Dark2")

plot.comp.outlier = ggplot(comp.summary.outlier, aes(x=senseAmplitudes, y=value, color=variable)) + 
  geom_line() +
  geom_errorbar(aes(ymin=value-se, ymax=value+se), width=.1) + 
  geom_point() + xlab("Mean expression differnce") + ylab("Enrichment score") + ggtitle("with noise") + theme_bw()  + scale_color_brewer(palette="Dark2")

grid.arrange(plot.comp, plot.comp.outlier, ncol=2)

## ----benchmark, include=FALSE-------------------------------------------------
runWMW = function() {return(wmwTest(senseMat, tissueInds, valType="p.greater", simplify=TRUE))}
runKS = function() {return(do.test(senseMat, tissueInds[[selGeneSet]], test=function(x, y, alternative) {
    ks.test(y, x, alternative=alternative)}))}
benchmark.res = benchmark(runWMW(), runKS(), columns=c("test", "replications", "elapsed", "relative"), replications=5)

## ----benchmark_res, echo=FALSE------------------------------------------------
benchmark.res

## ----session_info-------------------------------------------------------------
sessionInfo()

library(BioQC)

context("Test IndexList")

testList <- list(GS_A=c(1,2,3,4,3),
                 GS_B=c(2,3,4,5),
                 GS_C=NULL,
                 GS_D=c(1,3,5,NA,4,5),
                 GS_E=c(2,4))
expList <- list(GS_A=c(1,2,3,4),
                GS_B=c(2,3,4,5),
                GS_C=NULL,
                GS_D=c(1,3,5,4),
                GS_E=c(2,4))
expListNA <- list(GS_A=c(1,2,3,4),
                  GS_B=c(2,3,4,5),
                  GS_C=NULL,
                  GS_D=c(1,3,5, NA ,4),
                  GS_E=c(2,4))
testIndexList <- IndexList(testList, offset=1L)
testIndexListNA <- IndexList(testList, offset=1L, keepNA=TRUE)

test_that("test IndexList", {
              expect_equivalent(expList, testIndexList@.Data)
              expect_equivalent(expListNA, testIndexListNA@.Data)
              expect_equal(offset(testIndexList), 1L)
          })
test_that("test IndexList: offset 0", {
              offset(testIndexList) <- 0L
              expect_equivalent(testIndexList@.Data,
                                list(0:3,
                                     1:4,
                                     NULL,
                                     c(0L, 2L, 4L, 3L),
                                     c(1L, 3L)))
          })
test_that("test IndexList: offset 2", {
              offset(testIndexList) <- 2L
              expect_equivalent(testIndexList@.Data,
                                list(2:5,
                                     3:6,
                                     NULL,
                                     c(2L, 4L, 6L, 5L),
                                     c(3L, 5L)))
          })

testLogical <- c(FALSE, TRUE, FALSE, TRUE)
testLogIndexList <- IndexList(testLogical)
testIntIndexList <- IndexList(which(testLogical))
test_that("test IndexList from logical vector", {
    expect_equivalent(testLogIndexList@.Data, list(c(2,4)))
    expect_equivalent(testIntIndexList@.Data, list(c(2,4)))
})

context("Test SignedIndexList")

inputSil <- list("GS_A"=list(pos=1:3, neg=NULL),
                 "GS_B"=list(pos=1:3, neg=4:7),
                 "GS_C"=list(pos=NULL, neg=c(3L,5L,NA)),
                 "GS_D"=list(pos=c(1L,3L,3L,4L), neg=c(3L,5L,NA)))
resSil <- SignedIndexList(inputSil)
test_that("SignedIndexList from list", {
    expect_equal(names(resSil), paste("GS", LETTERS[1:4], sep="_"))
    expect_equal(resSil@keepNA, FALSE)
    expect_equal(resSil@keepDup, FALSE)
    expect_equal(offset(resSil), 1L)
    expect_equivalent(resSil@.Data, list(list(pos=1:3, neg=NULL),
                                         list(pos=1:3, neg=4:7),
                                         list(pos=NULL, neg=c(3L, 5L)),
                                         list(pos=c(1L, 3L, 4L), neg=c(3L, 5L))))
})

test_that("SignedIndexList: offset 0", {
    offset(resSil) <- 0L
    expect_equivalent(resSil@.Data, list(list(pos=0:2, neg=NULL),
                                         list(pos=0:2, neg=3:6),
                                         list(pos=NULL, neg=c(2L, 4L)),
                                         list(pos=c(0L, 2L, 3L), neg=c(2L, 4L))))
})

test_that("SignedIndexList: offset 2", {
    offset(resSil) <- 2L
    expect_equivalent(resSil@.Data, list(list(pos=2:4, neg=NULL),
                                         list(pos=2:4, neg=5:8),
                                         list(pos=NULL, neg=c(4L, 6L)),
                                         list(pos=c(2L, 4L, 5L), neg=c(4L, 6L))))
})

context("Test matchGene for SignedGenesets")

# test rank-biserial correlation and common language effect size
library(BioQC)

context("Test correspondance between U and rank-biserial correlation")
n1 <- 300
n2 <- 200
rd <- c(rnorm(n1), rnorm(n2, -0.2))
ind <- c(rep(FALSE, n1), rep(TRUE, n2))

# one-dimensional vector
U <- wilcox.test(rd[ind], rd[!ind])$statistic
U_f <- U / n1 / n2
U_r <- 2 * U / n1 / n2 - 1
wmw_r <- wmwTest(rd, ind, valType = "r")
wmw_f <- wmwTest(rd, ind, valType = "f")


## matrix form
rmat <- matrix(c(rd, rd + rnorm(length(rd)), rd + rnorm(length(rd))), ncol = 3, byrow = FALSE)

mat_U <- apply(rmat, 2, function(v) wilcox.test(v[ind], v[!ind])$statistic)
mat_U_f <- mat_U / n1 / n2
mat_U_r <- 2 * mat_U / n1 / n2 - 1
wmw_mat_r <- wmwTest(rmat, ind, valType = "r")
wmw_mat_f <- wmwTest(rmat, ind, valType = "f")

test_that("Common language effect size and rank-biserial correlation values are compatible", {
  expect_equivalent(wmw_r, 2 * wmw_f - 1)
  expect_equivalent(wmw_mat_r, 2 * wmw_mat_f - 1)
})

test_that("Common language effect size corresponds to U value", {
  expect_equivalent(wmw_f, U_f)
  expect_equivalent(wmw_mat_f, mat_U_f)
})

test_that("Rank-biserial correlation corresponds to U value", {
  expect_equivalent(wmw_r, U_r)
  expect_equivalent(wmw_mat_r, mat_U_r)
})

# test U1 and U2 values
library(BioQC)

context("Test correspondance between U, U1, U2 and n1 * n2")

set.seed(10)
n1 <- sample(100, 1)
n2 <- sample(100, 1)
x <- rnorm(n1)
y <- rnorm(n2)

W_U <- wilcox.test(x, y)$statistic
W_U_inv <- wilcox.test(y, x)$statistic


U <- wmwTest(c(x, y), seq_along(x), valType = "U")
U1 <- wmwTest(c(x, y), seq_along(x), valType = "U1")
U2 <- wmwTest(c(x, y), seq_along(x), valType = "U2")

U_inv <- wmwTest(c(y, x), seq_along(y), valType = "U")
U1_inv <- wmwTest(c(y, x), seq_along(y), valType = "U1")
U2_inv <- wmwTest(c(y, x), seq_along(y), valType = "U2")


## matrix form
set.seed(11)
ind <- c(rep(FALSE, n1), rep(TRUE, n2))
nc <- 10
rmat <- matrix(rnorm(length(ind) * nc), ncol = nc)

mat_W_U <- apply(rmat, 2, function(v) wilcox.test(v[ind], v[!ind])$statistic)
mat_W_U_inv <- apply(rmat, 2, function(v) wilcox.test(v[!ind], v[ind])$statistic)

mat_U <- wmwTest(rmat, ind, valType = "U")
mat_U1 <- wmwTest(rmat, ind, valType = "U1")
mat_U2 <- wmwTest(rmat, ind, valType = "U2")

mat_U_inv <- wmwTest(rmat, !ind, valType = "U")
mat_U1_inv <- wmwTest(rmat, !ind, valType = "U1")
mat_U2_inv <- wmwTest(rmat, !ind, valType = "U2")

test_that("Test that U equals to U1", {
  expect_equal(U, U1)
  expect_equal(U_inv, U1_inv)
  expect_equal(mat_W_U, mat_U1)
  expect_equal(mat_W_U_inv, mat_U1_inv)
})

test_that("Test that U1 + U1 equals n1 * n2", {
  expect_equal(U1 + U2, n1 * n2)
  expect_equal(U1_inv + U2_inv, n1 * n2)
  expect_equal(mat_U1 + mat_U2, rep(n1 * n2, nc))
  expect_equal(mat_U1_inv + mat_U2_inv, rep(n1 * n2, nc))
})

test_that("Test that U1 corresponds to U2 if x and y are exchanged", {
  expect_equal(U1, U2_inv)
  expect_equal(U2, U1_inv)
  expect_equal(mat_U1, mat_U2_inv)
  expect_equal(mat_U2, mat_U1_inv)
})

test_that("Test that W from the Wilcoxon test equals to U1", {
  expect_equivalent(U1, W_U)
  expect_equivalent(U1_inv, W_U_inv)
  expect_equivalent(mat_U1, mat_W_U)
  expect_equivalent(mat_U1_inv, mat_W_U_inv)
})

## test out-of-range errors
library(BioQC)

mat <- matrix(rnorm(100), nrow=10)

## well-behaving indices
wbids <- list(gs1=c(1,3,5),
	 gs2=c(1),
	 gs3=1:10)
wmwTest(mat, wbids)

test_that("index out of range errors are found", {
  expect_error(wmwTest(mat, -1),
               "Index out of range: gene set 1, gene 1")
  expect_error(wmwTest(mat, 11),
               "Index out of range: gene set 1, gene 1")
  expect_error(wmwTest(mat, list(1, -1)),
               "Index out of range: gene set 2, gene 1")
	       })

---
title: "BioQC: Detect tissue heterogeneity in gene expression data"
author: "Jitao David Zhang, Klas Hatje, Gregor Sturm, Clemens Broger, Martin Ebeling, and Laura Badi"
date: "`r Sys.Date()`"
vignette: >
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteIndexEntry{BioQC: Detect tissue heterogeneity in gene expression data}
  %\usepackage[utf8]{inputenc}
output:
  rmarkdown::html_vignette:
    self_contained: no
  md_document:
    variant: markdown_phpextra
    preserve_yaml: TRUE 
---

```{r setup, include=FALSE}
library(knitr)
opts_chunk$set(out.extra='style="display:block; margin: auto"', fig.align="center")
```

Introduction
------------

The Wilcoxon-Mann-Whitney rank sum test is one of the most commonly used algorithms for non-parametric tests. The current implementations in R however are not optimal for multiple hypothesis testing. The *BioQC* package implements the algorithm in a native C routine to allow fast computation in scenarios where a numeric vector (*e.g.* expression values of all genes) is compared against a collection of subsets of the vector (*e.g.* expression values of genes belonging to a collection of gene sets). We demonstrate the use of the package with the *BioQC* algorithm, which uses the Wilcoxon-Mann-Whitney rank sum test and a collection of tissue-preferentially gene signatures to detect tissue heterogeneity in high-throughput gene expression studies. 

In this vignette we demonstrate the use of *BioQC* by a simulated expression dataset, and compare the performance of Wilcoxon-Mann-Whitney rank sum test algorithm implemented in *BioQC* to other implementations available in R. To use *BioQC*, the users only need to provide an expression dataset, in the form of a numeric matrix, or an *ExpressionSet* object. The *BioQC* package provides tissue-specific genes that can be used directly with the algorithm. The output is one score for each tissue type and each sample. The ranks of the score within each sample can be compared with prior knowledge about the sample to infer tissue heterogeneity. The hypotheses generated by *BioQC* should be further tested in follow-up experiments.



A dummy example
---------------
We demonstrate the basic use of the package with a dummy example. First, we load *BioQC* library and the tissue signatures into the R session.

```{r seed, include=FALSE}
set.seed(1887)
```

```{r lib, warning=FALSE, message=FALSE, results="hide"}
library(Biobase)
library(BioQC)
gmtFile <- system.file("extdata/exp.tissuemark.affy.roche.symbols.gmt", package="BioQC")
gmt <- readGmt(gmtFile)
```

Next, we synthesize an ExpressionSet object, using randomly generated data.

```{r gmtReadin}
Nrow <- 2000L
Nsample <- 5L
gss <- unique(unlist(sapply(gmt, function(x) x$genes)))
myEset <- new("ExpressionSet",
              exprs=matrix(rnorm(Nrow*Nsample), nrow=Nrow),
              featureData=new("AnnotatedDataFrame",
                              data.frame(GeneSymbol=sample(gss, Nrow))))
```

Finally we run the *BioQC* algorithm and print the summary of the results. As expected, no single tissue scored significantly after multiple correction.

```{r runBioQC}
dummyRes <- wmwTest(myEset, gmt, valType="p.greater", simplify=TRUE)
summary(p.adjust(dummyRes, "BH"))
```



### Using basic data structures
The dummy example above shows how to run *BioQC* algorithm with an *ExpressionSet* and a list read from a GMT file (a file format capturing gene sets). Users can also use more basic data structures (*e.g.* matrices and list of integer indexes) to run the algorithm as shown by the following example. Other data structures will be coerced into these basic data structures; we refer the interested user to the documentation of the *wmwTest* function.

```{r basicDs}
myMatrix <- matrix(rnorm(Nrow*Nsample),
                   ncol=Nsample,
                   dimnames=list(NULL, LETTERS[1:Nsample]))
myList <- list(signature1=sample(1:Nrow, 100),
               signature2=sample(1:Nrow, 50),
               signature3=sample(1:Nrow, 200))
wmwTest(myMatrix, myList, valType="p.greater", simplify=TRUE)
```


Case study with real dataset
----------------------------
We have applied *BioQC* to a real gene expression profiling dataset. *BioQC* helped to generated
hypotheses about potential contamination of rat kidney examples by pancreas tissues, which could be confirmed with qRT-PCR. 

See the [BioQC-kidney](https://accio.github.io/BioQC/articles/BioQC.html) vignette for more details.


Benchmarking against R implementation
-------------------------------------
In the core of *wmwTest*, an efficient C-implementation makes it feasible to run a large number of Wilcoxon-Mann-Whitney tests on large-scale expression profiling datasets and with many signature lists. Compared to native R implementations in *stats* (*wilcox.text*) and *limma* (*rankSumTestWithCorrelation*) packages, the *BioQC* implementation requires less memory and avoids repetitive statistical ranking of data.

The following code, though in a small scale, demonstrates the difference between the performances of two implementations.

```{r benchmark}
bm.Nrow <- 22000
bw.Nsample <- 5
bm.Ngs <- 5
bm.Ngssize <- sapply(1:bm.Ngs, function(x) sample(1:bm.Nrow/2, replace=TRUE))
ind <- lapply(1:bm.Ngs, function(i) sample(1:bm.Nrow, bm.Ngssize[i]))
exprs <- matrix(round(rnorm(bm.Nrow*bw.Nsample),4), nrow=bm.Nrow)

system.time(Cres <- wmwTest(exprs, ind, valType="p.less", simplify=TRUE))
system.time(Rres <- apply(exprs, 2, function(x)
                          sapply(ind, function(y)
                                 wmwTestInR(x, y, valType="p.less"))))

```

With 22000 genes, five samples, and five gene sets, the *BioQC* implementation is about 20x faster than the R implementation (dependent on individual machines and settings). Our benchmark shows that with the same number of genes, 2000 samples and 200 gene sets (similar to the total number of tissues collected in the *BioQC* signature list), the *BioQC* implementation can be about 1000x faster than the R implementation.

Technical notes
----------------

### Background matters

Even using the same set of signatures, the *p*-value of enrichment may vary depending on the background. A toy example is shown below: assuming that in a gene expression profile with values of 20000 genes, there are half of them lower expressed than the other half, mimicking the commonly observed patterns that some genes are almost not or very low expressed; depending on whether including these lowly expressed genes or not, the *p*-value reported by the Wilcoxon's test varies.

```{r backgroundMatters}
bgVec <- rnorm(20000)
bgVec[1:10000] <- bgVec[1:10000] + 2
bgVec[1:10] <- bgVec[1:10] + 1
ind <- c(1:10)

(pAllGenes <- wmwTest(bgVec, ind))
(pHighExpGenes <- wmwTest(bgVec[1:10000], ind))
```

It is therefore recommended to report any filtering prior to the BioQC analysis to make the results reproducible.

Acknowledgement
----------------
We would like to thank Iakov Davydov for suggestions to improve the package.

R Session Info
----------------
```{r session_info}
sessionInfo()
```

---
title: "BioQC-kidney: The kidney expression example"
date: "`r Sys.Date()`"
vignette: >
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteIndexEntry{BioQC-kidney: The kidney expression example}
  %\usepackage[utf8]{inputenc}
output:
  rmarkdown::html_vignette:
    self_contained: no
  md_document:
    variant: markdown_phpextra
    preserve_yaml: TRUE
---

```{r setup, include=FALSE}
options(fig_caption=TRUE)
library(knitr)
opts_chunk$set(out.extra='style="display:block; margin: auto"', fig.align="center")
```

Introduction
------------
In this vignette, we demonstrate the use of *BioQC* with a case study where mouse kidney samples were profiled for gene expression.
Results of BioQC pointed to potential tissue heterogeneity caused by pancreas contamination which was confirmed by qRT-PCR experiments.


Importing the data
------------------

Let's first load the BioQC signatures using `readGmt`: 

```{r lib, warning=FALSE, message=FALSE, results="hide"}
library(BioQC)
gmtFile <- system.file("extdata/exp.tissuemark.affy.roche.symbols.gmt",
                        package="BioQC")
gmt <- readGmt(gmtFile)
```

The example data for this vignette is shipped as part of the BioQC package. 
Alternatively, it is available in the [GitHub repository](https://github.com/accio/BioQC/inst/extdata/kidney_example.rda). 
The expression data is stored in a [ExpressionSet](https://www.bioconductor.org/packages/release/bioc/vignettes/Biobase/inst/doc/ExpressionSetIntroduction.pdf). 

```{r load_data, warning=FALSE, message=FALSE, results="hide"}
file <- system.file("extdata/kidney_example.rda", package="BioQC")
load(file)
print(eset)
```

The dataset contains expression of `r nrow(eset)` genes in `r ncol(eset)` samples. The expression profile was normalized with RMA normalization. The signals were  also `log2`-transformed; however, this step does not affect the result of *BioQC* since it is essentially a non-parametric statistical test.


Running BioQC
-------------
Next we run the core function of the *BioQC* package, `wmwTest`, to perform the analysis.

```{r run_bioqc}
system.time(bioqcRes <- wmwTest(eset, gmt,
                                 valType="p.greater"))
```

The function returns *one-sided* $p$-values of Wilcoxon-Mann-Whitney test. 

For better visualization we 
 * filter the p-value matrix (a sensible cutoff depends on the actual dataset and the expected signatures);
 * transform the p-values using $|\log_{10} p|$. We refer to thse transformed values as *BioQC score*. 

```{r visualize_bioqc_prepare}
bioqcResFil <- filterPmat(bioqcRes, 1E-6)
bioqcAbsLogRes <- absLog10p(bioqcResFil)
```

We inspect the BioQC scores by visualizing them as Heatmap. 
By closer examination we find that the expression of pancreas and adipose specific genes is significantly enriched in samples 23-25: 

```{r heatmap, fig.width=8, fig.height=4, dev='png', fig.cap="BioQC scores (defined as abs(log10(p))) of the samples visualized in heatmap. Red and blue indicate high and low scores respectively."}
library(RColorBrewer)
heatmap(bioqcAbsLogRes, Colv=NA, Rowv=TRUE,
        cexRow=0.85, scale="row",
        col=rev(brewer.pal(7, "RdBu")),
        labCol=1:ncol(bioqcAbsLogRes))
```

Visual inspection reveals that there might be contaminations in samples 23-25, potentially by pancreas and adipose tissue. 
```{r vis_bioqc, fig.width=8, fig.height=4, dev='png', fig.cap="BioQC scores (defined as abs(log10(p))) of the samples. K and P represent kidney and pancreas signature scores respectively."}
filRes <- bioqcAbsLogRes[c("Kidney_NGS_RNASEQATLAS_0.6_3",
                           "Pancreas_Islets_NR_0.7_3"),]
matplot(t(filRes), pch=c("K", "P"), type="b", lty=1L,
        ylab="BioQC score", xlab="Sample index")
```




Validation with quantitative RT-PCR
-----------------------------------
To confirm the hypothesis generated by *BioQC*, we performed qRT-PCR experiments to test two pancreas-specific genes' expression in the same  set of samples. Note that the two genes (amylase and elastase) are not included in the signature set provided by BioQC.

The results are shown in the figure below. It seems likely that sample 23-25 are contaminated by nearby pancreas tissues when the kidney was dissected. Potential contamination by adipose tissues remains to be tested.

```{r rt_pcr_result, fig.width=8, fig.height=4, dev='png', fig.cap="Correlation between qRT-PCR results and BioQC pancreas score"}
amylase <- eset$Amylase
elastase <- eset$Elastase
pancreasScore <- bioqcAbsLogRes["Pancreas_Islets_NR_0.7_3",]
par(mfrow=c(1,2), mar=c(3,3,1,1), mgp=c(2,1,0))
plot(amylase~pancreasScore, log="y", pch=21, bg="red",
     xlab="BioQC pancreas score", ylab="Amylase")
text(pancreasScore[23:25],amylase[23:25], 23:25, pos=1)
plot(elastase~pancreasScore, log="y", pch=21, bg="red",
     xlab="BioQC pancreas score", ylab="Elastase")
text(pancreasScore[23:25],elastase[23:25], 23:25, pos=1)

```



Impact of sample removal on differential gene expression analysis
-----------------------------------------------------------------
In this study, four mice of the *FVB/NJ* strain received nephrectomy operation and treatment of Losartan, an angiotensin II receptor antagonist drug, and four mice reveiced an sham operation and Losartan. Within the Nephrectomy+Losartan group, one sample (index 24) is possibly contaminated by pancreas. Suppose now we are interested in the differential gene expression between the conditions. We now run the analysis twice, once with and once without the contaminated sample, to study the impact of removing heterogenous samples detected by *BioQC*.

```{r load_limma, warning=FALSE, message=FALSE, results="hide"}
library(limma)
```

```{r deg, fig.width=5, fig.height=5, dev='png', fig.cap="Log2 fold change (logFC) values reported by *limma* with one contaminated sample included (y-axis) or excluded (x-axis). Genes strongly affected by the contamination are indicated by red dots."}

isNeph <- with(pData(eset), Strain=="FVB/NJ" &
                 TREATMENTNAME %in% c("Nephrectomy-Losartan", "Sham-Losartan"))
isContam <- with(pData(eset), INDIVIDUALNAME %in% c("BN7", "FNL8", "FN6"))
esetNephContam <- eset[,isNeph]
esetNephExclContam <- eset[, isNeph & !isContam]
getDEG <- function(eset) {
  group <- factor(eset$TREATMENTNAME, levels=c("Sham-Losartan","Nephrectomy-Losartan"))
  design <- model.matrix(~group)
  colnames(design) <- c("ShamLo", "NephLo")
  contrast <- makeContrasts(contrasts="NephLo", levels=design)
  exprs(eset) <- normalizeBetweenArrays(log2(exprs(eset)))
  fit <- lmFit(eset, design=design)
  fit <- contrasts.fit(fit, contrast)
  fit <- eBayes(fit)
  tt <- topTable(fit, n=nrow(eset))
  return(tt)
}

esetNephContam.topTable <- getDEG(esetNephContam)
esetNephExclContam.topTable <- getDEG(esetNephExclContam)
esetFeats <- featureNames(eset)
esetNephTbl <- data.frame(feature=esetFeats,
                          OrigGeneSymbol=esetNephContam.topTable[esetFeats,]$OrigGeneSymbol,
                          GeneSymbol=esetNephContam.topTable[esetFeats,]$GeneSymbol,
                          Contam.logFC=esetNephContam.topTable[esetFeats,]$logFC,
                          ExclContam.logFC=esetNephExclContam.topTable[esetFeats,]$logFC)
par(mfrow=c(1,1), mar=c(3,3,1,1)+0.5, mgp=c(2,1,0))
with(esetNephTbl, smoothScatter(Contam.logFC~ExclContam.logFC,
                                xlab="Excluding one contaminating sample [logFC]",
                                ylab="Including one contaminating sample [logFC]"))
abline(0,1)
isDiff <- with(esetNephTbl, abs(Contam.logFC-ExclContam.logFC)>=2)
with(esetNephTbl, points(Contam.logFC[isDiff]~ExclContam.logFC[isDiff], pch=16, col="red"))
diffTable <- esetNephTbl[isDiff,]
diffGenes <- unique(diffTable[,"GeneSymbol"])
pancreasSignature <- gmt[["Pancreas_Islets_NR_0.7_3"]]$genes
diffGenesPancreas <- diffGenes %in% pancreasSignature
diffTable$isPancreasSignature <- diffTable$GeneSymbol %in% pancreasSignature
colnames(diffTable) <- c("Probeset", "GeneSymbol", "Human ortholog",
                         "Log2FC", "Log2FC (excl. contam.)",
                         "IsPancreasSignature")
diffTable <- diffTable[order(diffTable$Log2FC, decreasing=TRUE),]
```

We found that `r nrow(diffTable)` probesets representing `r length(diffGenes)` genes are associated with much stronger expression changes if the contaminated sample is not excluded (table below). Not surprisingly almost all of these genes are highly expressed in normal human pancreas tissues, and `r sum(diffGenesPancreas)` genes belong to the pancreas signature used by *BioQC*. 

```{r kable_deg, include=FALSE}
kable(diffTable, caption="Genes that are identified as strongly changed ony if the contaminated sample is included.")
```

In summary, we observe that tissue heterogeneity can impact down-stream analysis results and negatively affect reproducibility of gene expression data if it remains overlooked. It underlines again the value of applying *BioQC* as a first-line quality control tool.




R Session Info
----------------
```{r session_info}
sessionInfo()
```

---
title: "BioQC Algorithm: Speeding up the Wilcoxon-Mann-Whitney Test"
author: "Gregor Sturm and Jitao David Zhang"
package: BioQC
date: "`r Sys.Date()`"
vignette: >
  %\VignetteIndexEntry{BioQC Algorithm: Speeding up the Wilcoxon-Mann-Whitney Test}
  %\VignetteEngine{knitr::rmarkdown}
  %\usepackage[utf8]{inputenc}
output: 
  rmarkdown::html_vignette:
    self_contained: no
  md_document:
    variant: markdown_phpextra
    preserve_yaml: TRUE
---

```{r setup, include=FALSE}
options(fig_caption=TRUE)
library(knitr)
opts_chunk$set(out.extra='style="display:block; margin: auto"', fig.align="center")
```

In this vignette, we explain the underlaying algorithmic details of our implementation of the Wilcoxon-Mann-Whitney test. The source code used to produce this document can be found in the github repository [BioQC](https://github.com/Accio/BioQC/vignettes).

*BioQC* is a R/Bioconductor package to detect tissue heterogeneity from high-throughput gene expression profiling data. It implements an efficient Wilcoxon-Mann-Whitney test, and offers tissue-specific gene signatures that are ready to use 'out of the box'.


```{r lib, warning=FALSE, message=FALSE, results="hide", include=FALSE}
library(testthat)
library(BioQC)
library(org.Hs.eg.db) ## to simulate an microarray expression dataset
library(lattice)
library(latticeExtra)
library(gridExtra)
library(gplots)
library(rbenchmark)

pdf.options(family="ArialMT", useDingbats=FALSE)

set.seed(1887)

## list human genes
humanGenes <- keys(revmap(org.Hs.egSYMBOL))

## read tissue-specific gene signatures
gmtFile <- system.file("extdata/exp.tissuemark.affy.roche.symbols.gmt",
                       package="BioQC")
gmt <- readGmt(gmtFile)
```


Algorithmic improvements
------------------------

The Wilcoxon-Mann-Whitney (WMW) test is a non-parametric statistical test to test if median values of two population are equal or not. Unlike the t-test, it does not require the assumption of normal distributions, which makes it more robust against noise. 

We improved the computational efficiency of the Wilcoxon-Mann-Whitney test in comparison to the native R implementation based on three modifications:

1. The approximative WMW-statistic (Zar, J. H. (1999). Biostatistical analysis. Pearson Education India. *pp.* 174-177) is used. The differences to the exact version are negligible for high-throughput gene expression data. 
2. The core algorithm is implemented in C instead of R as programming language.
3. BioQC avoids futile expensive sorting operations.

While (1) and (2) are straightforward, we elaborate (3) in the following. 

Let $W_{a,b}$ be the approximative WMW test of two gene vectors $a,b$, where $a$ is the gene set of interest, typically containing less than a few hundreds of genes, and $b$ is the set of all genes outside the gene set (*background genes*) typically containing $>10000$ genes. In the context of BioQC, the gene sets are referred to as *tissue signatures*.  

Given an  $m \times n$ input matrix of gene expression data with $m$ genes and $n$ samples $s_1, \dots, s_n$, and $k$ gene sets $d_1, \dots, d_k$, the WMW-test needs to be applied for each sample $s_i, i \in 1..n$ and each gene set $d_j, j \in 1..k$. The runtime of the WMW-test is essentially determined by the sorting operation on the two input vectors. Using native R `wilcox.test`, the vectors $a$ and $b$ are sorted individually for each gene set. However, in the context of gene set analysis, this is futile, as the (large) background set changes insignificantly in relation to the (small) gene set, when testing different gene sets on the same sample. 

Therefore, we approximate the WMW-test by extending $b$ to all genes in the sample, keeping the background unchanged when testing multiple gene sets. Like this, $b$ has to be sorted only once per sample.
The individual gene sets still need to be sorted, which is not a major issue, as they are small in comparison to the set of background genes. 

```{r algofig, echo=FALSE, fig.height=4.5, fig.cap="BioQC speeds up the Wilcoxon-Mann-Whitney test by avoiding futile sorting operations on the same sample."}
knitr::include_graphics("wmw-speedup.png")
```


Time benchmark
--------------

To demonstrate BioQC's superior performance, we apply both BioQC and the native R `wilcox.test` to random expression matrices and measure the runtime. 

We setup random expression matrices of `r length(humanGenes)` human genes of 1, 5, 10, 50, or 100 samples. Genes are $i.i.d$ distributed following $\mathcal{N}(0,1)$. The native R and the *BioQC* implementations of the Wilcoxon-Mann-Whitney test are applied to the matrices respectively.


```{r time_benchmark, echo=FALSE}
randomMatrix <- function(rows=humanGenes, ncol=5) {
  nrow <- length(rows)
  mat <- matrix(rnorm(nrow*ncol),
                nrow=nrow, byrow=TRUE)
  rownames(mat) <- rows
  return(mat)
}
noSamples <- c(1, 5, 10, 20, 50, 100)
noBenchRep <- 100
tmRandomMats <- lapply(noSamples, function(x) randomMatrix(ncol=x))
tissueInds <- sapply(gmt, function(x) match(x$genes, humanGenes))

wmwTestR <- function(matrix, indices, alternative) {
  res <- apply(matrix, 2, function(x) {
    sapply(indices, function(index) {
      sub <- rep(FALSE, length(x))
      sub[index] <- TRUE
      wt <- wilcox.test(x[sub], x[!sub],
                        alternative=alternative,
                        exact=FALSE)
      return(wt$p.value)
      })
    })
  return(res)
  }

## WARNING: very slow (~1-2 hours)
benchmarkFile <- "simulation-benchmark.RData"
if(!file.exists(benchmarkFile)) {
  bioqcRes <- lapply(tmRandomMats, function(mat) {
    bench <- benchmark(wmwTestRes<- wmwTest(mat,
                                            tissueInds,
                                            valType="p.greater",
                                            simplify=TRUE),
                       replications=noBenchRep)
    elapTime <- c("elapsed"=bench$elapsed,
                  "user"=bench$user.self,
                  "sys"=bench$sys.self)/noBenchRep
    res <- list(elapTime=elapTime,
                wmwTestRes=wmwTestRes)
    return(res)
    })
  
  rRes <- lapply(tmRandomMats, function(mat) {
    elapTime <- system.time(wmwTestRes <- wmwTestR(mat, tissueInds, alternative="greater"))
    res <- list(elapTime=elapTime,
                wmwTestRes=wmwTestRes)
    return(res)
    })
  save(bioqcRes, rRes, file=benchmarkFile)
  } else {
    load(benchmarkFile)
 }
getWmwTestRes <- function(x) x$wmwTestRes
rNumRes <- lapply(rRes, getWmwTestRes)
bioqcNumRes <- lapply(bioqcRes, getWmwTestRes)
```

The numeric results of both implementations, `bioqcNumRes` (from *BioQC*) and `rNumRes` (from *R*), are equivalent, as shown by the next command.

```{r time_benchmark_identical}
expect_equal(bioqcNumRes, rNumRes)
```

The *BioQC* implementation is more than 500 times faster: while it takes about one second for BioQC to calculate enrichment scores of all `r length(gmt)` signatures in 100 samples, the native R implementation takes about 20 minutes: 

```{r trellis_prepare, echo=FALSE}
op <- list(layout.widths = list(left.padding = 0, key.ylab.padding = 0.5,
                                ylab.axis.padding = 0.5, axis.right = 0.5, right.padding = 0),
           layout.heights = list(top.padding = 0, bottom.padding = 0,
                                 axis.top = 0, main.key.padding = 0.5, key.axis.padding = 0.5),
           axis.text=list(cex=1.2),
           par.xlab.text=list(cex=1.4),
           par.sub.text=list(cex=1.4),
           add.text=list(cex=1.4),
           par.ylab.text=list(cex=1.4))
```

```{r time_benchmark_vis, echo=FALSE, fig.width=8, fig.height=4.5, dev='png', dev.args=list(pointsize=2.5), fig.cap="**Figure 2**: Time benchmark results of BioQC and R implementation of Wilcoxon-Mann-Whitney test. Left panel: elapsed time in seconds (logarithmic Y-axis). Right panel: ratio of elapsed time by two implementations. All results achieved by a single thread on in a RedHat Linux server."}
getTimeRes <- function(x) x$elapTime["elapsed"]
bioqcTimeRes <- sapply(bioqcRes, getTimeRes)
rTimeRes <- sapply(rRes, getTimeRes)
timeRes <- data.frame(NoSample=noSamples,
                      Time=c(bioqcTimeRes, rTimeRes),
                      Method=rep(c("BioQC", "NativeR"), each=length(noSamples)))

timeXY <- xyplot(Time ~ NoSample, group=Method, data=timeRes,  type="b",
                 auto.key=list(columns=2L),
                 xlab="Number of samples", ylab="Time [s]",
                 par.settings=list(superpose.symbol=list(cex=1.25, pch=16, col=c("black", "red")),
                                   superpose.line=list(col=c("black", "red"))),
                 scales=list(tck=c(1,0), alternating=1L,
                             x=list(at=noSamples),
                             y=list(log=2, at=10^c(-2, -1, 0,1,2,3, log10(2000)), labels=c(0.01, 0.1, 1, 10,100,1000, 2000))))
timeFactor <- with(timeRes,
                   tapply(1:nrow(timeRes),
                          list(NoSample),  function(x) {
                            bioqcTime <- subset(timeRes[x,], Method=="BioQC")$Time
                            rTime <- subset(timeRes[x,], Method=="NativeR")$Time
                            rTime/bioqcTime
                            }))
timeFactor.yCeiling <- max(ceiling(timeFactor/500))*500
timeFactorBar <- barchart(timeFactor ~ noSamples, horizontal=FALSE,
                          xlab="Number of samples", ylab="Ratio of elapsed time [R/BioQC]",
                          ylim=c(-20, timeFactor.yCeiling+50), col=colorRampPalette(c("lightblue", "navyblue"))(length(noSamples)),
                          scales=list(tck=c(1, 0), alternating=1L,
                                      y=list(at=seq(0,timeFactor.yCeiling, by=500)),
                                      x=list(at=seq(along=timeFactor), labels=noSamples)))

grid.arrange(timeXY, timeFactorBar, ncol=2)

```


Conclusion
----------
We have shown that *BioQC* achieves identical results as the native implementation in two orders of magnitude less time. This renders *BioQC* a highly efficient tool for quality control of large-scale high-throughput gene expression data.


R Session Info
----------------
```{r session_info}
sessionInfo()
```

---
title: "Comparing the Wilcoxon-Mann-Whitney to alternative statistical tests"
date: "`r Sys.Date()`"
vignette: >
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteIndexEntry{Comparing the Wilcoxon-Mann-Whitney to alternative statistical tests}
  %\usepackage[utf8]{inputenc}
output:
  rmarkdown::html_vignette:
    self_contained: no
  md_document:
    variant: markdown_phpextra
    preserve_yaml: TRUE
---

```{r setup, include=FALSE}
options(fig_caption=TRUE)
library(knitr)
opts_chunk$set(out.extra='style="display:block; margin: auto"', fig.align="center")
```

In this document, we show that the Wilcoxon-Mann-Whitney test is comparable or superior to alternative methods. 


```{r lib, warning=FALSE, message=FALSE, results="hide", include=FALSE}
library(testthat)
library(BioQC)
library(hgu133plus2.db) ## to simulate an microarray expression dataset
library(lattice)
library(latticeExtra)
library(gridExtra)
library(gplots)
library(reshape2)
library(plyr)
library(ggplot2)
library(rbenchmark)

pdf.options(family="ArialMT", useDingbats=FALSE)

set.seed(1887)

## list human genes
humanGenes <- unique(na.omit(unlist(as.list(hgu133plus2SYMBOL))))

## read tissue-specific gene signatures
gmtFile <- system.file("extdata/exp.tissuemark.affy.roche.symbols.gmt",
                       package="BioQC")
gmt <- readGmt(gmtFile)
```

Two alternative methods could be compared with the Wilcoxon-Mann-Whitney (WMW) test proposed by BioQC: the Kolmogorov-Smirnov (KS) test, and the Student’s t-test, or more particularly, the Welch’s test which does not assume equal sample number or equal variance, which is appropriate in the setting of gene expression studies. 

1. It is documented in statistics literature that the WMW test offers a higher power than the Kolmogorov-Smirnov test[^1],[^2]. 
2. Compared with parameterized test methods such as the t-test, the WMW test is (a) resistance to monotone transformation,  (b) suffers less from outliers, and (c) provides higher efficiency when many genes are profiled and the distribution of gene expression deviates from the normal distribution, which are important criteria in genome-wide expression data. 

Based on these considerations, BioQC implements a computationally efficient version of the WMW test. In order not to confuse end-users, no alternative methods are implemented.

Nevertheless, in order to demonstrate the power of WMW test in comparison with the KS-test or the t-test, we performed the sensitivity benchmark described in the [simulation studies](bioqc-simulation.html), for the two alternative tests respectively. 

```{r helper, include=FALSE}
## Summarizes data.
## Gives count, mean, standard deviation, standard error of the mean, and confidence interval (default 95%).
##   data: a data frame.
##   measurevar: the name of a column that contains the variable to be summariezed
##   groupvars: a vector containing names of columns that contain grouping variables
##   na.rm: a boolean that indicates whether to ignore NA's
##   conf.interval: the percent range of the confidence interval (default is 95%)
## 
## Source: http://www.cookbook-r.com/Manipulating_data/Summarizing_data/
summarySE <- function(data=NULL, measurevar, groupvars=NULL, na.rm=FALSE,
        conf.interval=.95, .drop=TRUE) {

# New version of length which can handle NA's: if na.rm==T, don't count them
        length2 <- function (x, na.rm=FALSE) {
            if (na.rm) sum(!is.na(x))
            else       length(x)
        }

# This does the summary. For each group's data frame, return a vector with
# N, mean, and sd
    datac <- ddply(data, groupvars, .drop=.drop,
            .fun = function(xx, col) {
            c(N    = length2(xx[[col]], na.rm=na.rm),
                mean = mean   (xx[[col]], na.rm=na.rm),
                sd   = sd     (xx[[col]], na.rm=na.rm)
             )
            },
            measurevar
            )

# Rename the "mean" column    
        datac <- rename(datac, c("mean" = measurevar))

        datac$se <- datac$sd / sqrt(datac$N)  # Calculate standard error of the mean

# Confidence interval multiplier for standard error
# Calculate t-statistic for confidence interval: 
# e.g., if conf.interval is .95, use .975 (above/below), and use df=N-1
        ciMult <- qt(conf.interval/2 + .5, datac$N-1)
        datac$ci <- datac$se * ciMult

        return(datac)
}
```

```{r sensitivity_benchmark_fig, echo=FALSE, warning=FALSE, fig.width=8, fig.height=3.5, dev='png', fig.cap="**Figure 1:** Sensitivity benchmark. Expression levels of genes in the ovary signature are dedicately sampled randomly from normal distributions with different mean values. The lines show the enrichment score for the Wilcoxon-Mann-Whitney test, the t-test and the Kolmogorov-Smirnov test respectively. In the right panel, outliers were added by adding a random value to 1% of the simulated genes. "}

randomMatrixButOneSignature <- function(rows=humanGenes, signatureGenes,
                                        amplitudes=seq(0, 3, by=0.5)) {
  nrow <- length(rows)
  ncol <- length(amplitudes)
  mat <- matrix(rnorm(nrow*ncol),
                nrow=nrow, byrow=FALSE)
  rownames(mat) <- rows
  sigInd <- na.omit(match(signatureGenes, humanGenes))
  
  colClass <- factor(amplitudes)
  
  for(colInd in unique(colClass)) {
    isCurrCol <- colInd==colClass
    replaceMatrix <- matrix(rnorm(length(sigInd)*sum(isCurrCol),
                                  mean=amplitudes[isCurrCol][1]),
                            nrow=length(sigInd), byrow=FALSE)
    mat[sigInd, isCurrCol] <-  replaceMatrix
    }
  return(mat)
}

addNoise = function(matrix, fractionAffected=.1, stdv=1) {
  noise = matrix(rnorm(nrow(matrix)*ncol(matrix), sd=stdv), nrow=nrow(matrix), byrow=FALSE)
  addNoise = matrix(runif(nrow(matrix)*ncol(matrix)), nrow=nrow(matrix), byrow=FALSE) < fractionAffected
  matrix = matrix + addNoise*noise
  return(matrix)
}

do.test <- function(senseMat, tissueInd, test=t.test, alternative="greater") {
  bgInds = setdiff(1:nrow(senseMat), tissueInd)
  res = apply(senseMat, 2, function(col) {
    gs = col[tissueInd]
    bg = col[bgInds]
    return(test(gs, bg, alternative=alternative)$p.value)
  })
  return(res)
}


selGeneSet <- "Ovary_NGS_RNASEQATLAS_0.6_3"
selSignature <- gmt[[selGeneSet]]$genes
tissueInds <- sapply(gmt, function(x) match(x$genes, humanGenes))
senseAmplitudes <- rep(c(seq(0, 1, by=0.25), seq(1.5, 3, by=0.5)), each=50)
senseMat <- randomMatrixButOneSignature(rows=humanGenes,
                                        signatureGenes=selSignature,
                                        amplitudes=senseAmplitudes)
senseMatOutlier = addNoise(senseMat, stdv=15, fractionAffected=.01)

compare.tests = function(senseMat) {
  senseBioQC <- wmwTest(senseMat, tissueInds, valType="p.greater", simplify=TRUE)[selGeneSet,]
  senseTTest <- do.test(senseMat, tissueInds[[selGeneSet]], test=t.test)
  senseKS <- do.test(senseMat, tissueInds[[selGeneSet]], test=function(x, y, alternative) {
    ks.test(y, x, alternative=alternative)})
  
  comp = data.frame(BioQC=-log10(senseBioQC), t.test=-log10(senseTTest), ks.test=-log10(senseKS), senseAmplitudes=senseAmplitudes)
  comp.molten = melt(comp, id="senseAmplitudes")
  comp.summary = summarySE(comp.molten, measurevar="value", groupvars=c("senseAmplitudes", "variable"))
  return(comp.summary)
}

comp.summary = compare.tests(senseMat)
comp.summary.outlier = compare.tests(senseMatOutlier)

plot.comp = ggplot(comp.summary, aes(x=senseAmplitudes, y=value, color=variable)) + 
  geom_line() +
  geom_errorbar(aes(ymin=value-se, ymax=value+se), width=.1) + 
  geom_point() + xlab("Mean expression differnce") + ylab("Enrichment score") + ggtitle("without noise") + theme_bw() + scale_color_brewer(palette="Dark2")

plot.comp.outlier = ggplot(comp.summary.outlier, aes(x=senseAmplitudes, y=value, color=variable)) + 
  geom_line() +
  geom_errorbar(aes(ymin=value-se, ymax=value+se), width=.1) + 
  geom_point() + xlab("Mean expression differnce") + ylab("Enrichment score") + ggtitle("with noise") + theme_bw()  + scale_color_brewer(palette="Dark2")

grid.arrange(plot.comp, plot.comp.outlier, ncol=2)
``` 


As expected, the results suggest, that both the KS-test and the WMW-test are robust to noise, while the performance of the t-test drops significantly on noisy data. Additionally, the WMW-test appears to be superior to the KS-test for low expression differences. 



Computational Performance
-------------------------

```{r benchmark, include=FALSE}
runWMW = function() {return(wmwTest(senseMat, tissueInds, valType="p.greater", simplify=TRUE))}
runKS = function() {return(do.test(senseMat, tissueInds[[selGeneSet]], test=function(x, y, alternative) {
    ks.test(y, x, alternative=alternative)}))}
benchmark.res = benchmark(runWMW(), runKS(), columns=c("test", "replications", "elapsed", "relative"), replications=5)
```

Since the KS-test is so slow, we did not replicate the sensitivity benchmark from the [simulation studies](bioqc-simulation.html) using the enrichment score rank. While it takes BioQC about `r round(benchmark.res[2, 'elapsed']/benchmark.res[2, 'replications'])` seconds on a single thread to test all 155 signatures, it already takes the KS-test about `r round(benchmark.res[1, 'elapsed']/benchmark.res[1, 'replications'])` seconds to test a single signature. 

```{r benchmark_res, echo=FALSE}
benchmark.res
```

R Session Info
----------------
```{r session_info}
sessionInfo()
```

References
----------
[^1]: Irizarry, Rafael A., et al. "Gene set enrichment analysis made simple."Statistical methods in medical research 18.6 (2009): 565-575.
[^2]: Filion, Guillaume J. "The signed Kolmogorov-Smirnov test: why it should not be used." GigaScience 4.1 (2015): 1.

---
title: Using BioQC with signed genesets
author: "Jitao David Zhang and Gregor Sturm"
date: "`r Sys.Date()`"
vignette: >
 %\VignetteIndexEntry{Using BioQC with signed genesets}
 %\VignetteEngine{knitr::rmarkdown}
 %\usepackage[utf8]{inputenc}
output: 
  rmarkdown::html_vignette:
    self_contained: no
  md_document:
    variant: markdown_phpextra
    preserve_yaml: TRUE 
---

```{r setup, include=FALSE}
library(knitr)
opts_chunk$set(out.extra='style="display:block; margin: auto"', fig.align="center")
```

Introduction
------------

Besides being used as a QC-tool for gene expression data, _BioQC_ can be used for general-purpose gene-set enrichment analysis. Its core function, _wmwTest_, can handle not only "normal", unsigned gene sets, but also signed genesets where two sets of genes represent signatures that are positively and negatively regulated respectively. 

This vignette describes an example of such applications. First we load the BioQC library.

```{r lib, warning=FALSE, message=FALSE, results="hide"}
library(BioQC)
```

GMT read-in
----------
We first open a toy GMT file containing signed genesets with _readSignedGmt_. The function reads the file into a _SignedGenesets_ object.

```{r gmt}
gmtFile <- system.file("extdata/test.gmt", package="BioQC")
## print the file context
cat(readLines(gmtFile), sep="\n")
## read the file into SignedGenesets
genesets <- readSignedGmt(gmtFile)
print(genesets)
```

Note that though the GMT file contains 6 non-empty lines, only five signed genesets are returned by _readSignedGmt_ because a pair of negative and negative geneset of GS_C is available.

Note that in its current form, no genes are reported for GS_A and GS_B, because the names of both sets do not match the patterns. User can decide how to handle such genesets that match patterns of neither positive nor negative sets (GS_A and GS_B in this case). By default, such genesets are ignored; however, user can decide to treat them as either positive or negative genesets. For the purpose of this tutorial, we treat these genesets as positive.

```{r gmtPos}
genesets <- readSignedGmt(gmtFile, nomatch="pos")
print(genesets)
```

Expression object construction
------------------------------
Next we construct an ExpressionSet object containing 100 genes, with some of the genes in the GMT file present. 

```{r data}
set.seed(1887)
testN <- 100L
testSampleCount <- 3L
testGenes <- c("AKT1", "AKT2", "ERBB2", "ERBB3", "EGFR","TSC1", "TSC2", "GATA2", "GATA4", "GATA1", "GATA3")
testRows <- c(testGenes, paste("Gene", (length(testGenes)+1):testN, sep=""))
testMatrix <- matrix(rnorm(testN*testSampleCount, sd=0.1), nrow=testN, dimnames=list(testRows, NULL))
testMatrix[1:2,] <- testMatrix[1:2,]+10
testMatrix[6:7,] <- testMatrix[6:7,]-10
testMatrix[3:4,] <- testMatrix[3:4,]-5
testMatrix[5,] <- testMatrix[5,]+5
testEset <- new("ExpressionSet", exprs=testMatrix)
```
The expression of somes genes are delibrately changed so that some genesets are expected to be significantly enriched.

* AKT1 and AKT2 are higher expressed than the background genes ('background' hereafter). 
* TSC1 and TSC2 are lower expressed than the background.
* ERBB2 and ERBB3 are moderately lower expressed than the background.
* EGFR is moderately higher expressed than the background.

Considering the geneset composition printed above, we can expect that
* GS_A shall be significantly up-regulated; 
* GS_B shall not be significant, because its genes are missing;
* GS_C shall be significantly down-regulated, with positive targets down-regulated and negative targets up-regulated;
* GS_D shall not always be significant, because its genes are comparable to the background;
* GS_E shall be significantly up-regulated, because its negative targets are down-regulated.


Match genes
-----------

With both _SignedGenesets_ and _ExpressionSet_ objects at hand, we can query the indices of genes of genesets in the _ExpressionSet_ object.

```{r index}
testIndex <- matchGenes(genesets, testEset, col=NULL)
print(testIndex)
```

If the _ExpressionSet_ object has GeneSymbols in its featureData other than the feature names, user can specify the parameter _col_ in _matchGenes_ to make the match working. _matchGene_ also works with characters and matrix as input, please check out the help pages for such uses.

Perform the analysis
--------------------

```{r runWmwGreater}
wmwResult.greater <- wmwTest(testEset, testIndex, valType="p.greater")
print(wmwResult.greater)
```

As expected, GS\_A and GS\_E are significant when the alternative hypothesis is _greater_.

```{r runWmwLess}
wmwResult.less <- wmwTest(testEset, testIndex, valType="p.less")
print(wmwResult.less)
```
As expected, GS\_C is significant when the alternative hypothesis is _less_.

```{r runWmwTwoSided}
wmwResult.two.sided <- wmwTest(testEset, testIndex, valType="p.two.sided")
print(wmwResult.two.sided)
```

As expected, GS\_A, GS\_C, and GS\_E are significant when the alternative hypothesis is _two.sided_. 

A simple way to integrate the results of both alternative hypotheses into one numerical value is the _Q_ value, which is defined as absolute log10 transformation of the smaller p value of the two hypotheses, times the sign defined by _greater_=1 and _less_=-1.

```{r runWmwQ}
wmwResult.Q <- wmwTest(testEset, testIndex, valType="Q")
print(wmwResult.Q)
```

As expected, genesets GS\_A, GS\_C, and GS\_E have large absolute values, suggesting they are potentially enriched; while GS\_A and GS\_E are positively enriched, GS\_C is negatively enriched.

Conclusions
-----------
In summary, _BioQC_ can be used to perform gene-set enrichment analysis with signed genesets. Its speed is comparable to that of unsigned genesets.

Acknowledgement
----------------
I would like to thank the authors of the gCMAP package with the idea of integrating signed genesets into the framework of Wilcoxon-Mann-Whitney test.

R session info
--------------
```{r session}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GmtList-funcs.R
\name{gsNamespace}
\alias{gsNamespace}
\title{Gene-set namespaces}
\usage{
gsNamespace(x)
}
\arguments{
\item{x}{A \code{GmtList} object}
}
\value{
Namespaces as a vector of character strings of the same length as \code{x}
}
\description{
Gene-set namespaces
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllMethods.R, R/matchGenes.R
\name{matchGenes}
\alias{matchGenes}
\alias{matchGenes,GmtList,character-method}
\alias{matchGenes,GmtList,matrix-method}
\alias{matchGenes,GmtList,eSet-method}
\alias{matchGenes,character,character-method}
\alias{matchGenes,character,matrix-method}
\alias{matchGenes,character,eSet-method}
\alias{matchGenes,character,DGEList-method}
\alias{matchGenes,GmtList,DGEList-method}
\alias{matchGenes,SignedGenesets,character-method}
\alias{matchGenes,SignedGenesets,matrix-method}
\alias{matchGenes,SignedGenesets,eSet-method}
\alias{matchGenes,SignedGenesets,DGEList-method}
\title{Match genes in a list-like object to a vector of genesymbols}
\usage{
matchGenes(list, object, ...)

\S4method{matchGenes}{GmtList,character}(list, object)

\S4method{matchGenes}{GmtList,matrix}(list, object)

\S4method{matchGenes}{GmtList,eSet}(list, object, col = "GeneSymbol")

\S4method{matchGenes}{character,character}(list, object)

\S4method{matchGenes}{character,matrix}(list, object)

\S4method{matchGenes}{character,eSet}(list, object)

\S4method{matchGenes}{character,DGEList}(list, object, col = "GeneSymbol")

\S4method{matchGenes}{GmtList,DGEList}(list, object, col = "GeneSymbol")

\S4method{matchGenes}{SignedGenesets,character}(list, object)

\S4method{matchGenes}{SignedGenesets,matrix}(list, object)

\S4method{matchGenes}{SignedGenesets,eSet}(list, object, col = "GeneSymbol")

\S4method{matchGenes}{SignedGenesets,DGEList}(list, object, col = "GeneSymbol")
}
\arguments{
\item{list}{A GmtList, list, character or SignedGenesets object}

\item{object}{Gene symbols to be matched; they can come from a vector of character strings, or
a column in the fData of an \code{eSet} object.}

\item{...}{additional arguments like \code{col}}

\item{col}{Column name of \code{fData} in an \code{eSet} object, or \code{genes} in an \code{DGEList} object, to specify where gene symbols are stored.
The default value is set to "GeneSymbol"}
}
\value{
An \code{IndexList} object, which is essentially a list of the same length as input (length of \code{1} in case characters are used as input), with matching indices.
}
\description{
Match genes in a list-like object to a vector of genesymbols
}
\examples{
## test GmtList, character
testGenes <- sprintf("gene\%d", 1:10)
testGeneSets <- GmtList(list(gs1=c("gene1", "gene2"), gs2=c("gene9", "gene10"), gs3=c("gene100")))
matchGenes(testGeneSets, testGenes)

## test GmtList, matrix
testGenes <- sprintf("gene\%d", 1:10)
testGeneSets <- GmtList(list(gs1=c("gene1", "gene2"), gs2=c("gene9", "gene10"), gs3=c("gene100")))
testGeneExprs <- matrix(rnorm(100), nrow=10, dimnames=list(testGenes, sprintf("sample\%d", 1:10)))
matchGenes(testGeneSets, testGeneExprs)

## test GmtList, eSet
testGenes <- sprintf("gene\%d", 1:10)
testGeneSets <- GmtList(list(gs1=c("gene1", "gene2"), gs2=c("gene9", "gene10"), gs3=c("gene100")))
testGeneExprs <- matrix(rnorm(100), nrow=10, dimnames=list(testGenes, sprintf("sample\%d", 1:10)))
testFeat <- data.frame(GeneSymbol=rownames(testGeneExprs), row.names=testGenes)
testPheno <- data.frame(SampleId=colnames(testGeneExprs), row.names=colnames(testGeneExprs))
testEset <- ExpressionSet(assayData=testGeneExprs,
    featureData=AnnotatedDataFrame(testFeat),
    phenoData=AnnotatedDataFrame(testPheno))
matchGenes(testGeneSets, testGeneExprs)
## force using row names
matchGenes(testGeneSets, testEset, col=NULL)

 ## test GmtList, DGEList
 if(requireNamespace("edgeR")) {
    mat <- matrix(rnbinom(100, mu=5, size=2), ncol=10)
    rownames(mat) <- sprintf("gene\%d", 1:nrow(mat))
    y <- edgeR::DGEList(counts=mat, group=rep(1:2, each=5))

    ## if genes are not set, row names of the count matrix will be used for lookup
    myGeneSet <- GmtList(list(gs1=rownames(mat)[1:2], gs2=rownames(mat)[9:10], gs3="gene100"))
    matchGenes(myGeneSet, y)

    matchGenes(c("gene1", "gene2"), y)
    ## alternatively, use 'col' parameter to specify the column in 'genes'
    y2 <- edgeR::DGEList(counts=mat,
      group=rep(1:2, each=5),
      genes=data.frame(GeneIdentifier=rownames(mat), row.names=rownames(mat)))
    matchGenes(myGeneSet, y2, col="GeneIdentifier")
 }

## test character, character
matchGenes(c("gene1", "gene2"), testGenes)

## test character, matrix
matchGenes(c("gene1", "gene2"), testGeneExprs)

## test character, eset
matchGenes(c("gene1", "gene2"), testEset)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/uniqGenesetsByNamespace.R
\name{uniqGenesetsByNamespace}
\alias{uniqGenesetsByNamespace}
\title{Make names of gene-sets unique by namespace, and member genes of gene-sets unique}
\usage{
uniqGenesetsByNamespace(gmtList)
}
\arguments{
\item{gmtList}{A \code{GmtList} object, probably from \code{\link[BioQC]{readGmt}}. The object must have namespaces defined by \code{setNamespace}.

The function make sure that
\itemize{
  \item names of gene-sets within each namespace are unique, by merging gene-sets with duplicated names
  \item genes within each gene-set are unique, by removing duplicated genes
}

Gene-sets with duplicated names and different \code{desc} are merged, \code{desc} are made unique, and in case of multiple values, concatenated (with \code{|} as the collapse character).}
}
\value{
A \code{GmtList} object, with unique gene-sets and unique gene lists. If not already present, a new item \code{namespace} is appended to each \code{list} element in the \code{GmtList} object, recording the namespace used to make gene-sets unique. The order of the returned \code{GmtList} object is given by the unique gene-set name of the input object.
}
\description{
Make names of gene-sets unique by namespace, and member genes of gene-sets unique
}
\examples{
myGmtList <- GmtList(list(list(name="GeneSet1", desc="Namespace1", genes=LETTERS[1:3]),
  list(name="GeneSet2", desc="Namespace1", genes=rep(LETTERS[4:6],2)),
  list(name="GeneSet1", desc="Namespace1", genes=LETTERS[4:6]),
  list(name="GeneSet3", desc="Namespace2", genes=LETTERS[1:5])))
 
print(myGmtList)
myGmtList <- setNamespace(myGmtList, namespace=function(x) x$desc)
myUniqGmtList <- uniqGenesetsByNamespace(myGmtList)
print(myUniqGmtList)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readSignedGmt.R
\name{gmtlist2signedGenesets}
\alias{gmtlist2signedGenesets}
\title{Convert gmtlist into a list of signed genesets}
\usage{
gmtlist2signedGenesets(
  gmtlist,
  posPattern = "_UP$",
  negPattern = "_DN$",
  nomatch = c("ignore", "pos", "neg")
)
}
\arguments{
\item{gmtlist}{A gmtlist object, probably read-in by \code{readGmt}}

\item{posPattern}{Regular expression pattern of positive gene sets. It is trimmed from the original name to get the stem name of the gene set. See examples below.}

\item{negPattern}{Regular expression pattern of negative gene sets. It is trimmed from the original name to get the stem name of the gene set. See examples below.}

\item{nomatch}{Options to deal with gene sets that match neither positive nor negative patterns. ignore: they will be ignored (but not discarded, see details below); pos: they will be counted as positive signs; neg: they will be counted as negative signs}
}
\value{
An S4-object of \code{SignedGenesets}, which is a list of signed_geneset, each being a two-item list; the first item is 'pos', containing a character vector of positive genes; and the second item is 'neg', containing a character vector of negative genes.

Gene set names are detected whether they are positive or negative. If neither positive nor negative, nomatch will determine how will they be interpreted. In case of \code{pos} (or \code{neg}), such genesets will be treated as positive (or negative) gene sets.In case nomatch is set to \code{ignore}, the gene set will appear in the returned values with both positive and negative sets set to \code{NULL}.
}
\description{
Convert gmtlist into a list of signed genesets
}
\examples{
testInputList <- list(list(name="GeneSetA_UP",genes=LETTERS[1:3]),
               list(name="GeneSetA_DN", genes=LETTERS[4:6]),
               list(name="GeneSetB", genes=LETTERS[2:4]),
               list(name="GeneSetC_DN", genes=LETTERS[1:3]),
               list(name="GeneSetD_UP", genes=LETTERS[1:3]))
testOutputList.ignore <- gmtlist2signedGenesets(testInputList, nomatch="ignore")
testOutputList.pos <- gmtlist2signedGenesets(testInputList, nomatch="pos")
testOutputList.neg <- gmtlist2signedGenesets(testInputList, nomatch="neg")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllMethods.R
\name{SignedIndexList}
\alias{SignedIndexList}
\alias{SignedIndexList,list-method}
\title{Convert a list into a SignedIndexList}
\usage{
SignedIndexList(object, ...)

\S4method{SignedIndexList}{list}(object, keepNA = FALSE, keepDup = FALSE, offset = 1L)
}
\arguments{
\item{object}{A list of lists, each with two elements named `pos` or `neg`, can be logical vectors or integer indices}

\item{...}{additional arguments, currently ignored}

\item{keepNA}{Logical, whether NA indices should be kept or not. Default: 
FALSE (removed)}

\item{keepDup}{Logical, whether duplicated indices should be kept or not. 
Default: FALSE (removed)}

\item{offset}{offset; 1 if missing}
}
\value{
A SignedIndexList, a list of lists, containing two vectors named `positive` and `negative`, 
which contain the indices of genes that are either positively or negatively associated with a certain
phenotype
}
\description{
Convert a list into a SignedIndexList
}
\examples{
myList <- list(a = list(pos = list(1, 2, 2, 4), neg = c(TRUE, FALSE, TRUE)), 
b = list(NA), c = list(pos = c(c(2, 3), c(1, 3))))
SignedIndexList(myList)

## a special case of input is a single list with two elements, \code{pos} and \code{neg}
SignedIndexList(myList[[1]])
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prettySigNames.R
\name{prettySigNames}
\alias{prettySigNames}
\title{Prettify default signature names}
\usage{
prettySigNames(names, includeNamespace = TRUE)
}
\arguments{
\item{names}{Character strings, signature names}

\item{includeNamespace}{Logical, whether the namespace of the signatures should be included}
}
\value{
Character strings, pretty signature names
}
\description{
Prettify default signature names
}
\examples{
sig <- readCurrentSignatures()
prettyNames <- prettySigNames(names(sig))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllMethods.R
\name{show,SignedGenesets-method}
\alias{show,SignedGenesets-method}
\title{Show method for SignedGenesets}
\usage{
\S4method{show}{SignedGenesets}(object)
}
\arguments{
\item{object}{An object of the class \code{SignedGenesets}}
}
\description{
Show method for SignedGenesets
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllClasses.R
\name{isValidSignedIndexList}
\alias{isValidSignedIndexList}
\title{Function to validate a SignedIndexList object}
\usage{
isValidSignedIndexList(object)
}
\arguments{
\item{object}{a SignedIndexList object
Use \code{setValidity("SignedIndexList", "isValidSignedIndexList")} to check integrity of SignedIndexList objects.
It can be very slow, therefore the feature is not turned on by default}
}
\description{
Function to validate a SignedIndexList object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wmwTest.R
\name{wmwTestInR}
\alias{wmwTestInR}
\title{Wilcoxon-Mann-Whitney test in R}
\usage{
wmwTestInR(x, sub, valType = c("p.greater", "p.less", "p.two.sided", "W"))
}
\arguments{
\item{x}{A numerical vector}

\item{sub}{A logical vector or integer vector to subset \code{x}. Numbers in \code{sub} are compared with numbers out of \code{sub}}

\item{valType}{Type of retured-value. Supported values: p.greater, p.less, p.two.sided, and W statistic (note it is different from the U statistic)}
}
\description{
Wilcoxon-Mann-Whitney test in R
}
\examples{
testNums <- 1:10
testSub <- rep_len(c(TRUE, FALSE), length.out=length(testNums))
wmwTestInR(testNums, testSub)
wmwTestInR(testNums, testSub, valType="p.two.sided")
wmwTestInR(testNums, testSub, valType="p.less")
wmwTestInR(testNums, testSub, valType="W")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/leadingEdge.R
\name{wmwLeadingEdge}
\alias{wmwLeadingEdge}
\title{Identify BioQC leading-edge genes of one gene-set}
\usage{
wmwLeadingEdge(
  matrix,
  indexVector,
  valType = c("p.greater", "p.less", "p.two.sided", "U", "abs.log10p.greater",
    "log10p.less", "abs.log10p.two.sided", "Q", "r", "f", "U1", "U2"),
  thr = 0.05,
  reference = c("background", "geneset")
)
}
\arguments{
\item{matrix}{A numeric matrix}

\item{indexVector}{An integer vector, giving indices of a gene-set of interest}

\item{valType}{Value type, consistent with the types in \code{wmwTest}}

\item{thr}{Threshold of the value, greater or less than which the gene-set is considered significantly enriched in one sample}

\item{reference}{Character string, which reference is used? If \code{background}, genes with expression higher than the median of the background are reported. Otherwise in the case of \code{geneset}, genes with expression higher than the median of the gene-set is reported. Default is \code{background}, which is consistent with the results of the Wilcoxon-Mann-Whitney tests.}
}
\value{
A list of integer vectors.

BioQC leading-edge genes are defined as those features whose expression is higher than the median expression of the background in a sample.  The function identifies leading-edge genes of a given dataset (specified by the index vector) in a number of samples (specified by the matrix, with genes/features in rows and samples in columns) in three steps. The function calls \code{wmwTest} to run BioQC and identify samples in which the gene-set is significantly enriched. The enrichment criteria is specified by \code{valType} and \code{thr}. Then the function identifies genes in the gene-set that have greater or less expresion than the median value of the \code{reference} in those samples showing significant enrichment. Finally, it reports either leading-edge genes in individual samples, or the intersection/union of leading-edge genes in multiple samples.
}
\description{
Identify BioQC leading-edge genes of one gene-set
}
\examples{
myProfile <- c(rnorm(5, 3), rnorm(15, -3), rnorm(100, 0))
myProfile2 <- c(rnorm(15, 3), rnorm(5, -3), rnorm(100, 0))
myProfile3 <- c(rnorm(10, 5), rnorm(10, 0), rnorm(100, 0))
myProfileMat <- cbind(myProfile, myProfile2, myProfile3)
wmwLeadingEdge(myProfileMat, 1:20, valType="p.greater")
wmwLeadingEdge(myProfileMat, 1:20, valType="log10p.less")
wmwLeadingEdge(myProfileMat, 1:20, valType="U", reference="geneset")
wmwLeadingEdge(myProfileMat, 1:20, valType="abs.log10p.greater")
}
\seealso{
\code{\link{wmwTest}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllClasses.R
\name{isValidGmtList}
\alias{isValidGmtList}
\title{Function to validate a GmtList object}
\usage{
isValidGmtList(object)
}
\arguments{
\item{object}{A GmtList object
Use \code{setValidity("GmtList", "isValidGmtList")} to check integrity of GmtList objects.
It can be very slow, therefore the feature is not turned on by default}
}
\description{
Function to validate a GmtList object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GmtList-funcs.R
\name{gsNamespace<-}
\alias{gsNamespace<-}
\title{gsNamespace<- is the synonym of setGsNamespace}
\usage{
gsNamespace(x) <- value
}
\arguments{
\item{x}{A \code{GmtList} object}

\item{value}{\code{namespace} in \code{setGsNamespace}. It can be either a function that applies to a \code{gene-set list} element of the object (for instance \code{function(x) x$desc} to extract description), or a vector of the same length of \code{x}, or in the special case \code{NULL}, which will erase the field namespace.}
}
\description{
gsNamespace<- is the synonym of setGsNamespace
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllMethods.R
\name{offset<-}
\alias{offset<-}
\alias{offset-set}
\alias{offset<-,IndexList,numeric-method}
\alias{offset<-,SignedIndexList,numeric-method}
\title{Set the offset of an \code{IndexList} or a \code{SignedIndexList} object}
\usage{
`offset<-`(object, value)

\S4method{offset}{IndexList,numeric}(object) <- value

\S4method{offset}{SignedIndexList,numeric}(object) <- value
}
\arguments{
\item{object}{An \code{IndexList} or a \code{SignedIndexList} object}

\item{value}{The value, that the offset of \code{object} is set too. If it 
isn't an integer, it's coerced into an integer.}
}
\description{
Set the offset of an \code{IndexList} or a \code{SignedIndexList} object
}
\examples{
myIndexList <- IndexList(list(1:5, 2:7, 3:8), offset=1L)
offset(myIndexList)
offset(myIndexList) <- 3
offset(myIndexList)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllMethods.R
\name{[.GmtList}
\alias{[.GmtList}
\title{Subsetting GmtList object into another GmtList object}
\usage{
\method{[}{GmtList}(x, i, drop = FALSE)
}
\arguments{
\item{x}{A GmtList object}

\item{i}{Index to subset}

\item{drop}{In case only one element remains, should a list representing the single geneset returned? Default: FALSE}
}
\description{
Subsetting GmtList object into another GmtList object
}
\examples{
myGmtList <- GmtList(list(gs1=letters[1:3], gs2=letters[3:4], gs3=letters[4:5]))
myGmtList[1:2]
myGmtList[1] ## default behaviour: not dropping
myGmtList[1,drop=TRUE] ## force dropping
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllClasses.R
\name{SignedGenesets}
\alias{SignedGenesets}
\title{Convert a list to a SignedGenesets object}
\usage{
SignedGenesets(list)
}
\arguments{
\item{list}{A list of genesets; each geneset is a list of at least three fields: 'name', 'pos', and 'neg'. 'name' contains one non-null character string, and both 'pos' and 'neg' can be either NULL or a character vector.}
}
\description{
Convert a list to a SignedGenesets object
}
\examples{
testList <- list(list(name="GS_A", pos=NULL, neg=LETTERS[1:3]),
                 list(name="GS_B", pos=LETTERS[1:5], neg=LETTERS[7:9]),
                 list(name="GS_C", pos=LETTERS[1:5], neg=NULL),
                 list(name="GS_D", pos=NULL, neg=NULL))
testSigndGS <- SignedGenesets(testList)

}
\seealso{
\code{GmtList}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllClasses.R
\docType{class}
\name{IndexList-class}
\alias{IndexList-class}
\title{An S4 class to hold a list of integers as indices, with the possibility to specify the offset of the indices}
\description{
An S4 class to hold a list of integers as indices, with the possibility to specify the offset of the indices
}
\section{Slots}{

\describe{
\item{\code{offset}}{An integer specifying the value of first element. Default 1}

\item{\code{keepNA}}{Logical, whether NA is kept during construction}

\item{\code{keepDup}}{Logical, whether duplicated values are kept during construction}
}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readCurrentSignatures.R
\name{readCurrentSignatures}
\alias{readCurrentSignatures}
\title{Load current BioQC signatures}
\usage{
readCurrentSignatures(uniqGenes = TRUE, namespace = NULL)
}
\arguments{
\item{uniqGenes}{Logical, whether duplicated genes should be removed, passed
to \code{\link{readGmt}}.}

\item{namespace}{Character, namespace of the gene-set, or code{NULL}, passed
to \code{\link{readGmt}}}
}
\value{
A GmtList
}
\description{
Load current BioQC signatures
}
\examples{
readCurrentSignatures()
}
\seealso{
\code{\link{readGmt}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{simplifyMatrix}
\alias{simplifyMatrix}
\title{Simplify matrix in case of single row/columns}
\usage{
simplifyMatrix(matrix)
}
\arguments{
\item{matrix}{A matrix of any dimension

If only one row/column is present, the dimension is dropped and a vector will be returned}
}
\description{
Simplify matrix in case of single row/columns
}
\examples{
testMatrix <- matrix(round(rnorm(9),2), nrow=3)
simplifyMatrix(testMatrix)
simplifyMatrix(testMatrix[1L,,drop=FALSE])
simplifyMatrix(testMatrix[,1L,drop=FALSE])
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllMethods.R
\name{[[.GmtList}
\alias{[[.GmtList}
\title{Subsetting GmtList object to fetch one gene-set}
\usage{
\method{[[}{GmtList}(x, i)
}
\arguments{
\item{x}{A GmtList object}

\item{i}{The index to subset}
}
\description{
Subsetting GmtList object to fetch one gene-set
}
\examples{
myGmtList <- GmtList(list(gs1=letters[1:3], gs2=letters[3:4], gs3=letters[4:5]))
myGmtList[[1]]
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllClasses.R
\name{isValidBaseIndexList}
\alias{isValidBaseIndexList}
\title{Function to validate a BaseIndexList  object}
\usage{
isValidBaseIndexList(object)
}
\arguments{
\item{object}{A BaseIndexList  object
Use \code{setValidity("BaseIndexList", "isValidBaseIndexList")} to check integrity of BaseIndexList  objects.
It can be very slow, therefore the feature is not turned on by default}
}
\description{
Function to validate a BaseIndexList  object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gini.R
\name{gini}
\alias{gini}
\title{Calculate Gini Index of a numeric vector}
\usage{
gini(x)
}
\arguments{
\item{x}{A numeric vector.}
}
\value{
A numeric value between 0 and 1.
}
\description{
Calculate the Gini index of a numeric vector.
}
\details{
The Gini index (Gini coefficient) is a measure of statistical dispersion. A Gini
coefficient of zero expresses perfect equality where all values are
the same. A Gini coefficient of one expresses maximal inequality among values.
}
\examples{
testValues <- runif(100)
gini(testValues)
}
\references{
Gini. C. (1912) \emph{Variability and Mutability}, C. Cuppini, Bologna
156 pages.
}
\author{
Jitao David Zhang <jitao_david.zhang@roche.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllMethods.R
\name{offset}
\alias{offset}
\alias{offset,BaseIndexList-method}
\title{Get offset from an IndexList object}
\usage{
offset(object)

\S4method{offset}{BaseIndexList}(object)
}
\arguments{
\item{object}{An IndexList object}
}
\description{
Get offset from an IndexList object
}
\examples{
myIndexList <- IndexList(list(1:5, 2:7, 3:8), offset=1L)
offset(myIndexList)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/entropy.R
\name{entropyDiversity}
\alias{entropyDiversity}
\title{Entropy-based sample diversity}
\usage{
entropyDiversity(mat, norm = FALSE)
}
\arguments{
\item{mat}{A matrix (usually an expression matrix), with genes (features) in rows and samples in columns.}

\item{norm}{Logical, whether the diversity should be normalized by \code{log2(nrow(mat))}.}
}
\value{
A vector as long as the column number of the input matrix
}
\description{
Entropy-based sample diversity
}
\examples{
myMat <- rbind(c(3,4,5),c(6,6,6), c(0,2,4))
entropyDiversity(myMat)
entropyDiversity(myMat, norm=TRUE)

myRandomMat <- matrix(runif(1000), ncol=20)
entropyDiversity(myRandomMat)
entropyDiversity(myRandomMat, norm=TRUE)
}
\references{
Martinez and Reyes-Valdes (2008) Defining diversity, 
specialization, and gene specificity in transcriptomes through information 
theory. PNAS 105(28):9709--9714
}
\seealso{
\code{\link{entropy}} and \code{\link{sampleSpecialization}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GmtList-funcs.R
\name{setDescAsNamespace}
\alias{setDescAsNamespace}
\title{Set gene-set description as namespace}
\usage{
setDescAsNamespace(x)
}
\arguments{
\item{x}{A \code{GmtList} object

This function wrapps \code{setNamespace} to set gene-set description as namespace}
}
\description{
Set gene-set description as namespace
}
\seealso{
\code{\link{setNamespace}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GmtList-funcs.R
\name{setNamespace}
\alias{setNamespace}
\title{Set the namespace field in each gene-set within a GmtList}
\usage{
setNamespace(x, namespace)
}
\arguments{
\item{x}{A \code{GmtList} object encoding a list of gene-sets}

\item{namespace}{It can be either a function that applies to a \code{gene-set list} element of the object (for instance \code{function(x) x$desc} to extract description), or a vector of the same length of \code{x}, or in the special case \code{NULL}, which will erase the field namespace.

Note that using vectors as \code{namespace} leads to poor performance when the input object has many gene-sets.}
}
\description{
Set the namespace field in each gene-set within a GmtList
}
\examples{
myGmtList <- GmtList(list(list(name="GeneSet1", desc="Namespace1", genes=LETTERS[1:3]),
  list(name="GeneSet2", desc="Namespace1", genes=rep(LETTERS[4:6],2)),
  list(name="GeneSet1", desc="Namespace1", genes=LETTERS[4:6]),
  list(name="GeneSet3", desc="Namespace2", genes=LETTERS[1:5])))
hasNamespace(myGmtList)
myGmtList2 <- setNamespace(myGmtList, namespace=function(x) x$desc)
gsNamespace(myGmtList2)
## the function can provide flexible ways to encode the gene-set namespace
myGmtList3 <- setNamespace(myGmtList, namespace=function(x) gsub("Namespace", "C", x$desc))
gsNamespace(myGmtList3)
## using vectors
myGmtList4 <- setNamespace(myGmtList, namespace=c("C1", "C1", "C1", "C2"))
gsNamespace(myGmtList4)
myGmtList2null <- setNamespace(myGmtList2, namespace=NULL)
hasNamespace(myGmtList2null)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllClasses.R
\name{isValidIndexList}
\alias{isValidIndexList}
\title{Function to validate an IndexList object}
\usage{
isValidIndexList(object)
}
\arguments{
\item{object}{an IndexList object
Use \code{setValidity("BaseIndexList", "isValidBaseIndexList")} to check integrity of IndexList objects.
It can be very slow, therefore the feature is not turned on by default}
}
\description{
Function to validate an IndexList object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GmtList-funcs.R
\name{hasNamespace}
\alias{hasNamespace}
\title{Whether namespace is set}
\usage{
hasNamespace(x)
}
\arguments{
\item{x}{A \code{GmtList} object}
}
\value{
Logical, whether all gene-sets have the field \code{namespace} set
}
\description{
Whether namespace is set
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readGmt.R
\name{readGmt}
\alias{readGmt}
\title{Read in gene-sets from a GMT file}
\usage{
readGmt(..., uniqGenes = TRUE, namespace = NULL)
}
\arguments{
\item{...}{Named or unnamed characater string vector, giving file names of one or more GMT format files.}

\item{uniqGenes}{Logical, whether duplicated genes should be removed}

\item{namespace}{Character, namespace of the gene-set. It can be used to specify namespace or sources of the gene-sets. If  \code{NULL} is given, so no namespace is used and all gene-sets are assumed to come from the same unspecified namespace. The option can be helpful when gene-sets from multiple namespaces are jointly used.}
}
\value{
A \code{GmtList} object, which is a S4-class wrapper of a list. Each 
element in the object is a list of (at least) three items: 
\itemize{
  \item gene-set name (field \code{name}), character string, accessible with \code{\link{gsName}}
  \item gene-set description (field \code{desc}), character string, accessible with \code{\link{gsDesc}}
  \item genes (field \code{genes}), a vector of character strings, , accessible with \code{\link{gsGenes}}
  \item namespace (field \code{namespace}), accessible with \code{\link{gsNamespace}}
}
}
\description{
Read in gene-sets from a GMT file
}
\note{
Currently, when \code{namespace} is set as \code{NULL}, no namespace is used. This may change in the future, since we may use file base name as the default namespace.
}
\examples{
gmt_file <- system.file("extdata/exp.tissuemark.affy.roche.symbols.gmt", package="BioQC")
gmt_list <- readGmt(gmt_file)
gmt_nonUniqGenes_list <- readGmt(gmt_file, uniqGenes=FALSE)
gmt_namespace_list <- readGmt(gmt_file, uniqGenes=FALSE, namespace="myNamespace")

## suppose we have two lists of gene-sets to read in
test_gmt_file <- system.file("extdata/test.gmt", package="BioQC")
gmt_twons_list <- readGmt(gmt_file, test_gmt_file, namespace=c("BioQC", "test"))
## alternatively
gmt_twons_list <- readGmt(BioQC=gmt_file, test=test_gmt_file)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/appendGmtList.R
\name{appendGmtList}
\alias{appendGmtList}
\title{Append a GmtList object to another one}
\usage{
appendGmtList(gmtList, newGmtList, ...)
}
\arguments{
\item{gmtList}{A \code{GmtList} object}

\item{newGmtList}{Another \code{GmtList} object to be appended}

\item{...}{Further \code{GmtList} object to be appended}
}
\value{
A new \code{GmtList} list, with all elements in the input appended in the given order
}
\description{
Append a GmtList object to another one
}
\examples{
test_gmt_file<- system.file("extdata/test.gmt", package="BioQC")
testGmtList1 <- readGmt(test_gmt_file, namespace="test1")
testGmtList2 <- readGmt(test_gmt_file, namespace="test2")
testGmtList3 <- readGmt(test_gmt_file, namespace="test3")
testGmtAppended <- appendGmtList(testGmtList1, testGmtList2, testGmtList3)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllMethods.R
\name{show,GmtList-method}
\alias{show,GmtList-method}
\title{Show method for GmtList}
\usage{
\S4method{show}{GmtList}(object)
}
\arguments{
\item{object}{An object of the class \code{GmtList}}
}
\description{
Show method for GmtList
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllClasses.R
\docType{class}
\name{GmtList-class}
\alias{GmtList-class}
\title{An S4 class to hold geneset in the GMT file in a list, each item in the list 
is in in turn a list containing following items: name, desc, and genes.}
\description{
An S4 class to hold geneset in the GMT file in a list, each item in the list 
is in in turn a list containing following items: name, desc, and genes.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllMethods.R, R/wmwTest.R
\name{wmwTest}
\alias{wmwTest}
\alias{wmwTest,matrix,IndexList-method}
\alias{wmwTest,numeric,IndexList-method}
\alias{wmwTest,matrix,GmtList-method}
\alias{wmwTest,eSet,GmtList-method}
\alias{wmwTest,eSet,numeric-method}
\alias{wmwTest,eSet,logical-method}
\alias{wmwTest,eSet,list-method}
\alias{wmwTest,ANY,numeric-method}
\alias{wmwTest,ANY,logical-method}
\alias{wmwTest,ANY,list-method}
\alias{wmwTest,matrix,SignedIndexList-method}
\alias{wmwTest,matrix,SignedGenesets-method}
\alias{wmwTest,numeric,SignedIndexList-method}
\alias{wmwTest,eSet,SignedIndexList-method}
\alias{wmwTest,eSet,SignedGenesets-method}
\title{Wilcoxon-Mann-Whitney rank sum test for high-throughput expression
profiling data}
\usage{
wmwTest(
  x,
  indexList,
  col = "GeneSymbol",
  valType = c("p.greater", "p.less", "p.two.sided", "U", "abs.log10p.greater",
    "log10p.less", "abs.log10p.two.sided", "Q", "r", "f", "U1", "U2"),
  simplify = TRUE
)

\S4method{wmwTest}{matrix,IndexList}(x, indexList, valType = "p.greater", simplify = TRUE)

\S4method{wmwTest}{numeric,IndexList}(x, indexList, valType = "p.greater", simplify = TRUE)

\S4method{wmwTest}{matrix,GmtList}(x, indexList, valType = "p.greater", simplify = TRUE)

\S4method{wmwTest}{eSet,GmtList}(
  x,
  indexList,
  col = "GeneSymbol",
  valType = "p.greater",
  simplify = TRUE
)

\S4method{wmwTest}{eSet,numeric}(
  x,
  indexList,
  col = "GeneSymbol",
  valType = "p.greater",
  simplify = TRUE
)

\S4method{wmwTest}{eSet,logical}(
  x,
  indexList,
  col = "GeneSymbol",
  valType = "p.greater",
  simplify = TRUE
)

\S4method{wmwTest}{eSet,list}(
  x,
  indexList,
  col = "GeneSymbol",
  valType = "p.greater",
  simplify = TRUE
)

\S4method{wmwTest}{ANY,numeric}(x, indexList, valType = "p.greater", simplify = TRUE)

\S4method{wmwTest}{ANY,logical}(x, indexList, valType = "p.greater", simplify = TRUE)

\S4method{wmwTest}{ANY,list}(x, indexList, valType = "p.greater", simplify = TRUE)

\S4method{wmwTest}{matrix,SignedIndexList}(x, indexList, valType, simplify = TRUE)

\S4method{wmwTest}{matrix,SignedGenesets}(x, indexList, valType, simplify = TRUE)

\S4method{wmwTest}{numeric,SignedIndexList}(x, indexList, valType, simplify = TRUE)

\S4method{wmwTest}{eSet,SignedIndexList}(x, indexList, valType, simplify = TRUE)

\S4method{wmwTest}{eSet,SignedGenesets}(
  x,
  indexList,
  col = "GeneSymbol",
  valType = c("p.greater", "p.less", "p.two.sided", "U", "abs.log10p.greater",
    "log10p.less", "abs.log10p.two.sided", "Q", "r", "f", "U1", "U2"),
  simplify = TRUE
)
}
\arguments{
\item{x}{A numeric matrix. All other data types (e.g. numeric vectors
or \code{ExpressionSet} objects) are coerced into matrix.}

\item{indexList}{A list of integer indices (starting from 1) indicating
signature genes. Can be of length zero. Other data types (e.g. a list
of numeric or logical vectors, or a numeric or logical vector) are
coerced into such a list. See \code{details} below for a special case
using GMT files.}

\item{col}{a string sometimes used with a \code{eSet}}

\item{valType}{The value type to be returned, allowed values
include \code{p.greater}, \code{p.less}, \code{abs.log10p.greater} and
\code{abs.log10p.less} (one-sided tests),\code{p.two.sided}, and \code{U}
statistic (or more specifically, either \code{U1} or \code{U2}), and a few
other variants. See details below.}

\item{simplify}{Logical. If not, the returning value is in matrix
format; if set to \code{TRUE}, the results are simplified into
vectors when possible (default).}
}
\value{
A numeric matrix or vector containing the statistic.
}
\description{
wmwTest is a highly efficient Wilcoxon-Mann-Whitney rank sum
test for high-dimensional data, such as gene expression profiling. For datasets with
more than 100 features (genes), the function can be more than 1,000
times faster than its R implementations (\code{wilcox.test} in 
\code{stats}, or \code{rankSumTestWithCorrelation} in \code{limma}).
}
\details{
The basic application of the function is to test the enrichment of
gene sets in expression profiling data or differentially expressed
data (the matrix with feature/gene in rows and samples in columns).

A special case is when \code{x} is an \code{eSet} object
(e.g. \code{ExpressionSet}), and \code{indexList} is a list returned
from \code{readGmt} function. In this case, the only requirement is
that one column named \code{GeneSymbol} in the \code{featureData}
contain gene symbols used in the GMT file. The same applies to signed Gmt files. See the example below.

Besides the conventional value types such as \sQuote{p.greater},
\sQuote{p.less}, \sQuote{p.two.sided} , and \sQuote{U} (the U-statistic),
\code{wmwTest} (from version 0.99-1) provides further value types:
\code{abs.log10p.greater} and \code{log10p.less} perform log10
transformation on respective \emph{p}-values and give the
transformed value a proper sign (positive for greater than, and
negative for less than); \code{abs.log10p.two.sided} transforms
two-sided \emph{p}-values to non-negative values; and \code{Q} score
reports absolute log10-transformation of \emph{p}-value of the
two-side variant, and gives a proper sign to it, depending on whether it is
rather greater than (positive) or  less than (negative).

From version 1.19.1, the rank-biserial correlation coefficient (\sQuote{r})
and the common language effect size (\sQuote{f}) are supported value types.

Before version 1.19.3, the \sQuote{U} statistic returned is in fact
\sQuote{U2}. From version 1.19.3, \sQuote{U1} is returned when \sQuote{U} is
used, and users can specify additional parameter values \sQuote{U1} and
\sQuote{U2}. The sum of \sQuote{U1} and \sQuote{U2} is the product of the
sizes of two vectors to be compared.
}
\section{Methods (by class)}{
\itemize{
\item \code{x = matrix,indexList = IndexList}: \code{x} is a \code{matrix} and \code{indexList} is a \code{IndexList}

\item \code{x = numeric,indexList = IndexList}: \code{x} is a \code{numeric} and \code{indexList} is a \code{IndexList}

\item \code{x = matrix,indexList = GmtList}: \code{x} is a \code{matrix} and \code{indexList} is a \code{GmtList}

\item \code{x = eSet,indexList = GmtList}: \code{x} is a \code{eSet} and \code{indexList} is a \code{GmtList}

\item \code{x = eSet,indexList = numeric}: \code{x} is a \code{eSet} and \code{indexList} is a \code{numeric}

\item \code{x = eSet,indexList = logical}: \code{x} is a \code{eSet} and \code{indexList} is a \code{logical}

\item \code{x = eSet,indexList = list}: \code{x} is a \code{eSet} and \code{indexList} is a \code{list}

\item \code{x = ANY,indexList = numeric}: \code{x} is \code{ANY} and \code{indexList} is a \code{numeric}

\item \code{x = ANY,indexList = logical}: \code{x} is \code{ANY} and \code{indexList} is a \code{logical}

\item \code{x = ANY,indexList = list}: \code{x} is \code{ANY} and \code{indexList} is a \code{list}

\item \code{x = matrix,indexList = SignedIndexList}: \code{x} is a \code{matrix} and \code{indexList} is a 
\code{SignedIndexList}

\item \code{x = matrix,indexList = SignedGenesets}: \code{x} is a \code{eSet} and \code{indexList} is a 
\code{SignedIndexList}

\item \code{x = numeric,indexList = SignedIndexList}: \code{x} is a \code{numeric} and \code{indexList} is a 
\code{SignedIndexList}

\item \code{x = eSet,indexList = SignedIndexList}: \code{x} is a \code{eSet} and \code{indexList} is a 
\code{SignedIndexList}

\item \code{x = eSet,indexList = SignedGenesets}: \code{x} is a \code{eSet} and \code{indexList} is a 
\code{SignedIndexList}
}}

\note{
The function has been optimized for expression profiling data. It
avoids repetitive ranking of data as done by native R implementations
and uses efficient C code to increase the performance and control
memory use. Simulation studies using expression profiles of 22000
genes in 2000 samples and 200 gene sets suggested that the C
implementation can be >1000 times faster than the R
implementation. And it is possible to further accelerate by
parallel calling the function with \code{mclapply} in the \code{multicore} package.
}
\examples{
## R-native data structures
set.seed(1887)
rd <- rnorm(1000)
rl <- sample(c(TRUE, FALSE), 1000, replace=TRUE)
wmwTest(rd, rl, valType="p.two.sided")
wmwTest(rd, which(rl), valType="p.two.sided")
rd1 <- rd + ifelse(rl, 0.5, 0)
wmwTest(rd1, rl, valType="p.greater")
wmwTest(rd1, rl, valType="U")
rd2 <- rd - ifelse(rl, 0.2, 0)
wmwTest(rd2, rl, valType="p.greater")
wmwTest(rd2, rl, valType="p.two.sided")
wmwTest(rd2, rl, valType="p.less")
wmwTest(rd2, rl, valType="r")
wmwTest(rd2, rl, valType="f")

## matrix forms
rmat <- matrix(c(rd, rd1, rd2), ncol=3, byrow=FALSE)
wmwTest(rmat, rl, valType="p.two.sided")
wmwTest(rmat, rl, valType="p.greater")

wmwTest(rmat, which(rl), valType="p.two.sided")
wmwTest(rmat, which(rl), valType="p.greater")

## other valTypes
wmwTest(rmat, which(rl), valType="U")
wmwTest(rmat, which(rl), valType="abs.log10p.greater")
wmwTest(rmat, which(rl), valType="log10p.less")
wmwTest(rmat, which(rl), valType="abs.log10p.two.sided")
wmwTest(rmat, which(rl), valType="Q")
wmwTest(rmat, which(rl), valType="r")
wmwTest(rmat, which(rl), valType="f")

## using ExpressionSet
data(sample.ExpressionSet)
testSet <- sample.ExpressionSet
fData(testSet)$GeneSymbol <- paste("GENE_",1:nrow(testSet), sep="")
mySig1 <- sample(c(TRUE, FALSE), nrow(testSet), prob=c(0.25, 0.75), replace=TRUE)
wmwTest(testSet, which(mySig1), valType="p.greater")

## using integer
exprs(testSet)[,1L] <- exprs(testSet)[,1L] + ifelse(mySig1, 50, 0)
wmwTest(testSet, which(mySig1), valType="p.greater")

## using lists
mySig2 <- sample(c(TRUE, FALSE), nrow(testSet), prob=c(0.6, 0.4), replace=TRUE)
wmwTest(testSet, list(first=mySig1, second=mySig2))
## using GMT file
gmt_file <- system.file("extdata/exp.tissuemark.affy.roche.symbols.gmt", package="BioQC")
gmt_list <- readGmt(gmt_file)

gss <- sample(unlist(sapply(gmt_list, function(x) x$genes)), 1000)
eset<-new("ExpressionSet",
         exprs=matrix(rnorm(10000), nrow=1000L),
         phenoData=new("AnnotatedDataFrame", data.frame(Sample=LETTERS[1:10])),
         featureData=new("AnnotatedDataFrame",data.frame(GeneSymbol=gss)))
esetWmwRes <- wmwTest(eset ,gmt_list, valType="p.greater")
summary(esetWmwRes)

## using signed GMT file
signed_gmt_file <- system.file("extdata/test.gmt", package="BioQC")
signed_gmt <- readSignedGmt(signed_gmt_file)
esetSignedWmwRes <- wmwTest(eset, signed_gmt, valType="p.greater")

esetMat <- exprs(eset); rownames(esetMat) <- fData(eset)$GeneSymbol
esetSignedWmwRes2 <- wmwTest(esetMat, signed_gmt, valType="p.greater")
}
\references{
Barry, W.T., Nobel, A.B., and Wright, F.A. (2008). A statistical framework for testing functional namespaces in microarray data. _Annals of Applied Statistics_ 2, 286-315.

Wu, D, and Smyth, GK (2012). Camera: a competitive gene set test
accounting for inter-gene correlation. _Nucleic Acids Research_ 40(17):e133

Zar, JH (1999). _Biostatistical Analysis 4th Edition_. Prentice-Hall International, Upper Saddle River, New Jersey.
}
\seealso{
code{wilcox.test} in the \code{stats} package, and \code{rankSumTestWithCorrelation} in
the \code{limma} package.
}
\author{
Jitao David Zhang <jitao_david.zhang@roche.com>, with critical inputs
from Jan Aettig and Iakov Davydov about U statistics.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/leadingEdge.R
\name{getLeadingEdgeIndexFromVector}
\alias{getLeadingEdgeIndexFromVector}
\alias{getLeadingEdgeIndexFromMatrix}
\title{Getting leading-edge indices from a vector}
\usage{
getLeadingEdgeIndexFromVector(
  x,
  index,
  comparison = c("greater", "less"),
  reference = c("background", "geneset")
)

getLeadingEdgeIndexFromMatrix(
  x,
  index,
  comparison = c("greater", "less"),
  reference = c("background", "geneset")
)
}
\arguments{
\item{x}{A numeric vector (\code{getLeadingEdgeIndexFromVector}) or a numeric matrix (\code{getLeadingEdgeIndexFromMatrix}).}

\item{index}{An integer vector, indicating the indices of genes in a gene-set.}

\item{comparison}{Character string, are values greater than or less than the reference value considered as leading-edge? This depends on the type of value requested by the user in \code{wmwTest}.}

\item{reference}{Character string, which reference is used? If \code{background}, genes with expression higher than the median of the background are reported. Otherwise in the case of \code{geneset}, genes with expression higher than the median of the gene-set is reported. Default is \code{background}, which is consistent with the results of the Wilcoxon-Mann-Whitney tests.}
}
\value{
An integer vector, indicating the indices of leading-edge genes.
}
\description{
Getting leading-edge indices from a vector
}
\section{Functions}{
\itemize{
\item \code{getLeadingEdgeIndexFromMatrix}: \code{x} is a \code{matrix}.
}}

\examples{
myProfile <- c(rnorm(5, 3), rnorm(15, -3), rnorm(100, 0))
getLeadingEdgeIndexFromVector(myProfile, 1:20)
getLeadingEdgeIndexFromVector(myProfile, 1:20, comparison="less")
getLeadingEdgeIndexFromVector(myProfile, 1:20, comparison="less", reference="geneset")
myProfile2 <- c(rnorm(15, 3), rnorm(5, -3), rnorm(100, 0))
myProfileMat <- cbind(myProfile, myProfile2)
getLeadingEdgeIndexFromMatrix(myProfileMat, 1:20)
getLeadingEdgeIndexFromMatrix(myProfileMat, 1:20, comparison="less")
getLeadingEdgeIndexFromMatrix(myProfileMat, 1:20, comparison="less", reference="geneset")
}
\seealso{
\code{\link{wmwTest}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{absLog10p}
\alias{absLog10p}
\title{Absolute base-10 logarithm of p-values}
\usage{
absLog10p(x)
}
\arguments{
\item{x}{Numeric vector or matrix

The function returns the absolute values of base-10 logarithm of p-values.}
}
\value{
Numeric vector or matrix.
}
\description{
Absolute base-10 logarithm of p-values
}
\details{
The logarithm transformation of p-values is commonly used to visualize
results from statistical tests. Although it may cause misunderstanding
and therefore its use is disapproved by some experts, it helps to
visualize and interpret results of statistical tests intuitively.

The function transforms p-values with base-10 logarithm, and returns its
absolute value. The choice of base 10 is driven by the simplicity of
interpreting the results.
}
\examples{
testp <- runif(1000, 0, 1)
testp.al <- absLog10p(testp)

print(head(testp))
print(head(testp.al))

}
\author{
Jitao David Zhang <jitao_david.zhang@roche.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllMethods.R
\name{show,SignedIndexList-method}
\alias{show,SignedIndexList-method}
\title{Show method for SignedIndexList}
\usage{
\S4method{show}{SignedIndexList}(object)
}
\arguments{
\item{object}{An object of the class \code{SignedIndexList}}
}
\description{
Show method for SignedIndexList
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{filterPmat}
\alias{filterPmat}
\title{Filter rows of p-value matrix under the significance threshold}
\usage{
filterPmat(x, threshold)
}
\arguments{
\item{x}{A matrix of p-values. It must be raw p-values and should not be transformed (e.g. logarithmic).}

\item{threshold}{A numeric value, the minimal p-value used to filter
rows. If missing, given the values of \code{NA}, \code{NULL} or
number \code{0}, no filtering will be done and the input matrix will
be returned.}
}
\value{
Matrix of p-values. If no line is left, a empty matrix of the same dimension as input will be returned.
}
\description{
Filter rows of p-value matrix under the significance threshold
}
\examples{
set.seed(1235)
testMatrix <- matrix(runif(100,0,1), nrow=10)

## filtering
(testMatrix.filter <- filterPmat(testMatrix, threshold=0.05))
## more strict filtering
(testMatrix.strictfilter <- filterPmat(testMatrix, threshold=0.01))
## no filtering
(testMatrix.nofilter <- filterPmat(testMatrix))
}
\author{
Jitao David Zhang <jitao_david.zhang@roche.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllClasses.R
\name{isValidSignedGenesets}
\alias{isValidSignedGenesets}
\title{Function to validate a SignedGenesets object}
\usage{
isValidSignedGenesets(object)
}
\arguments{
\item{object}{A SignedGenesets object
Use \code{setValidity("SignedGenesets", "isValidSignedGenesets")} to check integrity of SignedGenesets objects.
It can be very slow, therefore the feature is not turned on by default}
}
\description{
Function to validate a SignedGenesets object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllMethods.R
\name{IndexList}
\alias{IndexList}
\alias{IndexList,numeric-method}
\alias{IndexList,logical-method}
\alias{IndexList,list-method}
\title{Convert a list to an IndexList object}
\usage{
IndexList(object, ..., keepNA = FALSE, keepDup = FALSE, offset = 1L)

\S4method{IndexList}{numeric}(object, ..., keepNA = FALSE, keepDup = FALSE, offset = 1L)

\S4method{IndexList}{logical}(object, ..., keepNA = FALSE, keepDup = FALSE, offset = 1L)

\S4method{IndexList}{list}(object, keepNA = FALSE, keepDup = FALSE, offset = 1L)
}
\arguments{
\item{object}{Either a list of unique integer indices, NULL and logical 
vectors (of same lengths), or a numerical vector or a logical vector. NA is discarded.}

\item{...}{If \code{object} isn't a list, additional vectors can go here.}

\item{keepNA}{Logical, whether NA indices should be kept or not. Default: FALSE (removed)}

\item{keepDup}{Logical, whether duplicated indices should be kept or not. Default: FALSE (removed)}

\item{offset}{Integer, the starting index. Default: 1 (as in the convention of R)}
}
\value{
The function returns a list of vectors
}
\description{
Convert a list to an IndexList object
}
\examples{
testList <- list(GS_A=c(1,2,3,4,3),
                 GS_B=c(2,3,4,5),
                 GS_C=NULL,
                 GS_D=c(1,3,5,NA),
                 GS_E=c(2,4))
testIndexList <- IndexList(testList)
IndexList(c(FALSE, TRUE, TRUE), c(FALSE, FALSE, TRUE), c(TRUE, FALSE, FALSE), offset=0)
IndexList(list(A=1:3, B=4:5, C=7:9))
IndexList(list(A=1:3, B=4:5, C=7:9), offset=0)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wmwTest.R
\name{valTypes}
\alias{valTypes}
\title{prints the options of valTypes of wmwTest}
\usage{
valTypes()
}
\description{
prints the options of valTypes of wmwTest
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GmtList-funcs.R
\name{gsGeneCount}
\alias{gsGeneCount}
\alias{gsSize}
\title{Gene-set gene counts}
\usage{
gsGeneCount(x, uniqGenes = TRUE)

gsSize(x, uniqGenes = TRUE)
}
\arguments{
\item{x}{A \code{GmtList} or similar object}

\item{uniqGenes}{Logical, whether only unique genes are counted}
}
\value{
Gene counts (aka gene-set sizes) as a vector of integer of the same length as \code{x}
}
\description{
Gene-set gene counts

gsSize is the synonym of gsGeneCount
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.GmtList.R
\name{as.GmtList}
\alias{as.GmtList}
\title{Convert a list of gene symbols into a gmtlist}
\usage{
as.GmtList(list, description = NULL, uniqGenes = TRUE, namespace = NULL)
}
\arguments{
\item{list}{A named list with character vectors of genes. Names will become names of gene sets; character vectors will become genes}

\item{description}{Character, description of gene-sets. The value will be expanded to the same length of the list.}

\item{uniqGenes}{Logical, whether redundant genes should be made unique?}

\item{namespace}{Character or \code{NULL}, namespace of the gene-set}
}
\description{
Convert a list of gene symbols into a gmtlist
}
\examples{
testVec <- list(GeneSet1=c("AKT1", "AKT2"),
               GeneSet2=c("MAPK1", "MAPK3"),
               GeneSet3=NULL)
testVecGmtlist <- as.GmtList(testVec)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/entropy.R
\name{entropy}
\alias{entropy}
\title{Shannon entropy}
\usage{
entropy(vector)
}
\arguments{
\item{vector}{A vector of numbers, or characters. Discrete probability of each item is calculated and the Shannon entropy is returned.}
}
\value{
Shannon entropy

Shannon entropy can be used as measures of gene expression
specificity, as well as measures of tissue diversity and
specialization. See references below.

We use \code{2} as base for the entropy calculation, because in this
base the unit of entropy is \emph{bit}.
}
\description{
Shannon entropy
}
\examples{
myVec0 <- 1:9
entropy(myVec0) ## log2(9)
myVec1 <- rep(1, 9)
entropy(myVec1)

entropy(LETTERS)
entropy(rep(LETTERS, 5))
}
\references{
Martinez and Reyes-Valdes (2008) Defining diversity, 
specialization, and gene specificity in transcriptomes through information 
theory. PNAS 105(28):9709--9714
}
\author{
Jitao David Zhang <jitao_david.zhang@roche.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllClasses.R
\name{GmtList}
\alias{GmtList}
\title{Convert a list to a GmtList object}
\usage{
GmtList(list)
}
\arguments{
\item{list}{A list of genesets; each geneset is a list of at least three fields: 'name', 'desc', and 'genes'. 'name' and 'desc' contains one character string ('desc' can be NULL while 'name' cannot), and 'genes' can be either NULL or a character vector. In addition, 'namespace' is accepted to represent the namespace.

For convenience, the function also accepts a list of character vectors, each containing a geneset. In this case, the function works as a wrapper of \code{as.GmtList}}
}
\description{
Convert a list to a GmtList object
}
\examples{
testList <- list(list(name="GS_A", desc=NULL, genes=LETTERS[1:3]),
                 list(name="GS_B", desc="gene set B", genes=LETTERS[1:5]),
                 list(name="GS_C", desc="gene set C", genes=NULL))
testGmt <- GmtList(testList)

# as wrapper of as.GmtList
testGeneList <- list(GS_A=LETTERS[1:3], GS_B=LETTERS[1:5], GS_C=NULL)
testGeneGmt <- GmtList(testGeneList)

}
\seealso{
If a list of gene symbols need to be converted into a GmtList, use 'as.GmtList' instead
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GmtList-funcs.R
\name{filterBySize}
\alias{filterBySize}
\title{Filter a GmtList by size}
\usage{
filterBySize(x, min, max)
}
\arguments{
\item{x}{A \code{GmtList} object}

\item{min}{Numeric, gene-sets with fewer genes than \code{min} will be removed}

\item{max}{Numeric, gene-sets with more genes than \code{max} will be removed}
}
\value{
A \code{GmtList} object with sizes (count of genes) between \code{min} and \code{max} (inclusive).
}
\description{
Filter a GmtList by size
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllClasses.R
\docType{class}
\name{SignedGenesets-class}
\alias{SignedGenesets-class}
\title{An S4 class to hold signed genesets, each item in the list is in in turn a 
list containing following items: name, pos, and neg.}
\description{
An S4 class to hold signed genesets, each item in the list is in in turn a 
list containing following items: name, pos, and neg.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readSignedGmt.R
\name{readSignedGmt}
\alias{readSignedGmt}
\title{Read signed GMT files}
\usage{
readSignedGmt(
  filename,
  posPattern = "_UP$",
  negPattern = "_DN$",
  nomatch = c("ignore", "pos", "neg"),
  uniqGenes = TRUE,
  namespace = NULL
)
}
\arguments{
\item{filename}{A gmt file}

\item{posPattern}{Pattern of positive gene sets}

\item{negPattern}{Pattern of negative gene sets}

\item{nomatch}{options to deal with gene sets that match to neither posPattern nor negPattern patterns}

\item{uniqGenes}{Logical, whether genes should be made unique}

\item{namespace}{Character string or \code{NULL}, namespace of gene-sets}
}
\description{
Read signed GMT files
}
\examples{
testGmtFile <- system.file("extdata/test.gmt", package="BioQC")
testSignedGenesets.ignore <- readSignedGmt(testGmtFile, nomatch="ignore")
testSignedGenesets.pos <- readSignedGmt(testGmtFile, nomatch="pos")
testSignedGenesets.neg <- readSignedGmt(testGmtFile, nomatch="neg")
}
\seealso{
\code{\link{gmtlist2signedGenesets}} for parameters \code{posPattern}, \code{negPattern}, and \code{nomatch}
}
